TRANSFORMATION FactorK

 -- Convert any function resulting from translation into
 -- form, K (x).  Note that because translation always produces a 
 -- function that is invoked on NIL, and because this NIL is simply
 -- a place holder and not used within the query's processing,
 -- any such function can be so converted.

 -- The transformation depends on auxiliary transformations
 -- PullP2SHRF and PullP2SHRP which transform data functions
 -- and data predicates appearing in queries (p and f in iterate (p, f)
 -- and g in unnest (f, g)) into the form,
 --         (... ((f o #2) o SHR) ... o SHR)
 -- and
 --         (... ((p OPLUS #2) OPLUS SHR) ... OPLUS SHR)
 -- respectively.  Presently, these two transformations
 -- work on functions and predicates that do not involve
 -- collections (i.e., no nested queries).  However, 
 -- complex functions and predicates such as path expressions,
 -- structs, arbitrary conjunctions, disjunctions and negations
 -- of comparisons are supported.  Eventually, these transformations
 -- will need to be made more powerful.  I believe that the trick
 -- will be to merge transformation, SimpLits (used by PullP2SHRF and
 -- PullP2SHRP to rewrite functions (predicates) into the forms f o #2
 -- or K (x) (p OPLUS #2 or K (b)) with this transformation (FactorK) which 
 -- has a similar purpose, but works on queries.

 USES
  sft: (f o g) o h --> f o (g o h),
  LB,
  PullP2SHRF, PullP2SHRP,
  FKAux
 BEGIN
  -- First do the work
  FALSE {LB;
         BU {sft -> {GIVEN (f o g) DO FKAux (g); sft INV};
			 GIVEN ITERATE (p, f) DO {PullP2SHRF (f); PullP2SHRP (p)};
			 GIVEN f o g DO FKAux;
			 GIVEN <f, g> DO FKAux
		    }
		};

  -- Return true if successful
  GIVEN K (_O) DO SKIP 
 END
