#include "DEVisConsole.H"
#include <stream.h>

DVisConsole::DVisConsole(DVertSplit * parent = 0, const char * name = 0):QMultiLineEdit(parent,name)
{
  currentEndRow=0;
  currentEndCol=0;
  begCount = 0;
  parentWid=parent;
}

void DVisConsole::gotoEnd()
{
  setCursorPosition(numLines()-1,0); //go to the last row
  end(); //go to the last column in this row
}


void DVisConsole::insertAtEnd(const char* toInsert)
{
  gotoEnd();
  int x; int y;
  getCursorPosition(&x,&y);
  insertAt(toInsert,x,y);
  gotoEnd();
  getCursorPosition(&x,&y);
  currentEndRow=x;
  currentEndCol=y;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: keyPressEvent
// Parameters:	()
//
// This method forces all text to be written to the end of the console. It
// checks to see if return or enter was pressed and initiates the response.
//         								    
//////////////////////////////////////////////////////////////////////////////

void DVisConsole::keyPressEvent(QKeyEvent * e)
{
  gotoEnd(); //bug: left-arrow doesn't really work because of this line
  int x; int y;
  getCursorPosition(&x,&y);
  //begCount measures how far we have progressed from the beginning of the line. This method does not allow a backspace if begCount is 0.
  if ((e->key() != Key_Backspace) || (begCount))
    QMultiLineEdit::keyPressEvent(e);
  int newx; int newy;
  getCursorPosition(&newx,&newy);
  if ((e->key() == Key_Backspace) && (begCount)){
    begCount--;
  }
  else if (newy != y)
    begCount++;
  if ((e->key() == Key_Return) || (e->key() == Key_Enter)){
    setSelection(currentEndRow,currentEndCol,x,y);
    QString str(markedText());
    parentWid->dealWithSelection(&str);
    begCount = 0;
  }
}
