//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi
//  FILE: RuleBlockList.C
//  PROJ: COKO DEBUGGER
//  DATE: SUMMER 2001                                                      
//
// SYNOPSIS
//  Classes for RuleBlock List.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _RULEBLOCKLIST_C_
#define _RULEBLOCKLIST_C_

#include "RuleBlockList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: RuleBlockNode
// Parameters:	(CRuleBlock*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

RuleBlockNode::RuleBlockNode(CRuleBlock * a):	
  _next(NULL),
  _value(NULL),
  _name(NULL),
  _breakArray(NULL),
  _symList(NULL),
  _conditionArray(NULL)
{
  _value = a;
  _symList = new SymbolList();
  char * temp = a->get_name();
  _name = new char[strlen(temp)+1];
  strcpy(_name, temp);
  delete [] temp;
  _breakNext=0;
  _arraySize=0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: RuleBlockNode
// Parameters:	(CRuleBlock*, int)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

RuleBlockNode::RuleBlockNode(CRuleBlock * a, int num_stmts):	
  _next(NULL),
  _value(NULL),
  _name(NULL),
  _breakArray(NULL),
  _symList(NULL),
  _conditionArray(NULL)
{
  _value = a;
  char * temp = a->get_name();
  _name = new char[strlen(temp)+1];
  strcpy(_name, temp);
  delete [] temp;
  _breakNext=0;
  _arraySize=num_stmts;
  _breakArray = new int[num_stmts];
  for (int i = 0; i < num_stmts; i++)
    _breakArray[i]=0;
  _conditionArray = new int[num_stmts];
  for (int i = 0; i < num_stmts; i++)
    _conditionArray[i]=0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~RuleBlockNode
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
  
RuleBlockNode::~RuleBlockNode(void) 
{
  if (_value) { _value = NULL; }
  if(_breakArray) {delete [] _breakArray; _breakArray = NULL;}
  if(_conditionArray) {delete [] _conditionArray; _conditionArray = NULL;}
  if(_symList) {delete _symList; _symList = NULL;}
  _next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only
//         								    
//////////////////////////////////////////////////////////////////////////////
  
void RuleBlockNode::display(void) 
{
  if (_value) {
      cout <<  _name  << endl;
  }
  cout << "_breakNext: " << _breakNext << endl;
  cout << "_arraySize: " << _arraySize << endl;
  if(_breakArray) {
    cout << "BREAK ARRAY:" << endl;
    for (int i = 0; i < _arraySize; i++)
      cout << _breakArray[i];
    cout << endl;
  }
  if(_conditionArray) {
    cout << "CONDITION ARRAY:" << endl;
    for (int i = 0; i < _arraySize; i++)
      cout << _conditionArray[i];
    cout << endl;
  }
  if (_symList) {
    cout << "SYMBOL LIST" << endl;
    _symList->display();
  }
  if (_breakArray) {
    cout << "Break List: ";
    for (int i = 0; i < _arraySize; i++)
      cout << _breakArray[i] << " " << endl;
    cout << endl;
  }
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (CRuleBlock*)
//
// Insert a new node with _value = a to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void RuleBlockList::insert(CRuleBlock* a) 
{
  //cout << "Inserting: " << a -> get_name() << endl;
  if (!_head)
	_head = _tail = new RuleBlockNode(a); 
  else 
	{
	  RuleBlockNode * tmp = new RuleBlockNode(a);
	  RuleBlockNode * tmp2 = _head;
	  _head = tmp;
	  _head->_next = tmp2;
	  tmp = NULL;
	  tmp2 = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (CRuleBlock*, int)
//
// Insert a new node with _value = a to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void RuleBlockList::insert(CRuleBlock* a, int num_stmts) 
{
  //cout << "Inserting: " << a -> get_name() << endl;
  if (!_head)
	_head = _tail = new RuleBlockNode(a, num_stmts); 
  else 
	{
	  RuleBlockNode * tmp = new RuleBlockNode(a, num_stmts);
	  RuleBlockNode * tmp2 = _head;
	  _head = tmp;
	  _head->_next = tmp2;
	  tmp = NULL;
	  tmp2 = NULL;
	}
  //cout << "Inserted:" << endl;
  //_head->display();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// Pop off a front node from the list.
// Return success value of the pop.
//         								    
//////////////////////////////////////////////////////////////////////////////

int RuleBlockList::pop(void)
{
  if (!_head)
	return 0;

  //cout << "Popping: " << _head -> _value -> get_name() << endl;

  RuleBlockNode * tmpnode = _head;

  if (_head == _tail)
	_head = _tail = NULL;
  else
	_head = _head->_next;

  if (tmpnode->_value) 
	{ delete tmpnode; tmpnode = NULL; }
  else
	{
	  cout << "Error @ RuleBlockList : pop " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return 0;
	}

  //cout << "New top: " << _head -> _value -> get_name() << endl;
  return 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topRB
// Parameters: ()
//
// Return _value of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

CRuleBlock* RuleBlockList::topRB(void)
{
  if (!_head)
	return NULL;

  if (_head->_value) 
	return _head->_value;
  else
	{
	  cout << "Error @ RuleBlockList : top " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topSym
// Parameters: ()
//
// Return _symList of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

SymbolList* RuleBlockList::topSym(void)
{
  if (!_head)
	return NULL;

  if (_head->_symList) 
	return _head->_symList;
  else
	{
	  cout << "Error @ RuleBlockList : top " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topBA
// Parameters:	(void)
//
// Returns _breakArray of the top node.
//
//////////////////////////////////////////////////////////////////////////////

int* RuleBlockList::topBA(void)
{
  if (_head)
	return _head->_breakArray;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topArraySize
// Parameters:	(void)
//
// Returns _arraySize of the top node.
//
//////////////////////////////////////////////////////////////////////////////

int RuleBlockList::topArraySize(void)
{
  if (_head)
	return _head->_arraySize;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topCA
// Parameters:	(void)
//
// Returns _conditionArray of the top node.
//
//////////////////////////////////////////////////////////////////////////////

int* RuleBlockList::topCA(void)
{
  if (_head)
	return _head->_conditionArray;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: topBN
// Parameters: ()
//
// Return _breakNext value of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

int RuleBlockList::topBN(void)
{
  if (!_head)
	return 0;

  if (_head->_value) 
	return _head->_breakNext;
  else
	{
	  cout << "Error @ RuleBlockList : top " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return 0;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: set_topBN
// Parameters: (int a)
//
// Set _breakNext value of the top node to a.
// Returns success valueof operation
//         								    
//////////////////////////////////////////////////////////////////////////////

int RuleBlockList::set_topBN(int a)
{
  if (!_head)
	return 0;

  if (_head->_value) {
	_head->_breakNext = a;
	return 1;
  }
  else
	{
	  cout << "Error @ RuleBlockList : top " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return 0;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: empty
// Parameters: (void)
//
// Is the list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int RuleBlockList::empty(void) 
{ 
  return (!_head); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIterator 
// Parameters: (void)
//
// Initialize _iterator.
// This method is used with getValue method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void RuleBlockList::initIterator(void)
{
  _iterator = _head;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorValue
// Parameters:	(void)
//
// Returns _value of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

CRuleBlock* RuleBlockList::iteratorValue(void)
{
  if (_iterator)
	return _iterator->_value;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorName
// Parameters:	(void)
//
// Returns copy of _name of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

char* RuleBlockList::iteratorName(void)
{
  if (_iterator) {
    char* temp = new char[strlen(_iterator->_name) + 1];
    strcpy(temp, _iterator->_name);
    return temp;
  }
  else
    return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setIteratorBA
// Parameters:	(int)
//
// Sets _breakArray[int] of the node that _iterator is pointing to to 1.
//
//////////////////////////////////////////////////////////////////////////////

//void RuleBlockList::setIteratorBA(int a)
//{
//  if (_iterator)
//	(_iterator->_breakArray)[a] = 1;
//  else {
//    cout << "Error: iterator null" << endl;
//    exit(1);
//  }
//}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorBA
// Parameters:	(void)
//
// Returns _breakArray of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int* RuleBlockList::iteratorBA(void)
{
  if (_iterator)
	return _iterator->_breakArray;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorArraySize
// Parameters:	(void)
//
// Returns _arraySize of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int RuleBlockList::iteratorArraySize(void)
{
  if (_iterator)
	return _iterator->_arraySize;
  else
	return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorCA
// Parameters:	(void)
//
// Returns _conditionArray of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

int* RuleBlockList::iteratorCA(void)
{
  if (_iterator)
	return _iterator->_conditionArray;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getNextName
// Parameters:	(void)
//
// Returns _name of the next node that _iterator is pointing to.
// After a call to this method, _iterator will point to the next node.
// One can call this methods repeatly to get the _value of all the nodes.
// However, initIterator has to be called before the initial call to this 
// method.  
//         								    
//////////////////////////////////////////////////////////////////////////////

char* RuleBlockList::getNextName()
{
  char* return_val;
  if (_iterator)
	{
	  _iterator = _iterator->_next;
	  if (_iterator) {
	    return_val = new char [strlen(_iterator->_name) + 1];
	    strcpy(return_val, _iterator->_name);
	  }
	  else
	    return_val = NULL;
	}
  else
	return_val = NULL;
  return return_val;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorNext
// Parameters: (void)
//
// Simply makes the iterator point to the next value in the list
// and does not return anything.
//         								    
//////////////////////////////////////////////////////////////////////////////

void RuleBlockList::iteratorNext()
{
  if (_iterator)
    _iterator = _iterator->_next;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: RuleBlockList
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

RuleBlockList::RuleBlockList():
  _head(NULL),
  _tail(NULL),
  _iterator(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:~RuleBlockList
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

RuleBlockList::~RuleBlockList()
{
  RuleBlockNode * tmp = _head;
  _tail = NULL;
  while (_head)
   	{
 	  _head = _head->_next;
 	  delete tmp;
 	  tmp = _head;
 	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: display
// Parameters:	(void)
//
// For debugging purposes only. Displays the contents of the entire list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void RuleBlockList::display()
{
  cout << "DISPLAYING LIST" << endl;
  initIterator();
  _iterator->display();
  for(;;)
    if (getNextName())
      _iterator->display();
    else
      break;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: displayStack
// Parameters:	(void)
//
// Goes thorugh the entire list and adds the names of each Rule Block to
// a char* that it returns at the end.
//         								    
//////////////////////////////////////////////////////////////////////////////

char* RuleBlockList::displayStack()
{
  initIterator();
  int count = 0;
  char* name;
  while (name = iteratorName()) {
    count += strlen(name)+1;
    delete[] name;
    _iterator = _iterator->_next;
  }
  initIterator();
  char* returnVal = new char[count+1];
  returnVal[0] = '\0';
  while (name = iteratorName()) {
    strcat(returnVal,name);
    strcat(returnVal,"\n");
    delete[] name;
    _iterator = _iterator->_next;
  }
  return returnVal;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getIterator
// Parameters: (void)
//
// Returns the iterator.
//         								    
//////////////////////////////////////////////////////////////////////////////

RuleBlockNode* RuleBlockList::getIterator()
{
  return _iterator;
}

#endif
