//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi
//  FILE: RuleBlockList.H
//  PROJ: COKO DEBUGGER
//  DATE: SUMMER 2001                                                     
//
// SYNOPSIS
//  Classes for NonType List.
//  
//////////////////////////////////////////////////////////////////////////////


#ifndef _RULEBLOCKLIST_H_
#define _RULEBLOCKLIST_H_

//#include <strings.h>
//#include <stdlib.h>
//#include <iostream.h>
//#include "KONonType.H"
#include "PCRuleBlock.H"
#include "SymbolList.H"

class RuleBlockNode
{
public:
  CRuleBlock * _value;
  char* _name;
  //I could have doen _value->get_name() but if _value was deleted elsewhere, then the get_name function doesn't work.
  int _breakNext;
  int _arraySize;
  int* _breakArray;
  int* _conditionArray;
  SymbolList* _symList;
  RuleBlockNode * _next;

  RuleBlockNode(CRuleBlock * c);
  // Constructor

  RuleBlockNode(CRuleBlock * c, int num_stmts);
  // Constructor

  ~RuleBlockNode();
  // Destructor

  void display();
  //for debugging purposes only

};

class RuleBlockList
{
private:
  RuleBlockNode * _iterator;
  RuleBlockNode * _head;
  RuleBlockNode * _tail;

public:

  RuleBlockList();
  // Constructor
  
  ~RuleBlockList();
  // Destructor

  void insert(CRuleBlock* a);
  //  Insert a new node with _value = a to the list

  void insert(CRuleBlock* a, int num_stmts);
  //  Insert a new node with _value = a to the list

  int pop();
  // Pop off a front node from the list
  // Return success value of the pop.

  CRuleBlock* topRB();
  // Returns _value of the top node.

  SymbolList* topSym();
  // Returns _symList of the top node.

  int set_topBN(int a);
  // sets _breakNext of the top node.
  // returns success value of operation

  int topBN();
  // Returns _breakNext of the top node.

  int* topBA();
  // Returns _breakArray of the top node.

  int topArraySize();
  // Returns _arraySize of the top node.

  int* topCA();
  // Returns _conditionArray of the top node.

  int empty();
  // Is the list empty?

  void initIterator();
  // Initialize _iterator
  // This method is used with getValue method

  CRuleBlock* iteratorValue();
  // Returns _value of the node that _iterator is pointing to.

  char* iteratorName();
  // Returns _name of the node that _iterator is pointing to.

  int* iteratorBA();
  // Returns _breakArray of the node that _iterator is pointing to.

  int iteratorArraySize();
  // Returns _arraySize of the node that _iterator is pointing to.

  int* iteratorCA();
  // Returns _conditionArray of the node that _iterator is pointing to.

  char* getNextName();
  // Returns _value of the next node that _iterator is pointing to.
  // After a call to this method, _iterator will point to the next node.
  // One can call this methods repeatly to get the _value of all the nodes.
  // However, initIterator has to be called before the initial call to this 
  // method.  

  void iteratorNext();
  // Simply makes the iterator point to the next value in the list
  // and does not return anything.

  RuleBlockNode* getIterator();
  // Returns _iterator;

  void display();
  //for debugging purposes only.

  char* displayStack();
  // Goes thorugh the entire list and adds the names of each Rule Block to
  // a char* that it returns at the end.
  
};

#endif

