//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon Suk Lee (jl) 
//  FILE: CondList.C
//  PROJ: COKO
//  DATE: 10/97                                                              
//
// SYNOPSIS
//  Classes for Condition list
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _CONDLIST_C_
#define _CONDLIST_C_

#include "CondList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CondNode
// Parameters:	(char*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CondNode::CondNode(CCond * a):	
  _next(NULL),
  _value(NULL)
{
  _value = a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CondNode
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
  
CondNode::~CondNode(void) 
{
  if (_value) { delete _value; _value = NULL; }
  _next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (char*)	
//
// Insert a new node with _value = a to the list.
// Insert a new node at the end of the list.
//        								    
//////////////////////////////////////////////////////////////////////////////

void CondList::insert(CCond* a) 
{
  if (!_head)
	_head = _tail = new CondNode(a); 
  else 
	{
	  CondNode * tmp = new CondNode(a);
	  _tail->_next = tmp;
	  _tail = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// Pop off a front node from the list.
// Return success value of the pop.
//         								    
//////////////////////////////////////////////////////////////////////////////

int CondList::pop(void)
{
  if (!_head)
	return NULL;

  CondNode * tmpnode = _head;

  if (_head == _tail)
	_head = _tail = NULL;
  else
	_head = _head->_next;

  if (tmpnode->_value) 
	{ delete tmpnode; tmpnode = NULL; }
  else
	{
	  cout << "Error @ CondList : pop " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}

  return 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: top
// Parameters: (CCond*)
//
// Return _value of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

CCond* CondList::top(void)
{
  if (!_head)
	return NULL;

  if (_head->_value) 
	return _head->_value;
  else
	{
	  cout << "Error @ CondList : top " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: empty
// Parameters: (void)
//
// Is the list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int CondList::empty(void) 
{ 
  return (!_head); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIterator 
// Parameters: (void)
//
// Initialize _iterator.
// This method is used with getValue method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CondList::initIterator(void)
{
  _iterator = _head;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getValue
// Parameters:	(void)
//
// Returns _value of the node that _iterator is pointing to.
// After a call to this method, _iterator will point to the next node.
// One can call this methods repeatly to get the _value of all the nodes.
// However, initIterator has to be called before the initial call to this 
// method.  
//         								    
//////////////////////////////////////////////////////////////////////////////

CCond* CondList::getValue()
{
  CCond* return_val;
  if (_iterator)
	{
	  return_val = _iterator->_value;
	  _iterator = _iterator->_next;
	}
  else
	return_val = NULL;
  return return_val;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CondList
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CondList::CondList():
  _head(NULL),
  _tail(NULL),
  _iterator(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:~CondList
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CondList::~CondList()
{
    CondNode * tmp = _head;
	_tail = NULL;
    while (_head)
  	{
	  _head = _head->_next;
	  delete tmp;
	  tmp = _head;
	}
}

#endif

