//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon-Suk Lee (jl)
//  FILE: PCCond.C
//  PROJ: COKO
//  DATE: 10/9/96                                                              
//
// SYNOPSIS
// Class definitions for CCond.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _PCCCOND_C_
#define _PCCOND_C_

#include "PCCond.H"
#include <string>
#include "regex.h"
#include "PCIdent.H"
#include "KOFUser.H"
#include "KOBName.H"
#include "KOPUser.H"
#include "KOOName.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CCond
// Parameters:	(char* a ... )
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CCond::CCond(Property *p, NonTypeArgs *n ): prop(p), args(n)
{
 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CCond
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CCond::~CCond(void)
{
  if (args) {delete args; args = NULL;}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters:	(CState*)
//
// Evaluates the property. If the NonTypeArgs object returned has a success 
// value of true  - the property was successfully evaluated. If there are 
// any arguments passed back - these represent the arguments for which values
// were successfully found. These arguments are all either an BVar, FVar, PVar,
// or OVar. Their names are of the form var-name:var-value. A variable binding
// is then created of type BName, FUser, PUser and OName, respectively. The
// key is var-name, and the value is var-value.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState* CCond::Exec(CState * s)
{
  NonTypeArgs *a = args->build(s);
  
  CState *state = new CState( s->Store()->Copy_kTree(), new CIEnvStack(), 0 );

  NonTypeArgs *arguments = prop->isTrue(state,a);
  
  if (arguments->GetSuccess()){  

    // execute the following if there are variable bindings to be added
    if (arguments->numArgs()!=0){
      NonTypeList *list = arguments->theList();
      list->initIterator();
      NonType *current;
      regex_t r;

      // the name of each variable is: var-name:var-value.
      // this regular expression should capture the two elements
      // compile a regular expression to match this
      regcomp(&r,"(.+):(.+)", REG_EXTENDED|REG_ICASE);

      regmatch_t matches[r.re_nsub+1];
      size_t size = r.re_nsub +1;
      string temp;
      int start1, length1, start2,length2;

      // while there is a next element in the list
      while(current=list->getValue()){ 
      
	// store the name in temp
	temp = current->Name();
	
	// try to match the compiled expression to the name of the variable
	regexec(&r, temp.c_str(), size, matches, 0);
      

	// retrieve start and length positions for both matches
	start1 = matches[1].rm_so;
	length1 = matches[1].rm_eo-start1;	
	start2 = matches[2].rm_so;
	length2 = matches[2].rm_eo-start2;

	NameUnion *tmp = current->Myname();
	
	// if the variable was a boolean, create a named boolean
	if ((tmp->tag==BNAME) && (tmp->bn == BOOL_VAR))
	  s->IInsert(temp.substr(start1, length1).c_str(), 
		     new CIdent(NULL,new BName((char*)temp.substr(start2,length2).c_str()),0));
	
	// if the variable was a function, create a user defined function
	if ((tmp->tag==FNAME) && (tmp->fn == FUN_VAR))
	  s->IInsert(temp.substr(start1, length1).c_str(), 
		     new CIdent(NULL,new FUser((char*)temp.substr(start2,length2).c_str()),0));
	
	// if the variable was a predicate, create a user defined predicate
	if ((tmp->tag==PNAME) && (tmp->pn == PRED_VAR))
	  s->IInsert(temp.substr(start1, length1).c_str(), 
		     new CIdent(NULL,new PUser((char*)temp.substr(start2,length2).c_str()),0));

	// if the variable was an object, create a named object
	if ((tmp->tag==ONAME) && (tmp->on == OBJ_VAR))
	  s->IInsert(temp.substr(start1, length1).c_str(), 
		     new CIdent(NULL,new OName((char*)temp.substr(start2,length2).c_str()),0));
     
	delete tmp;
      }

      regfree(&r);

      NonTypeList *l = a->theList();
      l->initIterator();
      NonType * c;
      int count = 0;

      while(c = l->getValue())
	if (c->isChildVar(c))
	  count++;
      
      if (arguments->numArgs() != count)
	s->SetSuccess(F);
      else
	s->SetSuccess(T);
      
      delete l;
    }  
    else{
      s->SetSuccess(T);  
     
      NonTypeList *list = a->theList();
      list->initIterator();
      NonType *current;
      // this while loop checks to make sure that even though 0 variable values were returned,
      // that args has no variables that are expecting to get bound to values.
      // This is necessary in the case where a variable that is expected to get bound in the
      // property, is matched with a don't care variable in the property. Say we want to evaluate
      // Inj(g), and the value of g has to be created in the property. The property has a rule:
      // Key(_f) ==> Inj(f) (this is a stupid rule, but a valid one). This rule will evaluate true,
      // because the f on the RHS gets matched to a variable that will get its value - when Key
      // is evaluated, _f matches something in Schema. But there is no way for Schema to know
      // that it has to bind this variable. Hence, in this case the check to make sure there are
      // no variables expecting values is necessary.
      while( current = list->getValue())
	if (current->isChildVar(current)){
	  s->SetSuccess(F);
	  break;
	}
      
      delete list;
    }
  }
  else   
    s->SetSuccess(F);  
  
  delete a;
  delete arguments;
  delete state;
  
  return s;
}

#endif
