//////////////////////////////////////////////////////////////////////////////
//
//  NAME: Joon S.  Lee (jl)
//  FILE: PCEqn.C
//  PROJ: COKO
//  DATE: 10/9/96
//
// SYNOPSIS
//  Class definitions for Eqns.
//
//////////////////////////////////////////////////////////////////////////////
 
#ifndef _PCEQN_C_
#define _PCEQN_C_

#include "PCEqn.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CEqn
// Parameters: (char*, CPattern*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CEqn::CEqn(char* lhs, CPattern* rhs):
  _rhs(NULL),
  _lhs(NULL),
  _evalFlag(0)
{
  if (lhs)
  	{
  	  _lhs = new char[strlen(lhs)+1];
  	  strcpy(_lhs, lhs);
  	}
  _rhs = rhs;
  _evalFlag = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CEqn 
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CEqn::~CEqn(void)
{
  if (_lhs) { delete [] _lhs; _lhs = NULL; }
  if (_rhs) { delete _rhs; _rhs = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SetEF
// Parameters: (void)
//
// Set _evalFlag ( _evalFlag = 1 )
//         								    
//////////////////////////////////////////////////////////////////////////////

void CEqn::SetEF(void)
{
  _evalFlag = 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: UnsetEF
// Parameters: (void)
//
// Unset _evalFlag ( _evalFlag = 0 )
//         								    
//////////////////////////////////////////////////////////////////////////////

void CEqn::UnsetEF(void)
{
  _evalFlag = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters: (CState*)
//
// What to do when executed
//         								    
//////////////////////////////////////////////////////////////////////////////

CState* CEqn::Exec(CState * s)
{

  if (s->ISuccess())
	{
	  if (!_lhs)
		{
		  // Special case: Eqn is the form of "(current =) CPattern"
		  if (!s->CEmptyInvDep())
			{
			  cout << "Error @ CEqn : Exec" << endl;
			  cout << "current is already defined" << endl;
			  cout << "varable aliasing not yet allowed error" << endl;
			  exit(1);
			  s->SetSuccess(F);
			}
		  else
			{
			  if (_debug_)
				cout << "@CEqn::Exec - match for current start" << endl;
			  s->SetCurrentPattern(_rhs);
			  s->CurrentPattern()->SetStackPointer(s->IEnv()->getData());
			  s = _rhs->matchSTM(s);
			  if (s->ISuccess())
				{
				  if (_debug_)
					cout << "@CEqn::Exec - success match for current" << endl;
			  	  SetEF();
				  s = _rhs->DepListCheck(_lhs,s);	   
				}
			}
		}
	  else 
		{
		  if (!s->IEmptyInvDep(_lhs))
			{
			  cout << "Error @ CEqn : Exec " << endl;
			  cout << _lhs << " is already defined" << endl; 
			  cout << "varable aliasing not yet allowed error" << endl;
			  exit(1);
			  s->SetSuccess(F);
			}
		  else
			{
			  if (_debug_)
				cout << "@CEqn::Exec - match for " << _lhs << " start" << endl;

			  // Eqn is the form of "variable name = CPattern"
			  NonType* old_nontype = (s->Store())->Copy_kTree();
			  NonType* tmp_nontype = (s->IFindDeep(_lhs))->Child();
			  if (_debug_)
				{
				  cout << _lhs << "matched with ";
				  char* res;
				  cout << (res = tmp_nontype->representation()) << endl;
				  if(res) delete res;
				}

			  s->SetStore(tmp_nontype);
			  s->SetPatternDeep(_lhs, _rhs);
			  s = _rhs->matchSTM(s);
			  if (s->ISuccess())
				{
				  if (_debug_)
					cout << "@CEqn::Exec - success match for " << _lhs << endl;
				  SetEF();
				  s = _rhs->DepListCheck(_lhs,s);	   
				}
			  s->SetStore(old_nontype);

			  old_nontype = NULL; 
			  tmp_nontype = NULL;
			}
		}
	}
  return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters:	(CState*, DDebugController*)
//
// What to do when executed.
// First evaluate all equations.
// If all eqns match was successful, execute the body statement. 
// If there was a failure during match eqn then undo all changes made.
//
//////////////////////////////////////////////////////////////////////////////

CState *CEqn::ExecDebug(CState *s, DDebugController *db)
{
  if (db->really_execute()) {
    if (db->check_for_break(key_number))
      db->doBreak(this, s);
    if (s->ISuccess())
      {
	if (!_lhs)
	  {
	    // Special case: Eqn is the form of "(current =) CPattern"
	    if (!s->CEmptyInvDep())
	      {
		cout << "Error @ CEqn : Exec" << endl;
		cout << "current is already defined" << endl;
		cout << "varable aliasing not yet allowed error" << endl;
		exit(1);
		s->SetSuccess(F);
	      }
	    else
	      {
		if (_debug_)
		  cout << "@CEqn::Exec - match for current start" << endl;
		s->SetCurrentPattern(_rhs);
		s->CurrentPattern()->SetStackPointer(s->IEnv()->getData());
		s = _rhs->matchSTM(s);
		if (s->ISuccess())
		  {
		    if (_debug_)
		      cout << "@CEqn::Exec - success match for current" << endl;
		    SetEF();
		    s = _rhs->DepListCheck(_lhs,s);	   
		  }
	      }
	  }
	else 
	  {
	    if (!s->IEmptyInvDep(_lhs))
	      {
		cout << "Error @ CEqn : Exec " << endl;
		cout << _lhs << " is already defined" << endl; 
		cout << "varable aliasing not yet allowed error" << endl;
		exit(1);
		s->SetSuccess(F);
	      }
	    else
	      {
		if (_debug_)
		  cout << "@CEqn::Exec - match for " << _lhs << " start" << endl;
		
		// Eqn is the form of "variable name = CPattern"
		NonType* old_nontype = (s->Store())->Copy_kTree();
		NonType* tmp_nontype = (s->IFindDeep(_lhs))->Child();
		if (_debug_)
		  {
		    cout << _lhs << "matched with ";
		    char* res;
		    cout << (res = tmp_nontype->representation()) << endl;
		    if(res) delete res;
		  }
		
		s->SetStore(tmp_nontype);
		s->SetPatternDeep(_lhs, _rhs);
		s = _rhs->matchSTM(s);
		if (s->ISuccess())
		  {
		    if (_debug_)
		      cout << "@CEqn::Exec - success match for " << _lhs << endl;
		    SetEF();
		    s = _rhs->DepListCheck(_lhs,s);	   
		  }
		s->SetStore(old_nontype);
		
		old_nontype = NULL; 
		tmp_nontype = NULL;
	      }
	  }
      }
    db->check_for_condition(s, rep_string(), key_number);
  }
    return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: undoDEP
// Parameters: (CState*)
//
// undo any changes on dependency list
//         								    
//////////////////////////////////////////////////////////////////////////////

CState* CEqn::undoDEP(CState * s)
{
  if (_evalFlag)
	_rhs->undoDEP(_lhs,s);
  UnsetEF();
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CEqn::rep_string () 
{
  char *right = _rhs->representation();
  if (_lhs) {
    char *res = new char [strlen (_lhs) + strlen (right) + strlen (" = ") + 1];
    sprintf (res, "%s = %s", _lhs, right);
    delete [] right;
    return res; 
  }
  else {
    char *res = new char [strlen (right) + 1];
    sprintf (res, "%s", right);
    delete [] right;
    return res; 
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	()
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CEqn::representation ()
{
  return rep_string();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CEqn::representationWDI (DDebugController *db)
{
  char* temp = rep_string();
  int currLine = db->getLine();
  int currCol = db->getCol();
  db->updateArray(key_number-1, currLine, currCol, currLine, currCol + strlen(temp));
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Uses start parameter to assign key_number data_member and returns 
// current key-number
//         								    
//////////////////////////////////////////////////////////////////////////////

int CEqn::number_self(int start) {
  key_number = start;
  return (start + 1);
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: output_self
// Parameters:	(DDebugController *)
//
// ADDED DJA SUMMER 2001
// Outputs current statement in list format (for the debugger) if it is a 
// breakable statement; otherwise just calls output_self on any sub-statements
//         								    
//////////////////////////////////////////////////////////////////////////////

void CEqn::output_self(DDebugController *db)
{
  char* rep = rep_string();
  db->format_output(key_number, rep);
  delete[] rep;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: get_key_number
// Parameters:	(void)
//
// ADDED DJA SUMMER 2001
// Returns value stored in Key_number data member (for debugger)
//         								    
//////////////////////////////////////////////////////////////////////////////

int CEqn::get_key_number()
{
  return key_number;
}

#endif


