//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon-Suk Lee (jl)
//  FILE: PCGivenStmt.C
//  PROJ: COKO
//  DATE: 10/9/96                                                              
//
// SYNOPSIS
// Class definitions for CGivenStmt.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _PCGIVENSTMT_C_
#define _PCGIVENSTMT_C_

#include "PCGivenStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CGivenStmt
// Parameters:	(CStmt* a ... )
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CGivenStmt::CGivenStmt(CStmt* a ... )
{
  va_list ap;
  va_start (ap,a);
  _stmt = a;
  _count = 0;
  for(;;)
	{
	  CEqn* p = va_arg(ap,CEqn*);
	  if (p == NULL) 
		break;
	  else 
		_eqnList[_count++] = p;
	}
  _count--;
  va_end(ap);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CGivenStmt
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CGivenStmt::~CGivenStmt(void)
{
  for (int i=0; i <= _count; i++)
	if (_eqnList[i]) { delete _eqnList[i]; _eqnList[i] = NULL; }
  if (_stmt) { delete _stmt; _stmt = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters:	(CState*)
//
// What to do when executed.
// First evaluate all equations.
// If all eqns match was successful, execute the body statement. 
// If there was a failure during match eqn then undo all changes made.
//
//////////////////////////////////////////////////////////////////////////////

CState* CGivenStmt::Exec(CState * s)
{
  if (_debug_)
	cout << "-- GivenStmt Begins" << endl;
  
  // Save the pattern we were using and update it with the new one
  // specified in this statement. "CurrentPattern"
  // will be used to build modified kola query right after returning
  // from rule invokes.
  CPattern *old_pattern = s->CurrentPattern();

  s->INew(); // Clear our the top of identifier stack by pushing an empty
             // element.


  s->SetSuccess(T);

  int i = 0;
  int j = 0;
  while ((s->ISuccess() == T) && (i <= _count))
	{
	  s->INew(); 
	  // put new stack frame for each equation
	  s = (_eqnList[i])->Exec(s);
	  i++;
	  j++;
  	}
  i--; 

  if (s->ISuccess()) 
	// If all eqns match was successful, execute the body statement.
	{
	  s = _stmt->Exec(s);
	}
  else
	// If there was a failure during match eqn then undo all changes made.
	{
	  while (i >= 0)
		{
		  if (_debug_)
			{
			  cout << "@PCGiven:EXEC" << endl;
			  cout << "eqn match failed - start undoDEP" << endl;
			}
		  s = (_eqnList[i])->undoDEP(s);
		  i--;
		}
	}

  // clear up stack frames that were created for each equation
  while (j) 
	{
	  s->IOld();
	  (_eqnList[j - 1])->endOfEval();

	  // endOfEval added.
	  // It solves a bug that occured when I used Given stmt and TD 
	  // or BU together.
	  // A stmt inside of TD or BU is evaluated more than one
	  // time on different kola trees and not setting back an equations
	  // _evalFlag causes an error.

	  j--;
	}

  s->IOld(); // Restore the old identifier environment.
  s->SetCurrentPattern(old_pattern); // Restore the old pattern.
  s->CCleanInvDep();
  
  if (_debug_)
	cout << "-- GivenStmt Ends" << endl;

  return s;  
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters:	(CState*, DDebugController*)
//
// What to do when executed.
// First evaluate all equations.
// If all eqns match was successful, execute the body statement. 
// If there was a failure during match eqn then undo all changes made.
//
//////////////////////////////////////////////////////////////////////////////

CState* CGivenStmt::ExecDebug(CState *s, DDebugController *db)
{
  if (db->really_execute()) {
    // Save the pattern we were using and update it with the new one
    // specified in this statement. "CurrentPattern"
    // will be used to build modified kola query right after returning
    // from rule invokes.
    
    if (db->check_for_break(key_number))
      db->doBreak(this, s);
    
    if (db->really_execute()) {
      
      CPattern *old_pattern = s->CurrentPattern();
      
      s->INew(); // Clear our the top of identifier stack by pushing an empty
      // element.
      
      
      s->SetSuccess(T);
      
      int i = 0;
      int j = 0;
      while ((s->ISuccess() == T) && (i <= _count))
	{
	  s->INew(); 
	  // put new stack frame for each equation
	  s = (_eqnList[i])->ExecDebug(s,db);
	  i++;
	  j++;
	}
      i--; 
      
      if (s->ISuccess()) 
	// If all eqns match was successful, execute the body statement.
	{
	  s = _stmt->ExecDebug(s,db);
	}
      else
	// If there was a failure during match eqn then undo all changes made.
	{
	  while (i >= 0)
	    {
	      s = (_eqnList[i])->undoDEP(s);
	      i--;
	    }
	}
      
      // clear up stack frames that were created for each equation
      while (j) 
	{
	  s->IOld();
	  (_eqnList[j - 1])->endOfEval();
	  
	  // endOfEval added.
	  // It solves a bug that occured when I used Given stmt and TD 
	  // or BU together.
	  // A stmt inside of TD or BU is evaluated more than one
	  // time on different kola trees and not setting back an equations
	  // _evalFlag causes an error.
	  
	  j--;
	}
      
      s->IOld(); // Restore the old identifier environment.
      s->SetCurrentPattern(old_pattern); // Restore the old pattern.
      s->CCleanInvDep();
      
      db->check_for_condition(s, rep_string(), key_number);
    }
  }
  return s;  
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CGivenStmt::rep_string()
{
  char *res = new char [strlen ("GIVEN") + 1];
  sprintf (res, "GIVEN");
  return res;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CGivenStmt::representation (int indent)
{
  int count = 0;
  char *indent1 = get_indent_string(indent);
  char *indent2 = get_indent_string(indent+2);
  //how long are the text representations of all the equations put together?
  for (int i=0; i <= _count; i++)
    if (_eqnList[i]) { 
      count += strlen(_eqnList[i]->representation()) + strlen(indent2) + strlen(",\n");
    }
  count--;//because there is no comma at the end of the last equation
  char *stmt;
  if (_stmt->is_leaf())
    stmt = _stmt -> representation (indent+2);
  else {
    char *stmt_orig = _stmt -> representation (indent+2);
    stmt = new char[strlen(stmt_orig) + strlen("{}") + 1];
    sprintf (stmt, "{%s}", stmt_orig);
    delete[] stmt_orig;
  }
  char *rep = rep_string();
  char *res = new char [count + strlen (stmt) + strlen (rep) + strlen(indent1) + strlen(indent1) + strlen(indent2) + strlen ("\n\nDO\n") + 1];
  sprintf (res, "\n%s%s\n", indent1, rep);
  for (int i = 0; i < _count; i++)
    if (_eqnList[i]) {
      strcat(res,indent2);
      strcat(res,_eqnList[i]->representation());
      strcat(res, ",\n");
    }
  if (_eqnList[_count]) {
    strcat(res,indent2);
    strcat(res, _eqnList[_count]->representation());
    strcat(res, "\n");
  }
  strcat(res,indent1);
  strcat(res, "DO\n");
  strcat(res, indent2);
  strcat(res, stmt);
  delete [] stmt;
  delete [] rep;
  delete [] indent1;
  delete [] indent2;
  return res; 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CGivenStmt::representationWDI (int indent, DDebugController *db)
{
  char *rep = rep_string();
  int currLine = db->getLine();
  db->updateArray(key_number-1, currLine+1, indent, currLine+1, indent + strlen(rep));
  db->setLine(currLine+2);
  db->setCol(indent+2);
  int count = 0;
  char *indent1 = get_indent_string(indent);
  char *indent2 = get_indent_string(indent+2);
  //how long are the text representations of all the equations put together?
  for (int i=0; i <= _count; i++)
    if (_eqnList[i]) { 
      count += strlen(_eqnList[i]->representation()) + strlen(indent2) + strlen(",\n");
    }
  count--;//because there is no comma at the end of the last equation
  
  char *stmt;
  if (_stmt->is_leaf())
    stmt = _stmt -> representation (indent+2);
  else {
    char *stmt_orig = _stmt -> representation (indent+2);
    stmt = new char[strlen(stmt_orig) + strlen("{}") + 1];
    sprintf (stmt, "{%s}", stmt_orig);
    delete[] stmt_orig;
  }
  //char *res = new char [strlen (rep) + strlen(indent1) + strlen ("\n\n") + 1];
  char *res = new char [count + strlen (stmt) + strlen (rep) + strlen(indent1) + strlen(indent1) + strlen(indent2) + strlen ("\n\nDO\n") + 1];
  delete[] stmt;
  sprintf (res, "\n%s%s\n", indent1, rep);
  for (int i = 0; i < _count; i++)
    if (_eqnList[i]) {
      strcat(res,indent2);
      strcat(res,_eqnList[i]->representationWDI(db));
      strcat(res, ",\n");
      db->setLine(db->getLine()+1);
      db->setCol(indent+2);
    }
  if (_eqnList[_count]) {
    strcat(res,indent2);
    strcat(res, _eqnList[_count]->representationWDI(db));
    strcat(res, "\n");
    db->setLine(db->getLine()+2);
    db->setCol(indent+2);
  }
  if (_stmt->is_leaf())
    stmt = _stmt -> representationWDI (indent+2, db);
  else {
    db->setCol(db->getCol()+1);
    char *stmt_orig = _stmt -> representationWDI (indent+2,db);
    stmt = new char[strlen(stmt_orig) + strlen("{}") + 1];
    sprintf (stmt, "{%s}", stmt_orig);
    delete[] stmt_orig;
    db->setCol(db->getCol()+1);
  }
  strcat(res,indent1);
  strcat(res, "DO\n");
  strcat(res, indent2);
  strcat(res, stmt);
  delete [] stmt;
  delete [] rep;
  delete [] indent1;
  delete [] indent2;
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Uses start parameter to assign key_number data_member, calls number_self 
// on any sub_statements/equations and returns current key-number
//         								    
//////////////////////////////////////////////////////////////////////////////

int CGivenStmt::number_self (int start)
{
  key_number=start;
  int temp = start + 1;
  for (int i = 0; i < _count; i++)
    if (_eqnList[i]) {
      temp = _eqnList[i] -> number_self(temp);
    }
  if (_eqnList[_count]) {
    temp = _eqnList[_count] -> number_self(temp);
  }
  return (_stmt -> number_self(temp));
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: output_self
// Parameters:	(DDebugController *)
//
// ADDED DJA SUMMER 2001
// Outputs current statement in list format (for the debugger) if it is a 
// breakable statement; otherwise just calls output_self on any sub-statements
//         								    
//////////////////////////////////////////////////////////////////////////////

void CGivenStmt::output_self (DDebugController *db)
{
  //output GIVEN
  char* rep = rep_string();
  db->format_output(key_number, rep);
  delete[] rep;

  //output each equation
  for (int i = 0; i < _count; i++)
    if (_eqnList[i]) {
      _eqnList[i] -> output_self(db);
    }
  if (_eqnList[_count]) {
    _eqnList[_count] -> output_self(db);
  }

  //output sub-statement
  _stmt -> output_self(db);
}

int CGivenStmt::is_leaf()
  //returns whether current statement is a leaf in the COKO tree
{
  //I know that GIVEN isn't really a leaf, but for the purposes of the
  //representation method which uses is_leaf, GIVEN is just one statement
  //like a leaf
  return 1;
}

#endif

