//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Kee-Eung Kim (kek), Joon S. Lee (jl)                                             
//  FILE: PCIEnv.C
//  PROJ: COKO
//  DATE: 5/5/97
//
// SYNOPSIS
//  Mothod definitions file for classes in PCIEnv.H
//  
//////////////////////////////////////////////////////////////////////////////

#include "PCIEnv.H"
#include <string.h>
#include <stdlib.h>

#define FALSE 0
#define TRUE 1


/******************************************************************************
**
**
**  CLASS: CIEnvValueType
**
**
******************************************************************************/

// Get methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: GetKey 
// Parameters: (void)
//
// Returns _key.
//         								    
//////////////////////////////////////////////////////////////////////////////

const char * CIEnvValueType::GetKey(void)
{
  return _key;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: GetContent 
// Parameters: (void)
//
// Returns _content.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIdent * CIEnvValueType::GetContent(void)
{
  return _content;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: GetList
// Parameters: (void)
//
// Returns _DEPList.
//         								    
//////////////////////////////////////////////////////////////////////////////

DEPList * CIEnvValueType::GetList(void)
{
  return _DEPList;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: GetInvList
// Parameters: (void)
//
// Returns _InvDEPList.
//         								    
//////////////////////////////////////////////////////////////////////////////

DEPList * CIEnvValueType::GetInvList(void)
{
  return _InvDEPList;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: GetPattern
// Parameters: (void)
//
// Returns _pattern.
//         								    
//////////////////////////////////////////////////////////////////////////////

CPattern * CIEnvValueType::GetPattern(void)
{
  return _pattern;
}

// Set methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SetKey
// Parameters: (char*)
//
// _key modifier
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnvValueType::SetKey(char *key)
{
  _key = new char[strlen(key) + 1];
  strcpy(_key, key);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SetContent 
// Parameters: (CIdent*)
//
// _content modifier
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnvValueType::SetContent(CIdent * content)
{
  if (_content) { delete _content; _content = NULL; }
  _content = content;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SetPattern
// Parameters: (CPattern*)
//
// _pattern modifier
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnvValueType::SetPattern(CPattern * p)
{
  _pattern = p;
}

// constructor and destructor

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CIEnvValueType
// Parameters: (char*, CIdent*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvValueType::CIEnvValueType(const char * key, CIdent * content)
{
  _key = new char[strlen(key) + 1];
  strcpy(_key, key);

  _content = content;
  _DEPList = new DEPList();
  _InvDEPList = new DEPList();
  _pattern = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CIEnvValueType
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvValueType::~CIEnvValueType(void)
{
  if (_key) 
	{ 
	  delete [] _key; _key = NULL; 
	}
  if (_DEPList) 
	{
	  delete _DEPList; _DEPList = NULL;
	}
  if (_InvDEPList) 
	{
	  delete _InvDEPList; _InvDEPList = NULL;
	}
  if (_content) 
	{
	  delete _content; _content = NULL;
	}
  _pattern = NULL;
  // Do not delete _pattern.
  // It is deleted somewhere else.
}

// dependency list methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  insertDepList
// Parameters: (char*)
//
// Insert an item into the dependency list
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvValueType::insertDepList(char* a)
{
  if (_DEPList->find(a) != NULL) 
	{
  	  cout << "Error @ PCIEnv : insertDepList" << endl;
 	  cout << "duplicate entry error" << endl;
  	  cout << a << " is inserted twice. " << endl;
 	  exit(1);
  	  return 0;
  	}
  else
 	{
	  _DEPList->insert(a);
	  return 1;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: emptyDepList
// Parameters: (void) 
//
// Is dependency list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvValueType::emptyDepList(void) 
{ 
  return _DEPList->empty();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findDepList
// Parameters: (const char*)
//
// Is item in the dependency list?
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvValueType::findDepList(const char* item)
{
  if (_DEPList->find(item) == NULL)
	return 0;
  else
	return 1;
}

// inverse dependency list methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: popInvDepList
// Parameters: (void)
//
// Pop an item from inverse dependency list
// and return it
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CIEnvValueType::popInvDepList(void)
{
  return _InvDEPList->pop();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insertInvDepList
// Parameters: (char*)
//
// Insert entry into the inverse dependency list
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvValueType::insertInvDepList(char* a)
{
  if (_InvDEPList->find(a) != NULL) 
 	{
	  cout << "Error @ PCIEnv : insertInvDepList" << endl;
	  cout << "duplicate entry error" << endl;
	  cout << a << " is inserted twice. " << endl;
	  exit(1);
 	  return 0;
 	}
  else
 	{
	  _InvDEPList->insert(a);
	  return 1;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: emptyInvDepList
// Parameters: (void) 
//
// Is inverse dependency list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvValueType::emptyInvDepList(void) 
{ 
  return _InvDEPList->empty();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findInvDepList
// Parameters: (const char*)
//
// is item in the inverse dependency list?
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvValueType::findInvDepList(const char* item)
{
  if (_InvDEPList->find(item) == NULL)
	return 0;
  else
	return 1;
}

/******************************************************************************
**
**
**  CLASS: CIEnvNodeType
**
**
******************************************************************************/

// Constructor and Destructor

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CIEnvNodeType
// Parameters: (void)
//
// Parameterless Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvNodeType::CIEnvNodeType(void):
  _data(NULL),
  _pre(NULL),
  _next(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CIEnvNodeType
// Parameters: (CIEnvValueType*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvNodeType::CIEnvNodeType(CIEnvValueType * data):
  _pre(NULL),
  _next(NULL)
{
  _pre = NULL;
  _next = NULL;
  _data = data;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CIEnvNodeType
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvNodeType::~CIEnvNodeType(void)
{
  if (_data) { delete _data; _data = NULL; }
  _pre = NULL; _next = NULL;
}

// Get methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: GetData
// Parameters: (void)
//
// Returns _data.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvValueType * CIEnvNodeType::GetData(void)
{
  return _data;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: GetPattern
// Parameters: (void)
//
// Get _pattern of _data.
//         								    
//////////////////////////////////////////////////////////////////////////////

CPattern * CIEnvNodeType::GetPattern(void)
{
  if (_data != NULL)
 	return _data->GetPattern();
  else
 	return NULL;
}

// Set methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SetData
// Parameters: (CIdent*)
//
// _data modifier.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnvNodeType::SetData(CIdent* data)
{
  _data->SetContent(data);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SetPattern
// Parameters: (CPattern*)
//
// Modify _pattern of _data.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnvNodeType::SetPattern(CPattern * p)
{
  _data->SetPattern(p);
}

// dependency list methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insertDepList
// Parameters: (char*)
//
// Insert an item into the dependency list of _data.
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvNodeType::insertDepList(char* a)
{
  return _data->insertDepList(a);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: emptyDepList
// Parameters: (void) 
//
// Is dependency list of _data empty? 
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvNodeType::emptyDepList(void) 
{ 
  return _data->emptyDepList();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findDepList
// Parameters: (const char*)
//
// Is item in the dependency list of _data?
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvNodeType::findDepList(const char * item)
{
  return _data->findDepList(item);
}

// inverse dependency list methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: popInvDepList
// Parameters: (void)
//
// Pop an item from inverse dependency list of _data
// and return it
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CIEnvNodeType::popInvDepList(void)
{
  return _data->popInvDepList();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insertInvDepList
// Parameters: (char*)
//
// Insert entry into the inverse dependency list of _data
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvNodeType::insertInvDepList(char* a)
{
  return _data->insertInvDepList(a);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: emptyInvDepList
// Parameters: (void) 
//
// Is inverse dependency list of _data empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvNodeType::emptyInvDepList(void) 
{ 
  return _data->emptyInvDepList();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: findInvDepList
// Parameters: (const char*)
//
// Is item in the inverse dependency list of _data?
//         								    
//////////////////////////////////////////////////////////////////////////////

int CIEnvNodeType::findInvDepList(const char * item)
{
  return _data->findInvDepList(item);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIteratorDEP
// Parameters: (void)
//
// Call initIterator method of _DEPList of _data
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnvNodeType::initIteratorDEP(void)
{
  (_data->GetList())->initIterator();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getValueDEP
// Parameters: (void)
//
// Call getValueDEP method of _DEPList of _data
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CIEnvNodeType::getValueDEP(void)
{
  return (_data->GetList())->getValue();
}


/******************************************************************************
**
**
**  CLASS: CIEnv
**
**
******************************************************************************/

// Constructor and Destructor

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CIEnv
// Parameters: (void)
//
// Parameterless constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnv::CIEnv(void):
  _head(NULL),
  _tail(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CIEnv
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnv::~CIEnv(void)
{
  CIEnvNodeType * iterator = _head;
  CIEnvNodeType * tmp = NULL;
  if (_head)
   	{
   	  _head = NULL;
   	  _tail = NULL;
   	  tmp = iterator;
   	  if (iterator)
   		iterator = iterator->_next;
   	  while (iterator)
   		{
   		  if (tmp) delete tmp;
   		  tmp = iterator;
   		  if (iterator)
   			iterator = iterator->_next;
   		}
   	  if (tmp) delete tmp;
   	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: first
// Parameters: (void)
//
// Returns the top node value.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvValueType * CIEnv::first(void)
{
  if(_head) 
	return _head->GetData(); 
  else 
	return NULL; 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: firstNode
// Parameters: (void)
//
// Returns the top node.
// ADDED BY DJA WINTER 2002
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvNodeType * CIEnv::firstNode(void)
{
  return _head; 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: find
// Parameters: (const char*) 
//
// Search function for a given key.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIdent * CIEnv::find(const char *key)
{
  CIEnvNodeType *iterator = _head;
  if (iterator)
	while (iterator)
	  if (strcmp(iterator->GetData()->GetKey(), key) == 0)
		return iterator->GetData()->GetContent();
	  else iterator = iterator->_next;
  return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: _findNode
// Parameters: (const char*)
//
//  This function searches for node with the given key. It is different
//  from CIEnv::find function in the sense that it returns the pointer
//  to the node rather than a pointer of the content.
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvNodeType *CIEnv::_findNode(const char *key)
{
  CIEnvNodeType *iterator = _head;

  if (iterator)
	while (iterator)
	  if (strcmp(iterator->GetData()->GetKey(), key) == 0)
		return iterator;
	  else iterator = iterator->_next;
  return NULL;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (CIEnvValueType*)
//
// Inserts a given node value into the stack.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnv::insert(CIEnvValueType * data)
{
  if ((_head == NULL) && (_tail == NULL))
	{
	  _head = new CIEnvNodeType(data);
	  _tail = _head;
	}
  else if (_head == _tail)
	{
	  _head = new CIEnvNodeType(data);
	  _head->_next = _tail;
	  _tail->_pre = _head;
	}
  else
	{
	  CIEnvNodeType * tmp = new CIEnvNodeType(data);
	  _head->_pre = tmp;
	  tmp->_next = _head;
	  _head = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: erase
// Parameters: (const char*)
//
// Find a node with a given key then delete it from the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnv::erase(const char * key)
{
  CIEnvNodeType * targetNode = _findNode(key);
  if (!targetNode) return;
  if ((!_head) && (!_tail))
	{
	  cout << "Error @ CIEnv : erase" << endl;
	  cout << "trying to delete an entry from empty list error" << endl;
	  exit(1);
	}
  else if ((targetNode == _head) && (_head == _tail))
	{
	  _head = NULL;
	  _tail = NULL;
	  delete targetNode;
	  targetNode = NULL;
	}
  else if ((targetNode == _tail) && (_head != _tail))
	{
	  (targetNode->_pre)->_next = NULL;
	  _tail = targetNode->_pre;
	  delete targetNode;
	  targetNode = NULL;
	}
  else if ((targetNode == _head) && (_head != _tail))
	{
	  (targetNode->_next)->_pre = NULL;
	  _head = targetNode->_next;
	  delete targetNode;
	  targetNode = NULL;
	}
  else
	{
	  CIEnvNodeType * tmp = targetNode->_pre;
	  tmp->_next = targetNode->_next;
	  (targetNode->_next)->_pre = tmp;
	  delete targetNode;
	  targetNode = NULL;
	}
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: modify
// Parameters: (char*, CIdent*)
//
// Modifies a given node value in the stack.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CIEnv::modify(char* key, CIdent* data)
{
  CIEnvNodeType *temp = _findNode(key);
  if (temp) {
	temp->SetData(data);
  }
}

