//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Kee-Eung Kim (kek)                                             
//  FILE: PCMulStmtDis.C
//  PROJ: COKO
//  DATE: 8/27/96                                                              
//
// SYNOPSIS
//  Method definitions for CMulStmtDis class.
//  
//////////////////////////////////////////////////////////////////////////////
#include <assert.h>
#include "PCState.H"
#include "PCStmt.H"
#include "PCMulStmtDis.H"


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CMulStmtDis
// Parameters:	(CStmt *first, CStmt *second)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CMulStmtDis::CMulStmtDis(CStmt *first, CStmt *second)
{
  _first = first;
  _second = second;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CMulStmtDis
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CMulStmtDis::~CMulStmtDis(void)
{
  if(_first) { delete _first; _first = NULL; }
  if(_second) { delete _second; _second = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters:	(CState*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CMulStmtDis::Exec(CState *s)
{

if (_debug_)
  cout << "-- MulStmtDis Begins" << endl;

  // First, execute the former statements.
  s = _first->Exec(s);

  if(s->ISuccess() == F) {
	// If the all of the statements failed, do the next statement.
	s = _second->Exec(s);
  }

if (_debug_)
  cout << "-- MulStmtDis Ends" << endl;

  return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters:	(CState*, DDebugController*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CMulStmtDis::ExecDebug(CState *s, DDebugController *db)
{

  if (db->really_execute()) {
    // First, execute the former statements.
    s = _first->ExecDebug(s, db);
    
    bool temp = s->ISuccess();
    if (db->check_for_break(key_number))
      db->doBreak(this, s);
    
    if(temp == F) {
      // If the all of the statements failed, do the next statement.
      s = _second->ExecDebug(s, db);
    }
    
    db->check_for_condition(s, rep_string(), key_number);
    
  }
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CMulStmtDis::rep_string ()
{
  char *res = new char [strlen ("||") + 1];
  sprintf (res, "||");
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CMulStmtDis::representation (int indent)
{
  char* left;
  char* right;
  if (_first->is_leaf())
    left = _first -> representation (indent);
  else {
    char *left_orig = _first -> representation (indent);
    left = new char[strlen(left_orig) + strlen("{}") + 1];
    sprintf (left, "{%s}", left_orig);
    delete[] left_orig;
  }
  if (_second->is_leaf())
    right = _second -> representation (indent);
  else {
    char *right_orig = _second -> representation (indent);
    right = new char[strlen(right_orig) + strlen("{}") + 1];
    sprintf (right, "{%s}", right_orig);
    delete[] right_orig;
  }
  char *rep = rep_string();
  char *res = new char [strlen (left) + strlen (right) + strlen ("  ") + strlen (rep) + 1];
  sprintf (res, "%s %s %s", left, rep, right);
  delete [] left;
  delete [] right;
  delete [] rep;
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CMulStmtDis::representationWDI (int indent, DDebugController *db)
{
  int currLineOrig = -1;
  int currColOrig = -1;
  int addedBraces = 0;
  char* left;
  char* right;
  if (_first->is_leaf())
    left = _first -> representationWDI (indent, db);
  else {
    currLineOrig = db->getLine();
    currColOrig = db->getCol();
    char *left_orig = _first -> representationWDI (indent, db);
    left = new char[strlen(left_orig) + strlen("{}") + 1];
    sprintf (left, "{%s}", left_orig);
    delete[] left_orig;
    addedBraces = 1;
  }
  char *rep = rep_string();
  int currLine = db->getLine();
  int currCol = db->getCol();
  if (addedBraces) {
    db->updateLine(currLineOrig, currColOrig);
    if (currLineOrig == currLine)
      currCol+=2;
    else
      currCol++;
    addedBraces = 0;
  }
  db->updateArray(key_number-1, currLine, currCol, currLine, currCol+strlen(rep)+2);
  if (_second->is_leaf())
    right = _second -> representationWDI (indent, db);
  else {
    db->setCol(currCol+strlen(rep)+3);
    char *right_orig = _second -> representationWDI (indent, db);
    right = new char[strlen(right_orig) + strlen("{}") + 1];
    sprintf (right, "{%s}", right_orig);
    delete[] right_orig;
    currCol = db->getCol();
    db->setCol(currCol+1);
  }
  char *res = new char [strlen (left) + strlen (right) + strlen ("  ") + strlen (rep) + 1];
  sprintf (res, "%s %s %s", left, rep, right);
  delete [] left;
  delete [] right;
  delete [] rep;
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Uses start parameter to assign key_number data_member, calls number_self 
// on any sub_statements and returns current key-number
//         								    
//////////////////////////////////////////////////////////////////////////////

int CMulStmtDis::number_self (int start)
{
  int temp = _first -> number_self(start);
  key_number=temp;
  return (_second -> number_self(temp+1));
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: output_self
// Parameters:	(DDebugController *)
//
// ADDED DJA SUMMER 2001
// Outputs current statement in list format (for the debugger) if it is a 
// breakable statement; otherwise just calls output_self on any sub-statements
//         								    
//////////////////////////////////////////////////////////////////////////////

void CMulStmtDis::output_self (DDebugController *db)
{
  _first -> output_self(db);
  char* rep = rep_string();
  db->format_output(key_number, rep);
  delete[] rep;
  _second -> output_self(db);
}

int CMulStmtDis::is_leaf()
//returns whether current statement is a leaf in the COKO tree
{
  return 0;
}
