//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Kee-Eung Kim (kek)                                             
//  FILE: PCMulStmtSeq.H
//  PROJ: COKO
//  DATE: 8/27/96                                                              
//
// SYNOPSIS
//  Class definition for CMulStmtSeq class.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _PCMULSTMTSEQ_H_
#define _PCMULSTMTSEQ_H_

#include <PCStmt.H>

//////////////////////////////////////////////////////////////////////////////
//
//  CLASS: CMulStmtSeq
//
// SYNOPSIS
//  This class is a derived class from CStmt. It is used for multiple 
//  statements connected by sequential execution semantics in COKO rule block.
//
//////////////////////////////////////////////////////////////////////////////

class CMulStmtSeq : public CStmt
{
protected:
  CStmt *_first;                  // Former statement(s)
  CStmt *_second;                 // Later statement
public:
  CMulStmtSeq(CStmt *, CStmt *);  // Constructor
  ~CMulStmtSeq(void);             // Destructor
  CState *Exec(CState *);         // What to do when executed


  //DJA SUMMER 2001:
  //Added rep_string, representation, number_self, output_self, is_leaf and ExecDebug methods
  
  char* rep_string ();
   //returns the string representation of only the current 
  //node of the COKO tree
  char* representation (int);
  //returns string representation of COKO tree
  char* representationWDI(int, DDebugController *);
  //returns string representation of COKO tree
  //but also update the debugger with the location (row and
  //column) of each breakable statement's substring
  CState *ExecDebug(CState *, DDebugController *);
  //what to do when executed in debug mode
  int number_self(int);
  //assigns an integer to the current node for use in the
  //debugger for the breakOptions command
  void output_self(DDebugController *db);
  //output key_number and rep_string for debugger
  //for breakOptions command
  int is_leaf();
  //returns whether current statement is a leaf in the COKO tree

};

#endif
