//////////////////////////////////////////////////////////////////////////////
//                                                                         
//  NAME: Joon S. Lee (jl), Kee-Eung Kim (kek)  
//  FILE: PCState.C
//  PROJ: COKO
//  DATE: 8/27/96                                                              
//
// SYNOPSIS
//  Methods definitions for CState class.
//  
//////////////////////////////////////////////////////////////////////////////

#include "PCState.H"
#include "PCPattern.H"
#ifdef DEBUG
#include <KONonType.H>
#endif

#define TRUE 1
#define FALSE 0



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  CState
// Parameters:	(NonType*, CIEnvStack*, int)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState::CState(NonType *store, CIEnvStack *ienv, int success)
{
  _Store = store;
  _rootStore = store; //added by DJA in FALL 2001
  _IEnv = ienv;
  _Success = success;
  _CurrPattern = NULL;
  _InvDEPList = new DEPList();
  
  if(_debug_)
	{
	  if (_IEnv && _IEnv->top()) {		// if... else added by mra
		cout << _IEnv->top() << endl;
	  }
	  else {
		cout << "_IEnv->top() = NULL" << endl;
	  }
	}



  if(_debug_)
	{
	  cout << "% constructor : CState " << endl;
	  cout << "_Store = " << _Store << " _Success = " << success << endl;
	}

}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~CState
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState::~CState()
{
  if(_Store) { delete _Store; _Store = NULL; }
  //if(_rootStore) {delete _rootStore; _rootStore = NULL; } //Added by DJA in FALL 2001
  if(_IEnv) { delete _IEnv; _IEnv = NULL; }
  if(_CurrPattern) { delete _CurrPattern; _CurrPattern = NULL; }
  if(_InvDEPList) { delete _InvDEPList; _InvDEPList = NULL; }
}

// Get Methods


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Store
// Parameters:	(void)
//
// returns _Store
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType *CState::Store(void)
{
  if (_Store)
	return _Store;
  else
	{
	  cout << "Error @ CState : Store" << endl;
	  cout << "Store is null error" << endl;
	  exit(1);
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  rootStore
// Parameters:	(void)
//
// returns _rootStore
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType *CState::rootStore(void)
{
  if (_rootStore)
	return _rootStore;
  else
	{
	  cout << "Error @ CState : rootStore" << endl;
	  cout << "rootStore is null error" << endl;
	  exit(1);
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  IEnv
// Parameters:	(void)
//
// returns _IEnv
//         								    
//////////////////////////////////////////////////////////////////////////////

CIEnvStack * CState::IEnv(void)
{
  return _IEnv;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Success
// Parameters:	(void)
//
// returns _Success
//         								    
//////////////////////////////////////////////////////////////////////////////

int CState::ISuccess(void)
{
  return _Success;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  CurrentPattern
// Parameters:	(void)
//
// returns _CurrPattern
//         								    
//////////////////////////////////////////////////////////////////////////////

CPattern *CState::CurrentPattern(void)
{
  return _CurrPattern;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: GetPatternFromHere 
// Parameters:	(const char*, CIEnvStackDataType*)
//
// First search the stack with a given key.
//
// Start searching from the startingPt.
//
// Searching from the starting point instead of the top of the stack makes code
// more efficient.
//
// Return a pattern stored in the found node
//         								    
//////////////////////////////////////////////////////////////////////////////

CPattern* CState::GetPatternFromHere(const char* key, CIEnvStackDataType* startingPt)
{
  if (_IEnv->top() == NULL)
	{
	  cout << "Error @ CState : setPatternFromHere" << endl;
	  cout << "stack is empty error" << endl;
	  exit(1);
	  return NULL;
	}
  else
	return _IEnv->findNodeFromHere(key, startingPt)->GetPattern();
}


// Set Methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  SetStore
// Parameters:	(NonType*)
//
// Modify _Store.
//
// By deleting old _Store before setting _Store to new NonType*, prevent
// possible memory leaks.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CState::SetStore(NonType *pc)
{
  if (_Store) { delete _Store; _Store = NULL; } //added on Feb 15
  _Store = pc;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  SetRootStore
// Parameters:	(NonType*)
//
// Modify _rootStore.
//
// By deleting old _rootStore before setting _rootStore to new NonType*,
// prevent
// possible memory leaks.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CState::SetRootStore(NonType *pc)
{
  //if (_rootStore) { delete _rootStore; _rootStore = NULL; } //added on Feb 15
  _rootStore = pc;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SetIEnv
// Parameters: (CIEnvStack*)
//
// Modify _IEnv.
//
//////////////////////////////////////////////////////////////////////////////

void CState::SetIEnv(CIEnvStack* st)
{
  _IEnv = st;
}
  
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  SetSuccess
// Parameters:	(int)
//
// Modify _Success.
//
//////////////////////////////////////////////////////////////////////////////

void CState::SetSuccess(int success)
{
  _Success = success;

  if(_debug_)
	{
	  cout << "% SetSuccess : CState " << endl;
	  cout << "_Success = " << _Success << endl;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  SetCurrentPattern
// Parameters:	(CPattern*)
//
// Modify _CurrPattern
// 
// Do not delete old _CurrPattern before setting _CurrPattern to new one.
// No memory leak will occur. Old _CurrPattern will be deleted somewhere else.
//
//////////////////////////////////////////////////////////////////////////////

void CState::SetCurrentPattern(CPattern *currPattern)
{
  _CurrPattern = currPattern;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  SetCurrentPattern
// Parameters:	(CPattern*)
//
// First search the stack with a given key.
// 
// Modify a pattern stored in the found node.
// 
//////////////////////////////////////////////////////////////////////////////

void CState::SetPatternDeep(const char * key, CPattern * p)
{
  if (_IEnv->top() == NULL)
	{
	  cout << "Error @ CState : setPatternDeep" << endl;
	  cout << "stack is empty error" << endl;
	  exit(1);
	}
  else
	_IEnv->findNodeDeep(key)->SetPattern(p);
}


// stack methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IInsert
// Parameters:	(Char*, CIdent*)
//
// Identifier inserter (inserts into
// the top frame of the stack.
// 
//////////////////////////////////////////////////////////////////////////////

int CState::IInsert(const char * key, CIdent * content)
{
  if (!_IEnv->top())
	_IEnv->push( new CIEnv() );

  _IEnv->top()->insert( new CIEnvValueType(key, content) );

  return T;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IDeleteDeep
// Parameters: (Char*)	
//
// Delete identifier from the stack
// Search for the entire stack.
// 
//////////////////////////////////////////////////////////////////////////////

void CState::IDeleteDeep(char * key)
{
  if (!_IEnv->top()) 
	{
	  cout << "Error @ CState : IDeleteDeep" << endl;
	  cout << "stack is empty error" << endl;
	  exit(1);
	}
  else 
	_IEnv->deleteNodeDeep(key);
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IFind
// Parameters: (char*)	
//
// Search only for the top frame (another stack type) of the stack
// 
//////////////////////////////////////////////////////////////////////////////

CIdent * CState::IFind(const char * key)
{
  if(!_IEnv->top()) 
	return NULL;
  else 
	return _IEnv->top()->find(key);
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IFindDeep
// Parameters:  (char*)	
//
// Search for the entire stack.
// 
//////////////////////////////////////////////////////////////////////////////

CIdent * CState::IFindDeep(char * key)
{
  if(!_IEnv->top())
	return NULL;
  else {
    return _IEnv->findNodeDeep(key)->GetData()->GetContent();
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IFindDeepDB
// Parameters:  (char*)	
//
// Search the entire stack for input key.
// Unlike above method, checks to see if findNodeDeepDB returns null
// ADDED BY DJA WINTER 2002
// 
//////////////////////////////////////////////////////////////////////////////

CIdent * CState::IFindDeepDB(char * key)
{
  if(!_IEnv->top())
	return NULL;
  else {
    CIEnvNodeType* temp = _IEnv->findNodeDeepDB(key);
    if (temp)
      return temp->GetData()->GetContent();
    else
      return NULL;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IFindFromHere
// Parameters: (char*, CIEnvStackDataType*)
//
// Search for entire stack starting from the indicated point to down.
// 
//////////////////////////////////////////////////////////////////////////////

CIdent * CState::IFindFromHere(char * key, CIEnvStackDataType* stPt)
{
  CIEnvStackDataType* startingPt = stPt;

  if (!_IEnv->top())
	return NULL;
  else 
	return _IEnv->findNodeFromHere(key,startingPt)->GetData()->GetContent();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: INew
// Parameters: (void)
//
// Pushes an empty frame (another stack type) into the stack
// 
//////////////////////////////////////////////////////////////////////////////

void CState::INew(void)
{
  CIEnv * temp = new CIEnv();
  _IEnv->push(temp);

  if(_debug_)
	{
	  cout << "% INew : CState" << endl;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IOld
// Parameters: (void)
//
// Pop up the top element of the stack
// 
//////////////////////////////////////////////////////////////////////////////

void CState::IOld(void)
{
  if(_debug_)
	cout << "_IEnv :" << _IEnv << endl;

  _IEnv->pop();

  if(_debug_)
	cout << "% IOld : CState" << endl;

}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IFirst
// Parameters: (void)
//
// Returns the root node value of 
// identifier search tree.
// 
//////////////////////////////////////////////////////////////////////////////

CIEnvValueType * CState::IFirst(void)
{
  if (!_IEnv->top())
	return NULL;
  else
	return _IEnv->top()->first();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IModify
// Parameters: (char*, NonType*, int)
//
// Modifier for a particular identifier
// 
//////////////////////////////////////////////////////////////////////////////

void CState::IModify(char *var, NonType *c, int w)
{
  if (!_IEnv->top())
	{
	  cout << "Error @ CState : IModify" << endl;
	  cout << "top is NULL error" << endl;
	  exit(1);
	}
  else
	_IEnv->top()->modify(var, new CIdent(c, w));
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IModifyFromHere
// Parameters:	(char*, NonType*, int, CIEnvStackDataType*)
//
// same as IModify but this one search the entire stack  
// starting from the indicated point to down.
// 
//////////////////////////////////////////////////////////////////////////////

void CState::IModifyFromHere(char *var, NonType *c,
					 int w, CIEnvStackDataType* startingPt)
{
  if (_debug_)
	  cout << "CState::IModifyFromHere starts" << endl;

  if (!_IEnv->top())
	{
	  cout << "Error @ CState : IModifyFromHere" << endl;
	  cout << "top is NULL error" << endl;
	  exit(1);
	}
  else 
	_IEnv->findNodeFromHere(var,startingPt)->SetData( new CIdent(c,w) );
  
  if (_debug_)
	cout << "CState::IModifyFromHere ends" << endl;

}

// dependency list methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IInsertDep
// Parameters: (const char* key, char* a) 
//
// find a node with key then insert a to its dependency list
// 
//////////////////////////////////////////////////////////////////////////////

int CState::IInsertDep(const char * key, char * a) 
{
  if (_IEnv->top() == NULL)
	{
	  cout << "Error @ CState : IInsertDep" << endl;
	  cout << "Stack is empty error" << endl;
	  exit(1);
	  return 0;
	}
  else
	return _IEnv->findNodeDeep(key)->insertDepList(a);
}

// inverse dependency list methods

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IInsertInvDep
// Parameters: (const char*, char*) 
//
// find a node with key then insert a to its inverse dependency list
// 
//////////////////////////////////////////////////////////////////////////////

int CState::IInsertInvDep(const char * key, char * a) 
{
  if (!_IEnv->top())
	{
	  cout << "Error @ CState : IInsertInvDep" << endl;
	  cout << "Stack is empty error" << endl;
	  exit(1);
	  return 0;
	}
  else
	{
	  return _IEnv->findNodeFromHere(key, _IEnv->getData()->_next)->insertInvDepList(a);
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IDeleteInvDep
// Parameters: (const char*)
//
// Find a node with key then retrieve all the key values 
// in its inverse dependency list. With all the retrieved key values, 
// search and delete all the entries from the stack.
//
// DJA WINTER 2002: I do not think that this method does what it intends
// It leaves much of the inverse dependancy list intact
// 
//////////////////////////////////////////////////////////////////////////////

void CState::IDeleteInvDep(const char * key)
{
  CIEnvNodeType * tmpNode = _IEnv->findNodeDeep(key);
  
  if (tmpNode)
	{
	  char* tmpchar = tmpNode->popInvDepList();
	  if (tmpchar) IDeleteInvDep(tmpchar);
	  while (tmpchar)
			{
			  IDeleteDeep(tmpchar);
			  delete [] tmpchar;
			  tmpchar = NULL;
			  tmpchar = tmpNode->popInvDepList();
			  if (tmpchar) IDeleteInvDep(tmpchar);
			  //delete [] tmpchar;
			  //tmpchar = NULL; //DJA WINTER 2002: while loop always ends here - I think this is a mistake
			}
	  if (tmpchar) {
	    delete [] tmpchar;
	    tmpchar = NULL;
	  }
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: DelInvDep
// Parameters: (void)
//
// Delete _InvDEPList.
//         								    
//////////////////////////////////////////////////////////////////////////////
void CState::DelInvDep(void)
{
  if (_InvDEPList) delete _InvDEPList;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CCleanInvDep
// Parameters: (void)
//
// Clean _InvDEPList. Make _InvDEPList as it was first created.
// To do so, just delete old _InvDEPList and assign new DEPList to
// _InvDEPList.
// 
//////////////////////////////////////////////////////////////////////////////

void CState::CCleanInvDep(void)
{
  if (_InvDEPList) { delete _InvDEPList; _InvDEPList = new DEPList(); }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CDeleteInvDep
// Parameters: (void)
//
// Retrieve all the key values from _InvDEPList.
//
// With all the retrieved key values, search and delete 
// all the entries from the stack.
// DJA WINTER 2002: Found bug in this method (only deletes the top value)
// 
//////////////////////////////////////////////////////////////////////////////

void CState::CDeleteInvDep(void)
{
  if (_IEnv->top())
	{
	  char* tmpchar = _InvDEPList->pop();
	  if (tmpchar) IDeleteInvDep(tmpchar);
	  while (tmpchar)
		{
		  IDeleteDeep(tmpchar);
		  delete [] tmpchar;
		  tmpchar = NULL;
		  tmpchar = _InvDEPList->pop();
		  if (tmpchar) IDeleteInvDep(tmpchar);
		}
	  if (tmpchar) {
	    delete [] tmpchar;
	    tmpchar = NULL;
	  }
	}
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: IEmptyInvDep
// Parameters: (const char*)	
//
// First search the stack with a given key.
// Is a inverse dependancy list stored in the found node empty?
//
//////////////////////////////////////////////////////////////////////////////

int CState::IEmptyInvDep(const char * key)
{
  if (!_IEnv->top())
	{
	  cout << "Warning @ CState : IEmptyInvDep" << endl;
	  cout << "Stack is empty warning" << endl;
	  return 0;
	}
  else
	return _IEnv->findNodeDeep(key)->emptyInvDepList();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CEmptyInvDep
// Parameters: (void)	
//
// is _InvDEPList empty?
//
//////////////////////////////////////////////////////////////////////////////

int CState::CEmptyInvDep()
{
  if (_InvDEPList)
	{
	  if (_debug_)
		{
		  cout << "current inverse dependency list is empty" << endl;
		  cout << _InvDEPList->empty() << endl;
		}
	return _InvDEPList->empty();
	}
  else
	{
	  cout << "Warning @ CState : CEmptyInvDep" << endl;
	  cout << "inverse dependency list for current is not initialized" << endl;
	  return 1;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CInsertInvDep 
// Parameters:	(Char*)
//
// Insert a char* entry into the _InvDEPList
// 
//////////////////////////////////////////////////////////////////////////////

void CState::CInsertInvDep(char* name)
{
  if (_InvDEPList)
	_InvDEPList->insert(name);
  else
	{
	  cout << "Error @ CState : CInsertInvDep" << endl;
	  cout << "inverse dependency list for current is not initialized error" << endl;
	  exit(1);
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: GetInvDep
// Parameters: (void)
//
// Returns _InvDEPList
// 
//////////////////////////////////////////////////////////////////////////////

DEPList* CState::GetInvDep() 
{ 
  return _InvDEPList; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: SetInvDep
// Parameters: (DEPList*)	
//
// Modify _InvDEPList
//
//////////////////////////////////////////////////////////////////////////////

void CState::SetInvDep(DEPList* a) 
{
  _InvDEPList = a; 
}




  
