//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi
//  FILE: DEDebugController.H
//  PROJ: COKO Debugger
//  DATE: SUMMER 2001
//
// SYNOPSIS
//  Class definitions for DDebugController.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _DEDEBUGCONTROLLER_H_
#define _DEDEBUGCONTROLLER_H_

//////////////////////////////////////////////////////////////////////////////
//
//  CLASS: DDebugController
//
// SYNOPSIS
//   Fill in later
//
//////////////////////////////////////////////////////////////////////////////

#include<string>
#include<sstream>
#include "LCSub.H"
#include "RuleBlockList.H"
#include "UsesList.H"
#include "DEHistoryList.H"
#include "PCStmt.H"
#include "PCEqn.H"
//#include <qapplication.h>
#include "VC.H"


class CEqn;
class MyMainWindow;
class DSymbolVisList;
class DTransformationList;
class DDebugController {

protected:
  
  //short _breakArray[30];           
  // bit array to indicate where debugger should break
  int currentlyRunning;
  static const int maxParameters = 3;
  int breakStep;
  int breakNext;
  int displayCurrTree;
  int displayRootTree;
  int displayCounter;
  int rule_on;
  int success_on;
  int pattern_on;
  int restart;
  int visual;
  int readyToGo;
  int currLine;
  int currCol;
  int currNumStmts;
  int histCounter;
  int varCounter;
  int zoomCounter;
  NonType * toExec;
  char* execName;
  char* execKey;
  LCSubstring* currLCArray;
  CRuleBlock* _root;
  CRuleBlock* _current;  
  RuleBlockList* _curr_rbList;
  RuleBlockList* _break_rbList;
  SymbolList* _symList;
  HistoryList* _histList;
  MyMainWindow* vis;
  //QMultiLineEdit* console;
  QApplication *a;
  ostringstream ost;
  CStmt* tempStmt;
  DSymbolVisList *_svl;
  DTransformationList* _transList;

public:

  DDebugController(int, int, char**);
  // Constructor

  ~DDebugController();                            
  // Destructor

  void doBreak(CStmt*, CState*);
  // This method is called from any COKO statement class that inherits from
  // CStmt. It is called if its previous call to check_for_break returned true.
  // If so, then it passes along a pointer to itself (stmt) and a pointer to
  // the current state (s). The key number of the COKO statment inside the
  // current transformation and the rep_string of the statement are found from
  // stmt and this information, along with s, is passed along to the breakLoop
  // method which takes control and allows the user to indicate his/her wishes
  // while the program execution has paused.

  void doBreak(CEqn*, CState*);
  // This method is called from any COKO statement class that inherits from
  // CEqn. It is called if its previous call to check_for_break returned true.
  // If so, then it passes along a pointer to itself (eqn) and a pointer to
  // the current state (s). The key number of the equation inside the current
  // transformation and the rep_string of the equation are found from eqn and 
  // this information, along with s, is passed along to the breakLoop method
  // which takes control and allows the user to indicate his/her wishes while
  // the program execution has paused.
  
  void start(CRuleBlock*);
  // This method is called to start the debugging process on the root
  // transformation. Two options are possible: this is the first time through
  // this transformation (the user just started up Visual COKO or Command Line
  // COKO) or this is not the first time through this transfomration (the user
  // restarted the debugger). In the former case, the current transformation is
  // added to the _curr and _break_rbList and a welcome line is printed to the
  // screen. In the latter case, the same thing occurs but the _break_rbList
  // is not reinitialized (so that the breaks that the user placed on various
  // statements will be remembered even though execution got restarted).
  
  void stop();
  // This method is called when the debugger is finished executing the root
  // transformation. This root transfomration (rule block) is simply popped off
  // the _curr_rbList stack.
  
  void format_output(char*, char*);
  // This method outputs to the textual display window the two parameters
  // separated by a '|' character.
  
  void format_output(int, char*);
  // This method outputs to the textual display window the two parameters
  // separated by a '|' character.
  
  int check_for_break(int);
  // This method checks to see if the debugger should break and take control
  // before executing the next COKO statement. It is called at the beginning of
  // every ExecDebug method of each COKO statement class. This method takes an 
  // int (key_number) and finds if there is a break on that key (statement of 
  // COKO code) in the current transformation (the transformation that is 
  // currently executing). Alternatively, it checks if the debugger should
  // break for other reasons (such as if the user typed "step" or next". If any
  // of these reasons are valid, it returns true indicating that the debugger
  // should break. Otherwise it returns false.
  
  NonType* setToExec(NonType*, char*, char*);
  // This method sets the global variable toExec when a COKO transformation
  // is about to call a sub-transformation on a branch of the tree (toExec
  // stands for to execute). toExec points to this branch of the tree. In
  // addition, the variable num of the root of this branch is set to be
  // negative so that an arrow can be drawn in the visual window to indicate
  // that a subroutine is about to be called. This branch with the new variable
  // num is returned. In addition, the global variable "execName" is set so
  // that the visual window can use this as the label of the arrow pointing to
  // the tree branch.
  
  NonType* resetOldVariableNum(NonType*, char*, CState*);
  // This method takes a NonType (pointer to a KOLA tree) and resets all of the
  // variable-nums of each node in the tree back to being a positive number (ie
  // if that node was marked (the mark is done by giving the varibale-num of
  // that node a negative number) as the node upon which a sub-transformation
  // was about to be called, then we now remove this "mark"). This method is
  // currently called from the PCRuleInvokeStmt class after the debugger has
  // returned control to this class and just before the transformation call
  // actually occurs. The char* key paramater is the name of the variable that
  // is bound to the branch of the tree that the sub-transformation is being
  // called upon. For this method, I only care whether or not key is NULL. If
  // it is not NULL, then s->Store has to be rebuilt to reflect these changes
  // in variableNum - otherwise the change was only to the root and the tree
  // does not have to be rebuilt. The function returns back the toExec tree
  // with the updated variableNums.
  
  void check_for_condition(CState*, char*, int);
  // This method is called from the COKO statement classes after they are
  // finished to check if the success flag is set. If it is, then the success
  // variable in the CState parameter s checked and printed to the screen. In
  // addition, if the success value was true, this method checks to see if
  // there was a conditional break set on that statement (in which case a
  // global variable is set saying that the debugger should stop as soon as it
  // can).
  
  int ruleOn();
  // This 1-line method simply returns the value of the rule_on flag.
  
  //void checkAndDisplaySuccess();
  
  int really_execute();
  // This method is called from every ExecDebug method in every COKO statement
  // to make sure that it should really go ahead with the execution. The reason
  // why this check has to be made is that if the user wants to restart the
  // program, no more execution should occur until the the current execution
  // unwinds from all its recursive calls and the while loop and begin again.
  
  int do_restart();
  // Once the outer while loop is ready to restart the COKO program (ready to
  // begin execution again, do_restart will be called to reinitialze the
  // restart variable to 0). But, before restarting, it checks to see if it
  // really should restart by checking the value of the restart boolean.
  
  void setReadyToGo();
  // This method sets the readyToGo member variable to equal 1.
  
  void repInitialize(int, int, int);
  // This method takes textual display information about the current
  // transformation including the number of lines (line) columns (col) and
  // statements (num_stmts) and sets corresponding member variables to equal
  // these parameter values. In addition, this method initializes an array
  // (currLCArray) of size num_stmts that will eventually contain the line and
  // column number of the beginning and end of each statement in the textual
  // represenation of the current transformation.
  
  void updateArray(int, int, int, int, int);
  // This method is called from the representationWDI mehtod of various COKO
  // breakable statement classes. This method will update the currLCArray with
  // input statment number (kn) the exact location (in terms of characters)
  // inside the QT textarea window of this statement (using the start line
  // (sL), end line (eL), start column (sC) and end column (eC)). This method
  // also updates where we are in the text area using the end line and end
  // column.
  
  void updateLine(int, int);
  // This method is called if CMulStmtCon or CMulStmtDis added a '{' to the
  // textual representation. If that is the case then every statement on that
  // line must be moved over by one character to account for this '{'.
  
  int getLine();
  // Returns the current line number in the text area that is being used to
  // display the contents of the current transformation.
  
  void setLine(int);
  // Sets the current line number in the text area that is being used to
  // display the contents of the current transformation.
  
  int getCol();
  // Returns the current column number in the text area that is being used to
  // display the contents of the current transformation.

  void setCol(int);
  // Sets the current column number in the text area that is being used to
  // display the contents of the current transformation.
  
  void outputDBArray();
  // This method is for debugging purposed only. It displays to cout the
  // current contents of currLCArray and currLine and currCol.
  
  void parseInput(const char*, char**, int);
  // This method is called by the command line version of the debugger to pasre
  // the char* charArrayToParse input string into an array of char* with each
  // element of the array pointing to 1 word (space delimited)
  
  void printInRightPlace(char*);
  // This method prints whatever is stored in the Char* parameter (toPrint) in
  // the correct location. If visual mode is turned on, then this text is
  // printed to the console window. Otherwise, it is printed directly to the
  // screen (cout).
  
  int findCurrent(CRuleBlock* actuallyFindThisOne=NULL, char* transName=NULL);
  // This is an important method that goes through the break rule block list
  // (this list maintains all of the transformations that have been called so
  // far and any break information that the debugger might need to know about
  // each transformation) and by default finds the one that is being currently
  // run (the top element in the current rule clock stack). Alternatively, if
  // a char* transformation name is given (transName) it will find that
  // transformation (and not the current one) or if a pointer to a RuleBlock
  // is given it will get the name of the RuleBlock by following the pointer
  // and calling the get_name() method and will find that transformation.
  // The key assumption here is that transformation name is a key. This method
  // returns a boolean value indicating whether it found that transformation in
  // the break list and additionally will result in the iterator of the global
  // variable _break_rbList pointing to the approporate element.
  
  void testRunAgain();
  // Once the COKO program has finished executing, this method will be called
  // to check if the user wants to run the program again, or just finish.
  
  void histPush(NonType*, int, int, char*, CIEnvStack*, NonType*);
  // This method simply passes along all of its parameters to the push method
  // of member variable _histList (which is an instance of HistoryList and is
  // used to reconstruct the global KOLA tree from the current KOLA tree and
  // the environment stack stored in this _histList variable).
  
  void histPop();
  // This method simply calls the pop() method of the _histList member variable
  
  NonType* histUnravel(CState*);
  // This method takes a pointer to the current state (s) and unravels the 
  // histList stack from the current state to the root of the stack. Each
  // element in this histList stack is the current environment when a
  // transformation was called on a branch of the tree resulting in a new
  // environment to be initialized that contains only information pertaining
  // to this new "zoomed in" portion of the tree. Thus this method can be seen
  // as unraveling all of these environments to "zoom out" to get the original
  // KOLA tree as it now stands. Since changes may have been made in each
  // "zoomed in" frame, care must be take to make sure that this new version of
  // this branch of the KOLA tree replaces the old branch in the older
  // envirment in the stack frame above.
  //
  // This method is called when the user types: "display root" in the command
  // line debugger, and in the root tree display in the visual debugger
  
  void setTempStmt(CStmt*);
  // This method sets member variable tempStmt to CStmt* parameter stmt. This
  // is done because unfortunately, updateDEP was written so that it is a
  // member method of the CStmt class, and sometimes the debugger needs to call
  // this method and doesn't have an instance of a CStmt around to call it on.
  // Since updateDEP doesn't actually affect the stmt that it was called on, it
  // doesn't matter what statement is sent to this method - as long as it will
  // always be non-NUll while the debugger is running. Really, updateDEP should
  // have been declared static when it was originally written.
  
  LCSubstring* getCurrLCArray();
  // This method returns currLCArray (the variable that contains the location
  // of each statement in the text area that is displaying the current
  // transformation).
  
  int getNumStmts();
  // This method returns the number of breakable statements in the current
  // transformation.
  
  void updatePaintWindow(CState*);
  // This method tells the visual display window (vis) to update itself,
  // sending the variable symbol list (_symList), and the current tree
  // (s->Store()) which is found using the State parameter to this method (s).
  
private:
  
  void doPrintOST();
  // This method displays to the screen whatever is stored so far in the member
  // variable ost. If visual mode is turned on, then this text is printed to
  // the console window. Otherwise, it is printed directly to the screen
  // (cout).

  void breakLoop(int, char*, CState*);
  // This is the central and most important method in this class! If the visual
  // option is turned on, then this method gathers all of the current
  // information: the transformation code, the global tree, the current tree,
  // and all trees bound by variables, and sends this information to vis (the
  // visual coko display controller) which displays this information to the
  // user. In addition, this method sets the variableNums of all of the nodes
  // in these trees so that the tree drawer can display arrows pointing to all
  // bound variables and so that the current node can be painted green. If the
  // visual option is not turned on, then the only information displayed is the
  // current statement upon which the COKO transformation has stopped. Further
  // information must be explicitly requested by the user.
  //
  // Once the information has been display, the method loops while the user
  // types commands that do not return control back to program execution (such
  // as break x, display y, print z, etc) - performing those commands and
  // waiting for the next command. Otherwise (if the user typed step, next, or
  // continue) global variables are set and control is returned back to the
  // COKO program execution.

  void getInput(string*);
  // This method is called by the command line version of the debugger to get
  // a line that the user inputted.
  
  void parseInput(string*, char**, int);
  // This method is called by the command line version of the debugger to pasre
  // the char* charArrayToParse input string into an array of char* with each
  // element of the array pointing to 1 word (space delimited)
  
  void outputInput(string*, int);
  // This method is simply a debugging method used to make sure that I am
  // parsing the input correctly.
  
  void doUndefinedCommand(char*);
  // If the user typed a command that is not recognized by the debugger, this
  // method will be called to inform the user.
  
  void doBreakOptions();
  // While the command was renamed to "list", the method name wasn't. This
  // method is only called in the command line version of the debugger. It
  // displays the current transformation that is being debugged to the user and
  // beneath it displays the same transformation with numbers before all the
  // breakable statements. That way, it is easy for the user to select which
  // statement upon which to break.
  
  void doAddBreak(char* whereToBreak, int key_number, CRuleBlock* toAddBreak = NULL);
  // This method will add a break to the indicated key number. If the parameter
  // whereToBreak is NULL then the parameter key_number is the indication of
  // which statement to add the break. Otherwise the integer conversion from
  // the char* whereToBreak is used. The default transformation to add the
  // break is the current transformation. Otherwise, a RuleBlock* must be
  // passed to indicate which transformation to add the break.
  
  void doAddBreakSpecific(char*, char*, int);
  // This method will add a break to the indicated key number to the
  // transformation transName. If the parameter whereToBreak is NULL then the
  // parameter key_number is the indication of which statement to add the
  // break. Otherwise the integer conversion from the char* whereToBreak is
  // used. The parameter transName is expected to come in as "<transname>" so
  // the < and > have to be eliminated before searching for the transformation
  // in the transformation list _transList.
  
  void doAddConditionBreak(char*, int);
  // This method will add a conditional break to the indicated key number. If
  // the parameter whereToBreak is NULL then the parameter key_number is the
  // indication of which statement to add the break. Otherwise the integer
  // conversion from the char* whereToBreak is used.
  
  void doAddConditionBreakSpecific(char*, char*, int);
  // This method will add a condition break to the indicated key number to the
  // transformation transName. If the parameter whereToBreak is NULL then the
  // parameter key_number is the indication of which statement to add the
  // break. Otherwise the integer conversion from the char* whereToBreak is
  // used. The parameter transName is expected to come in as "<transname>" so
  // the < and > have to be eliminated before searching for the transformation
  // in the transformation list _transList.
  
  void doPrint(char*, CState *);
  // This method will print in textual form the KOLA tree depending on toPrint.
  // If toPrint is null or is current, this method will print to the screen the
  // textual version of the current KOLA tree. If toPrint is "root", then this
  // method will print the KOLA tree starting from the query root. Otherwise,
  // it will print the KOLA tree bound to the variable inidicated in doPrint.
  
  void doDisplay(char*, CState *);
  // This method will display in textual form (or if Visual COKO is running
  // then in visual form) a KOLA tree which is indicated by the toDisplay
  // parameter. If toDisplay is null or is current, this method will display
  // the textual (or visual) version of the current KOLA tree. If toPrint is
  // "root", then this method will display the KOLA tree starting from the
  // query root. Otherwise, it will print the KOLA tree bound to the variable
  // inidicated in doDisplay. The biggest difference between the doDisplay and
  // doPrint methods is the display means continually display it (don't just
  // print it once). For instance, if a variable is to be displayed, it should
  // be displayed every time the debugger stops until the variable is no longer
  // in context. This method accomplishes this by adding the variable name to a
  // display list which is checked every time the debgger stops. There is one
  // variable list in every rule block in _curr_rbList and accessed using the
  // topSym() method.
  
  void doUndisplay(char*, CState *);
  // This method turns of the KOLA tree indicated by the toDisplay paramater
  // that the doDisplay method turned on. If it was a variable that was being
  // displayed, it is removed from the current rule block list. Otherwise, 
  // the approporiate flag is turned off.
  
  void doClearBreak(char*, int);
  // This method will clear a break at the indicated key number. If the
  // parameter whereToClear is NULL then the parameter key_number is the
  // indication of which statement to add the break. Otherwise the integer
  // conversion from the char* whereToClear is used.
  
  void doClearBreakSpecific(char*, char*, int);
  // This method will clear a break at the indicated key number in the
  // transformation transName. If the parameter whereToClear is NULL then the
  // parameter key_number is the indication of which statement to clear the
  // break. Otherwise the integer conversion from the char* whereToClear is
  // used. The parameter transName is expected to come in as "<transname>" so
  // the < and > have to be eliminated before searching for the transformation
  // in the transformation list _transList.
  
  void doHelp(char*);
  // If the user typed the command: "help" this method will be called. It
  // simply displays a list of all the command the user can type. At some
  // point, this should be updated so that the user can type "help clear" or
  // "help print" and get more detailed instructions on those specific
  // commands.

  int analyzeInput(char**, int, CState*);
  // This method is called once the input is parsed to perform the appropriate
  // command. It checkes the first parameter (which in every case is the
  // command) and calls the appropriate method depending on the command. Some
  // commands don't need separate methods, so they are dealt with directly in
  // this method. An integer is returned that specifies whether the command
  // should return control back to the COKO execution (such as continue or
  // next) or if the user should retain control (as in the case of most other
  // commands). 0 means return control to the execution, and 1 means retain
  // control in the debugger.

  void outputNode(int, char*, CIdent*);
  // This method is used only in the command line version of the debugger. It
  // takes a CIdent (that is holding the value of a variable) and outputs this
  // variable (val) and its textual value along with its display number (num).
  
  void displayAndDeleteList(SymbolList*, CState*);
  // This method takes a symbol list (sym) and checks to see if each variable
  // in this list is currently bound in the current state (s). If it is, then
  // it is displayed. Otherwise, it is removed from the list. It also displays
  // the current and root trees of the appropriate flags are set to true
  
  void reaffirmAndDeleteList(CState* s);
  // This method is used only in Visual COKO. The _svl is a list of all windows
  // currently being displayed by the visual debugger besides the root,
  // current, and isolate tree display windows (leaving just windows that
  // display branches of the tree that are bound to variables). For each window
  // specified, this method looks up the value of the variable and sets the
  // approprate variable in the window so that the correct tree will be
  // displayed. In other words, this method "refreshes" all of the variable
  // display windows.
  
  void doSet(char*, char*);
  // If the user typed a command that started with the word "set", this method
  // will be called. There are three possible flags that the user can set:
  // set rule on/off, set pattern on/off, and set success on/off. Set rule on
  // will display to the screen information about rule firing every time COKO
  // attempts to fire a rule. Set pattern on right now does nothing. Set
  // success on will print to the screen the success value of every COKO
  // statement after that statement is finished executing. This method just
  // checks the value of the second parameter (par1) for "rule". "pattern", and
  // "success" (and prints an error if it is not one of those three) and sets
  // the appropriate flag to on or off depending on the thrid parameter (par2).

  void doStack();
  // If the user typed the command: "stack" this method is callled and will
  // call the displayStack in vis (if Visual COKO) or on the _curr_rbList
  // directly if the command line version is being run. These methods will
  // simply print to the screen in some way the stack of all the COKO
  // transformation calls up to the current COKO transformation.

  void doExit();
  // If the user typed the command: "exit" or "quit" this method is callled and
  // will ask the user if they really want to exit. If they do, a system exit
  // is called.

  int doRun();
  // If the user typed the command: "run" this method is called and will ask
  // the user if they really want to restart the program. If they do, the
  // necessary variables are reset, the restart flag is set to true, and the
  // program continues. Later on, the program will check the value of the
  // restart flag (using the really_execute method) to stop excecuting the
  // current run. 
  
  NonType* findNodeFromNumHelper(NonType*, int);
  // This method initializes the member variable histCounter to 0 and then
  // calls the findNodeFromNum method which finds the "num"th element in a
  // preorder traversal of the parameter: tree.

  NonType* findNodeFromNum(NonType*, int);
  // This method recurses through the KOLA tree parameter (tree) counting each
  // node that it visits. Once it has visited num nodes, it returns the tree
  // that is rooted at the current node.

  void visMarkVariables(CState*);
  // This method takes a State (s) and goes through the environment of that
  // state. For each variable that is found in the environment, it assigns a
  // variableNum to that node starting from 4, and then counting by twos
  // upwards (4, 6, 8, 10, etc). Before setting the variableNum of the NonType
  // (KOLA tree) stored in that variable, it checks to see what it used to be.
  // If it was negative (meaning that a COKO transformation is about to be
  // called on this node) this information is preserved by assigning it a new
  // variableNum that is -1 times what it would have otherwise given it (ie
  // instead of giving it 4, it gives it -4). Once the variableNum has been
  // changed for each KOLA tree pointed to by each variable, s->Store is
  // rebuilt to reflect these changes in variableNums (this is because
  // unfortunately, the COKO system was built without the debugger in mind, and
  // for this reason the KOLA trees that are pointed to by each variable in the
  // environment are only copies of the branches of the corresponding current
  // tree, so any changes made to these copies must also be made to the current
  // KOLA tree in s->Store).

  void scaleVariableNums(NonType*);
  // This method takes a KOLA tree input (currNT) and scales the variableNum
  // of each node in the tree to a number between [-2, 1]. In so doing, it
  // retains the sign of the variableNum (if it was negative before it will
  // remain negative and if it was positive before it will remain positive or
  // 0) and whether or not it was divisible by 2 (odd numbers remain odd, even
  // numbers remain even). Thus the mapping is:
  // negative and odd  --> -1
  // negative and even --> -2
  // positive and odd  --> 1
  // positive and even --> 0
  //
  // By scaling variableNums to be within [-2, 1] this method takes away
  // magnitude information from the variableNums where each variable number
  // corresponds to a variable in the environment. This magnitude information
  // is set in the method visMarkVariables, which is usually called after the
  // the variableNums have been scaled in this method.
  
  NonType* findCurr(NonType*);
  // This method looks through its KOLA tree parameter (currNT) and finds the
  // parent of the node whose variableNum is not even (which signifies that
  // that is the "current" node of the tree and should be drawn with the color
  // green on the visual display of that tree). It returns a pointer to this
  // parent.
  
  int findCurrWhich(NonType*);
  // This method looks through its KOLA tree parameter (currNT) and finds the
  // parent of the node whose variableNum is not even (which signifies that
  // that is the "current" node of the tree and should be drawn with the color
  // green on the visual display of that tree). After finding the parent of
  // this node, it returns which child of the parent this node is (ie it
  // returns 1 if it is the first child of the parent, 2 if it is the second
  // child, etc). This information is used in the breakLoop method to replace
  // this branch of the tree (from this special node downwards) with s->Store.
  // But in order to call the replaceChild method on the parent, the second
  // parameter needs to know which child - hence this method is called.
  
  void resetVariables();
  // This method is called whenever the debugger is restarted. Right now, all
  // it does is set tempStmt back to NULL, but my guess is that certain other
  // variables must be reset as well, and that is why the program crashes
  // sometimes when the user restarts debugger execution (by pressing the run
  // button). This method should be revisited.

  void doTrack(char*);
  // If the user typed the command: "track <transname>" this method is called
  // and will place the specified transformation on the tracking list. Once on
  // the tracking list, the user can place breaks on that transformation even
  // if it is not the current tranformation that is being executed.
  
};

#endif
