/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)	
//        Joon S. Lee (jl)				 	    
//  FILE: KOFConst.H 
//  PROJ: COKO, KOLA
//  DATE: 5/97	                                                    
//
// SYNOPSIS
//  Constant function former.  Given object o
//  we have
//  
//    K (o) ! e = o
//      
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_FCONST_HEADER
#define KOLA_FCONST_HEADER 

#include "KGlobals.H"
#include "KOObject.H"
#include "KOFunction.H"
#include "KONameUnion.H"
#include "KONonType.H"

#include "PCState.H"
#include "KOOVar.H"
#include "KOOInvoke.H"

#include <string.h>

class FConst : public Function
{
public:
  FConst (koObject* o, TFunc * ftype = NULL);
  //  Constructor.  o is object argument to former.

  int compares(NonType* nt, CState* a);
  // Checks for match between NonType and NonType.
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 

  int comparesSTM(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This match method is used for matching statement.
  // No duplicated variable entry is allowed.
  
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.
 
  void setDEPnInvDEP(char*, CState*);
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*);
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  NonType* transformDeep(CState* s, CIEnvStackDataType* startingPt);
  // difference between transform and 
  // transformDeep is that the former uses
  // IFind method and later uses IFindDeep 
  // method. 

  void replaceChild(NonType* child, int which_child);
  // Replace a child of given tree.

  FConst (const FConst& other);
  //  Copy Constructor. 
  
  ~FConst ();
  //  Destructor. 

  Function *Copy () const;
  // Virtual Copy Constructor
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped in side of a class NameUnion.

  int numChildren() const { return 1; }
  //  Number of arguments.

  koObject* One () const {return one;}
  //  What is the Object that is the 1st argument to
  //  the former?
  
  //  OResult* reduce (OResult* a);
  //  return
  //    one -> reduce ()
	
  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  "FFO: CONST (" + one -> representation () + ")" 

  const char * Name() const { return namestring; }
  // Text representation of the constant

  void SetOne(NonType* a) { delete one; one = (koObject*)a; }
  // replace existing child to new one.

  void initCurrChild() {currChild = 0;}
  // set (reset) currChild to 0.
  // This needs to be called before using nextChild() for first time.

  NonType* nextChild();
  // returns next child. (next Child = current Child + 1) 
  //If current child is the last child return NULL.

  koObject *inv (koObject *o);
  // Symbolic invocation of this on o.  Eventually,
  // this method will replace "invoke" presently
  // only defined with _OSTORE_ flag set

#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

#ifdef _OSTORE_
  koObject* invoke(koObject* x) { return new OInvoke(this,x); }

  OSObject* exec(OSObject* x) { return One()->result(); }
#endif // _OSTORE_
 
protected:
  koObject* one;
  //  What is the object that is the first argument
  //  to the former?

  int currChild;
};


# endif


