/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOFMinus.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOFMinus.H"
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FMinus
// Parameters:	()							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FMinus::FMinus (TFunc * ftype) : Function (ftype, FUN_MINUS,"SUB")
{    
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FMinus
// Parameters:	(FMinus& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FMinus::FMinus (const FMinus& other) : Function ((Function &) other)
{
  variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~FMinus
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

FMinus::~FMinus ()
{
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as Function*
//         								    
//////////////////////////////////////////////////////////////////////////////

Function* FMinus::Copy () const
{
  FMinus* temp = new FMinus (*this);
  temp->setVariableNum(variableNum);
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//
//////////////////////////////////////////////////////////////////////////////

int FMinus::deepeq (NonType * other)
{ 
  NameUnion* name_ = other->Myname();
  if ((name_->tag == FNAME ) && (name_->fn == FUN_MINUS))
  {
    delete name_;
    return T;
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* FMinus::Myname()
{
	NameUnion* a = new NameUnion(FNAME, FUN_MINUS);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//  return 
//    ((OResPAIR *) a) -> fst -> reduce ()
//
/////////////////////////////////////////////////////////////////////////////

// OResult* FMinus::reduce (OResult* a)
// {
//   if (a -> isPair ()) {
//     OResPAIR *a2 = ((OResPAIR *) a);
//     OResINF *res = new OResINF (a2 -> One () -> Copy (), "-", a2 -> Two () -> Copy ());
//     delete a;
//     return res;
//   } else {
//     OResBB *res = new OResBB ("MINUS", a -> Copy ());
//     delete a;
//     return res;
//   }
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	())							  
//
//  "FPRIM: _Minus"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* FMinus::representation ()
{
  char *res = new char [strlen ("MINUS") + 1];
  sprintf (res, "%s", "MINUS");
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding Nontype object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* FMinus::transform(CState* s)
{
  FMinus* temp = new FMinus(Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here.
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\
//                    |_\         / building part
//                      / matching part
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair
// into the env stack. Then the next time we face another g (second one),
// we search the stack for the matching pair with variable name g and simply check
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID).
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int FMinus::compares(NonType* nt, CState* a)
{
  NameUnion* tmp_name = nt->Myname();
  if ((tmp_name->tag == FNAME) && (tmp_name->fn == FUN_MINUS))
	{
	  delete tmp_name;
	  return T;
	}
  else
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void FMinus::replaceChild(NonType* child, int which_child)
{
}

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: invoke
//  Parameters:(koObject*)
//
//  Subtracts the second integer in the pair from the first.
//
//////////////////////////////////////////////////////////////////////////////

koObject* FMinus::invoke(koObject* x)
{
  return new OInvoke(this,x);
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:(OSObject*)
//
//  Subtracts the second integer in the pair from the first.
//
//////////////////////////////////////////////////////////////////////////////

OSObject* FMinus::exec(OSObject* x)
{
  return new OSInt(((OSInt*)(((OSPair*)x)->One()))->Val() -
                   ((OSInt*)(((OSPair*)x)->Two()))->Val());
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* FMinus::prolog ()
{
  char *res = new char [strlen ("fminus") + 1];
  sprintf (res, "fminus");
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref FMinus::sp_prolog ()
{
  SP_term_ref res = SP_new_term_ref();
  SP_put_string(res,"fminus");
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FMinus::sql (koObject *o)
{    
    NameUnion *nu;
    nu = o -> Myname ();
    if (nu -> on == OBJ_PAIR) {
	  delete nu;
	  char *arg1 = o -> asPair () -> One () -> sql ();
	  char *arg2 = o -> asPair () -> Two () -> sql ();
	  if (strncmp (arg1, "ERROR", 5) == 0)
		return arg1;
	  else if (strncmp (arg2, "ERROR", 5) == 0)
		return arg2;
	  else {
		char *res = new char [strlen ("( - )") + strlen (arg1) + strlen (arg2) + 1];
		sprintf (res, "(%s - %s)", arg1, arg2);
		delete [] arg1;
		delete [] arg2;
		return res;
	  }
	}  else {
	  delete nu;
	  cerr << "ERROR: Calling SUB on non-Pair Object" << endl;
	  return NULL;
	}
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FMinus::oql (koObject *o)
{    
    NameUnion *nu;
    nu = o -> Myname ();
    if (nu -> on == OBJ_PAIR) {
	  delete nu;
	  char *arg1 = o -> asPair () -> One () -> oql ();
	  char *arg2 = o -> asPair () -> Two () -> oql ();
	  if (strncmp (arg1, "ERROR", 5) == 0)
		return arg1;
	  else if (strncmp (arg2, "ERROR", 5) == 0)
		return arg2;
	  else {
		char *res = new char [strlen ("( - )") + strlen (arg1) + strlen (arg2) + 1];
		sprintf (res, "(%s - %s)", arg1, arg2);
		delete [] arg1;
		delete [] arg2;
		return res;
	  }
	}  else {
	  delete nu;
	  cerr << "ERROR: Calling SUB on non-Pair Object" << endl;
	  return NULL;
	}
}
