/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOODif.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOODif.H"
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ODif
// Parameters:	(koObject* o1, koObject *o2)							  
//
//  Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

ODif::ODif (koObject *o1, koObject *o2, TObj * t) : koObject (t,"-(SET)")
{
    one = o1;
    two = o2;
    variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ODif
// Parameters:	(ODif& other)							  
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

ODif::ODif (const ODif& other) : koObject ((koObject &) other)
{
    one = (other . One ()) -> Copy ();
    two = (other . Two ()) -> Copy ();
    variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~ODif
// Parameters:	()							  
//
//  Destructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

ODif::~ODif ()
{
    delete one;
    delete two;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy ()
// Parameters:	()							  
//
//  Virtual Copy Constructor 
//         								    
//////////////////////////////////////////////////////////////////////////////

koObject *ODif::Copy () const
{
  ODif* temp = new ODif (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//
//////////////////////////////////////////////////////////////////////////////

int ODif::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == ONAME ) && (name_->on == OBJ_DIF))
  {
    delete name_;
    return ((one->deepeq(((ODif*)other)->One())) 
         && (two->deepeq(((ODif*)other)->Two())));
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////
 
NameUnion* ODif::Myname()
{
        NameUnion* a = new NameUnion(ONAME, OBJ_DIF);
        return a;
}
 
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  nextChild
// Parameters:  ()
//
// Returns "currChild++"th child.
// When currChild is equal to the number of children
// (when current child is the last child), it returns NULL.
//
//////////////////////////////////////////////////////////////////////////////
 
NonType* ODif::nextChild()
{
  if (currChild == 0)
	{
	  currChild++;
	  return one;
	}
  else if (currChild == 1)
	{
	  currChild++;
	  return two;
	}
  else return NULL;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//  Return new OResINF 
//    (one -> reduce (), "-",  two -> reduce ())
//
/////////////////////////////////////////////////////////////////////////////

// OResult* ODif::reduce ()
// {
//     return new OResINF (one -> reduce (), "-", two -> reduce ());
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "FFO: "DIF: " + one -> representation () + ", " + two -> representation () 
//         								    
//////////////////////////////////////////////////////////////////////////////

char* ODif::representation ()
{
    char *orr = one -> representation ();
    char *tr = two -> representation ();
    char *res = new char [strlen (orr) + strlen (tr) + strlen ("( - )") + 1];
    sprintf (res, "(%s - %s)", orr, tr);
    delete [] orr;    
    delete [] tr;    
    return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* ODif::transform(CState* s)
{
  koObject* first = (koObject*) One() -> transform(s);
  koObject* second = (koObject*) Two() -> transform(s);
  ODif* temp = new ODif(first,second,Type());
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  transformDeep
// Parameters:  (CState*, CIEnvStackDataType*)
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly.
//  The difference between this method and transform method is a search
//  method used in VAR classes. (Take a look at transform and transformDeep
//  methods in BVar, FVar, PVar, or OVar.)
//  Transform method searches the top frame only,
//  while TransformDeep methods searches entire stack starting from
//  a given point.
//
//////////////////////////////////////////////////////////////////////////////

NonType* ODif::transformDeep(CState* s, CIEnvStackDataType* startingPt)
{
  koObject* first = (koObject*) One() -> transformDeep(s, startingPt);
  koObject* second = (koObject*) Two() -> transformDeep(s, startingPt);
  ODif* temp = new ODif(first,second,Type());
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here.
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\
//                    |_\         / building part
//                      / matching part
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair
// into the env stack. Then the next time we face another g (second one),
// we search the stack for the matching pair with variable name g and simply check
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID).
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int ODif::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();

  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_DIF))
	{
	  // do it for first child.
	  if (one->compares(((ODif*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one)) 
			// child (one) is a variable.
			{
			  if ((a->IFind(((OVar*)one)->Name())) == NULL)
				{ 
				  // child is a new variable.
				  a->IInsert(((OVar*)one)->Name(), new CIdent((ODif*)nt,((ODif*)nt)->One(),1));
				  // Push entry into the stack.  
				}
			  else 
				{
				  // Child is a variable but not new one.
				  // In this case, I need to make sure that what's 
				  // already in the stack and what I have now is the same.
				  if (!(((a->IFind(((OVar*)one)->Name()))->Child_Look_Up())->deepeq(((ODif*)nt)->One())))
					return_val++;
				  // match failed.
				  // else do none.
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++; 
	  // compare failed   
	 
	  // do it for second child.
	  if (two->compares(((ODif*)nt)->Two(), a))
		{
		  if (isChildVar(two))
			{
			  if ((a->IFind(((OVar*)two)->Name())) == NULL)
				{ 
				  a->IInsert(((OVar*)two)->Name(), new CIdent((ODif*)nt, ((ODif*)nt)->Two(), 2));
				}
			  else 
				{
				  if (!(((a->IFind(((OVar*)two)->Name()))->Child_Look_Up())->deepeq(((ODif*)nt)->Two())))
					return_val++;
				}
			}
		}
	  else return_val++;
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0. 
  // In this case, return false(0).
  // Otherwise, return true(1).

  delete tmp_name;

  if (return_val)
	return F;
  else
	return T;
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: comparesSTM
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for statement matching only.
// Statement matching is different from rule matching.
// No duplicated variable entry is allowed here.
//
// For example,
//             match f ! _O,
//                   f =  g o g  is a illegal statement (equation).
//                       -------
//                         |
//                         |_\
//                           / matching part
//
// Assume that we have a query tree of the form "ID o ID ! 3."
// Variable f gets bounded to "ID o ID" and we have to match "ID o ID" to "g o g."
// Matching "ID o ID" to "g o g" is perfectly legal in a rule section.
// However, it is illegal in statement section of the program.
// Remember that every variable on the right hand side of an equation must be a fresh one.
// However, after we match first g to ID, the second g is no longer fresh variable.
//
// "No duplication of a variable is legal in a statement section."
//
//////////////////////////////////////////////////////////////////////////////

int ODif::comparesSTM(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();

  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_DIF))
	{
	  // do it for first child.
	  if (one->comparesSTM(((ODif*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one)) 
			// child (one) is a variable.
			{
			  if ((a->IFind(((OVar*)one)->Name())) == NULL)
				{ 
				  // child is a new variable.
				  a->IInsert(((OVar*)one)->Name(), new CIdent((ODif*)nt,((ODif*)nt)->One(),1));
				  // Push entry into the stack.  
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((OVar*)one)->Name() << " is used twice. " << endl;
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++; 
	  // compare failed   
	 
	  // do it for second child.
	  if (two->comparesSTM(((ODif*)nt)->Two(), a))
		{
		  if (isChildVar(two))
			{
			  if ((a->IFind(((OVar*)two)->Name())) == NULL)
				{ 
				  a->IInsert(((OVar*)two)->Name(), new CIdent((ODif*)nt, ((ODif*)nt)->Two(), 2));
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((OVar*)two)->Name() << " is used twice. " << endl;
				}
			}
		}
	  else return_val++;
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0. 
  // In this case, return false(0).
  // Otherwise, return true(1).

  delete tmp_name;

  if (return_val)
	return F;
  else
	return T;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  setDEPnInvDEP
// Parameters:  (char*, CState*)
//
// set dependency list (ancestor list) and inverse dependency list (descendant list)
//
//////////////////////////////////////////////////////////////////////////////

void ODif::setDEPnInvDEP(char* vn, CState* s)
{
  one->setDEPnInvDEP(vn,s);
  two->setDEPnInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  undoSetInvDEP
// Parameters:  (char*, CState*)
//
// undo changes made on descendant list (inverse dependency list).
// undo for dependency list (ancestor list) is not necessary, since
// any changes on dependency list is undone by poping up
// top frame from the environment stack.
//
//////////////////////////////////////////////////////////////////////////////

void ODif::undoSetInvDEP(char* vn, CState* s)
{
  one->undoSetInvDEP(vn,s);
  two->undoSetInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void ODif::replaceChild(NonType* child, int which_child)
{
  if ( which_child == 1 )
	SetOne(child);
  else if ( which_child == 2 )
	SetTwo(child);
}

#ifdef _OSTORE_

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: result
//  Parameters:
//
//  Implements the set difference koObject
//
//////////////////////////////////////////////////////////////////////////////

OSObject* ODif::result()
{
  return new LSJoinSieve(new PCFA(new PNotEqual),new FID,
                         (Iterator*)One()->result(),
                         (Iterator*)Two()->result());
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* ODif::prolog ()
{

    char *orr = one -> prolog ();
    char *tr = two -> prolog ();
    char *res = new char [strlen (orr) + strlen (tr) + strlen ("odif(, )") + 1];
    sprintf (res, "odif(%s, %s)", orr, tr);
    delete [] orr;    
    delete [] tr;    
    return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref ODif::sp_prolog ()
{
  SP_term_ref or = one -> sp_prolog ();
  SP_term_ref tr = two -> sp_prolog ();
  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("odif"),2,or,tr);
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* ODif::sql ()
{    

  char *arg1 = one -> sql ();
  char *arg2 = two -> sql ();

  if (strncmp (arg1, "ERROR", 5) == 0) {
	delete [] arg2;
	return arg1;
  } else if (strncmp (arg2, "ERROR", 5) == 0) {
	delete [] arg1;
	return arg2;
  } else {
	char *res = new char [strlen ("( EXCEPT )") + strlen (arg1) + strlen (arg2) + 1];
	sprintf (res, "(%s EXCEPT %s)", arg1, arg2);
	delete [] arg1;
	delete [] arg2;
	return res;
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* ODif::oql ()
{    

  char *arg1 = one -> oql ();
  char *arg2 = two -> oql ();

  if (strncmp (arg1, "ERROR", 5) == 0) {
	delete [] arg2;
	return arg1;
  } else if (strncmp (arg2, "ERROR", 5) == 0) {
	delete [] arg1;
	return arg2;
  } else {
	char *res = new char [strlen ("( EXCEPT )") + strlen (arg1) + strlen (arg2) + 1];
	sprintf (res, "(%s EXCEPT %s)", arg1, arg2);
	delete [] arg1;
	delete [] arg2;
	return res;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  obj
// Parameters:	()
//
// Symbolic simplification of this.  Eventually,
// this method will replace "object" presently
// only defined with _OSTORE_ flag set
//  
//////////////////////////////////////////////////////////////////////////////

koObject* ODif::obj ()
{
  return new ODif (one -> obj (), two -> obj ());
}

