/*
 * Copyright 1994, Brown University Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOOIf.C
// PROJ: COKO Compiler, KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOOIf.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OIf
// Parameters:	(koBool* b1, koBool* b2, TBool* t)							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OIf::OIf (koBool *b, koObject *o1, koObject *o2, TObj * t) : koObject (t,"IF")
{
  one = b;
  two = o1;
  three = o2;
  variableNum = 0;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OIf
// Parameters:	(const OIf&)
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OIf::OIf (const OIf& other) : koObject ((koObject &) other)
{
  one = (other . One ()) -> Copy ();
  two = (other . Two ()) -> Copy ();
  three = (other . Three ()) -> Copy ();
  variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OIf
// Parameters:	(koBool* b1, koBool* b2)							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OIf::~OIf ()
{
  delete one;
  delete two;
  delete three;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  Return copy of this as Bool
//         								    
//////////////////////////////////////////////////////////////////////////////

koObject *OIf::Copy () const
{
  OIf* temp = new OIf (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  deepeq
// Parameters:  ()
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//
//////////////////////////////////////////////////////////////////////////////
 
int OIf::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == ONAME ) && (name_->on == OBJ_IF))
  {
    delete name_;
    return ((one->deepeq(((OIf*)other)->One())) 
         && (two->deepeq(((OIf*)other)->Two()))
		 && (three->deepeq(((OIf*)other)->Three())));
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* OIf::Myname()
{
        NameUnion* a = new NameUnion(ONAME, OBJ_IF);
        return a;
}
 
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  nextChild
// Parameters:  ()
//
// Returns "currChild++"th child.
// When currChild is equal to the number of children
// (when current child is the last child), it returns NULL.
//
//////////////////////////////////////////////////////////////////////////////

NonType* OIf::nextChild()
{
  if (currChild == 0)
	{
	  currChild++;
	  return one;
	}
  else if (currChild == 1)
	{
	  currChild++;
	  return two;
	}
  else if (currChild == 2)
	{
	  currChild++;
	  return two;
	}
  else return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//
/////////////////////////////////////////////////////////////////////////////

// OResult* OIf::reduce ()
// {
//   return new OResIF (one -> reduce (), two -> reduce (), three -> reduce ());
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation 
// Parameters:	()							  
//
/////////////////////////////////////////////////////////////////////////////

char* OIf::representation ()
{
  char *orr = one -> representation ();
  char *tr = two -> representation ();
  char *thr = three -> representation ();
  char *res = new char [strlen (orr) + strlen (tr) + strlen (thr) + strlen ("(IF () THEN  ELSE  FI)") + 1];
  sprintf (res, "(IF (%s) THEN %s ELSE %s FI)", orr, tr, thr);
  delete [] orr;    
  delete [] tr;    
  delete [] thr;    
  return res;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* OIf::transform(CState* s)
{
  koBool* first = (koBool*) One() -> transform(s);
  koObject* second = (koObject*) Two() -> transform(s);
  koObject* third = (koObject*) Three() -> transform(s);
  OIf* temp = new OIf(first,second,third,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  transformDeep
// Parameters:  (CState*, CIEnvStackDataType*)
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly.
//  The difference between this method and transform method is a search
//  method used in VAR classes. (Take a look at transform and transformDeep
//  methods in BVar, FVar, PVar, or OVar.)
//  Transform method searches the top frame only,
//  while TransformDeep methods searches entire stack starting from
//  a given point.
//
//////////////////////////////////////////////////////////////////////////////

NonType* OIf::transformDeep(CState* s, CIEnvStackDataType* startingPt)
{
  koBool* first = (koBool*) One() -> transformDeep(s, startingPt);
  koObject* second = (koObject*) Two() -> transformDeep(s, startingPt);
  koObject* third = (koObject*) Three() -> transformDeep(s, startingPt);
  OIf* temp = new OIf(first,second,third,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here.
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\
//                    |_\         / building part
//                      / matching part
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair
// into the env stack. Then the next time we face another g (second one),
// we search the stack for the matching pair with variable name g and simply check
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID).
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int OIf::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();

  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_IF))
	{
	  // do it for first child.
	  if (one->compares(((OIf*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one)) 
			// child (one) is a variable.
			{
			  if ((a->IFind(((BVar*)one)->Name())) == NULL)
				{ 
				  // child is a new variable.
				  a->IInsert(((BVar*)one)->Name(), new CIdent((OIf*)nt,((OIf*)nt)->One(),1));
				  // Push entry into the stack.  
				}
			  else 
				{
				  // Child is a variable but not new one.
				  // In this case, I need to make sure that what's 
				  // already in the stack and what I have now is the same.
				  if (!(((a->IFind(((BVar*)one)->Name()))->Child_Look_Up())->deepeq(((OIf*)nt)->One())))
					return_val++;
				  // match failed.
				  // else do none.
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++; 
	  // compare failed   
	 
	  // do it for second child.
	  if (two->compares(((OIf*)nt)->Two(), a))
		{
		  if (isChildVar(two))
			{
			  if ((a->IFind(((OVar*)two)->Name())) == NULL)
				{ 
				  a->IInsert(((OVar*)two)->Name(), new CIdent((OIf*)nt, ((OIf*)nt)->Two(), 2));
				}
			  else 
				{
				  if (!(((a->IFind(((OVar*)two)->Name()))->Child_Look_Up())->deepeq(((OIf*)nt)->Two())))
					return_val++;
				}
			}
		}
	  else return_val++;

	  // do it for third child.
	  if (three->compares(((OIf*)nt)->Three(), a))
		{
		  if (isChildVar(three))
			{
			  if ((a->IFind(((OVar*)three)->Name())) == NULL)
				{ 
				  a->IInsert(((OVar*)three)->Name(), new CIdent((OIf*)nt, ((OIf*)nt)->Three(), 3));
				}
			  else 
				{
				  if (!(((a->IFind(((OVar*)three)->Name()))->Child_Look_Up())->deepeq(((OIf*)nt)->Three())))
					return_val++;
				}
			}
		}
	  else return_val++;

	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0. 
  // In this case, return false(0).
  // Otherwise, return true(1).

  delete tmp_name;

  if (return_val)
	return F;
  else
	return T;
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: comparesSTM
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for statement matching only.
// Statement matching is different from rule matching.
// No duplicated variable entry is allowed here.
//
// For example,
//             match f ! _O,
//                   f =  g o g  is a illegal statement (equation).
//                       -------
//                         |
//                         |_\
//                           / matching part
//
// Assume that we have a query tree of the form "ID o ID ! 3."
// Variable f gets bounded to "ID o ID" and we have to match "ID o ID" to "g o g."
// Matching "ID o ID" to "g o g" is perfectly legal in a rule section.
// However, it is illegal in statement section of the program.
// Remember that every variable on the right hand side of an equation must be a fresh one.
// However, after we match first g to ID, the second g is no longer fresh variable.
//
// "No duplication of a variable is legal in a statement section."
//
//////////////////////////////////////////////////////////////////////////////

int OIf::comparesSTM(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();

  if ((tmp_name->tag == ONAME) && (tmp_name->on == OBJ_IF))
	{
	  // do it for first child.
	  if (one->comparesSTM(((OIf*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one)) 
			// child (one) is a variable.
			{
			  if ((a->IFind(((BVar*)one)->Name())) == NULL)
				{ 
				  // child is a new variable.
				  a->IInsert(((BVar*)one)->Name(), new CIdent((OIf*)nt,((OIf*)nt)->One(),1));
				  // Push entry into the stack.  
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((BVar*)one)->Name() << " is used twice. " << endl;
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++; 
	  // compare failed   
	 
	  // do it for second child.
	  if (two->comparesSTM(((OIf*)nt)->Two(), a))
		{
		  if (isChildVar(two))
			{
			  if ((a->IFind(((OVar*)two)->Name())) == NULL)
				{ 
				  a->IInsert(((OVar*)two)->Name(), new CIdent((OIf*)nt, ((OIf*)nt)->Two(), 2));
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((OVar*)two)->Name() << " is used twice. " << endl;
				}
			}
		}
	  else return_val++;

	  // do it for third child.
	  if (three->comparesSTM(((OIf*)nt)->Three(), a))
		{
		  if (isChildVar(three))
			{
			  if ((a->IFind(((OVar*)three)->Name())) == NULL)
				{ 
				  a->IInsert(((OVar*)three)->Name(), new CIdent((OIf*)nt, ((OIf*)nt)->Three(), 3));
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((OVar*)three)->Name() << " is used twice. " << endl;
				}
			}
		}
	  else return_val++;

	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0. 
  // In this case, return false(0).
  // Otherwise, return true(1).

  delete tmp_name;

  if (return_val)
	return F;
  else
	return T;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  setDEPnInvDEP
// Parameters:  (char*, CState*)
//
// set dependency list (ancestor list) and inverse dependency list (descendant list)
//
//////////////////////////////////////////////////////////////////////////////

void OIf::setDEPnInvDEP(char* vn, CState* s)
{
  one->setDEPnInvDEP(vn,s);
  two->setDEPnInvDEP(vn,s);
  three->setDEPnInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  undoSetInvDEP
// Parameters:  (char*, CState*)
//
// undo changes made on descendant list (inverse dependency list).
// undo for dependency list (ancestor list) is not necessary, since
// any changes on dependency list is undone by poping up
// top frame from the environment stack.
//
//////////////////////////////////////////////////////////////////////////////

void OIf::undoSetInvDEP(char* vn, CState* s)
{
  one->undoSetInvDEP(vn,s);
  two->undoSetInvDEP(vn,s);
  three->undoSetInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void OIf::replaceChild(NonType* child, int which_child)
{
  if ( which_child == 1 )
	SetOne(child);
  else if ( which_child == 2 )
	SetTwo(child);
  else if ( which_child == 3 )
	SetThree(child);
}

#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: object
//  Parameters:
//
//  Implements the koObject if former.
//
//////////////////////////////////////////////////////////////////////////////

koObject* OIf::object()
{
  return new OIf(One()->bool(), Two()->object(),Three()->object());
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: result
//  Parameters:
//
//  Implements the OSObject if former.
//
//////////////////////////////////////////////////////////////////////////////

OSObject* OIf::result()
{
  if (((OSBPrimitive*)One()->result())->Val())
    return Two()->result();
  else
    return Three()->result();
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* OIf::prolog ()
{
  char *orr = one -> prolog ();
  char *tr = two -> prolog ();
  char *thr = three -> prolog ();
  char *res = new char [strlen (orr) + strlen (tr) + strlen (thr) + strlen ("oif(, , )") + 1];
  sprintf (res, "oif(%s, %s, %s)", orr, tr, thr);
  delete [] orr;    
  delete [] tr;    
  delete [] thr;    
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref OIf::sp_prolog ()
{
  SP_term_ref or = one -> sp_prolog ();
  SP_term_ref tr = two -> sp_prolog ();
  SP_term_ref thr = three -> sp_prolog ();
  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("oif"),3,or,tr,thr);
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	(koObject *o)
//
// Return the SQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* OIf::sql ()
{    
    char *arg1 = one -> sql ();
    char *arg2 = two -> sql ();
    char *arg3 = three -> sql ();
    if (strncmp (arg1, "ERROR", 5) == 0) {
	  delete [] arg2;
	  delete [] arg3;
      return arg1;
    } else if (strncmp (arg2, "ERROR", 5) == 0) {
	  delete [] arg1;
	  delete [] arg3;
      return arg2;
    } else if (strncmp (arg3, "ERROR", 5) == 0) {
	  delete [] arg2;
	  delete [] arg1;
      return arg3;
    } else {
      char *res = new char [strlen ("(if () then  else )") + strlen (arg1) + strlen (arg2) + strlen (arg3) + 1];
      sprintf (res, "(if (%s) then %s else %s)", arg1, arg2, arg3);
      delete [] arg1;
      delete [] arg2;
      delete [] arg3;
	  return res;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* OIf::oql ()
{    
    char *arg1 = one -> oql ();
    char *arg2 = two -> oql ();
    char *arg3 = three -> oql ();
    if (strncmp (arg1, "ERROR", 5) == 0) {
	  delete [] arg2;
	  delete [] arg3;
      return arg1;
    } else if (strncmp (arg2, "ERROR", 5) == 0) {
	  delete [] arg1;
	  delete [] arg3;
      return arg2;
    } else if (strncmp (arg3, "ERROR", 5) == 0) {
	  delete [] arg2;
	  delete [] arg1;
      return arg3;
    } else {
      char *res = new char [strlen ("(if () then  else )") + strlen (arg1) + strlen (arg2) + strlen (arg3) + 1];
      sprintf (res, "(if (%s) then %s else %s)", arg1, arg2, arg3);
      delete [] arg1;
      delete [] arg2;
      delete [] arg3;
	  return res;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  obj
// Parameters:	()
//
// Symbolic simplification of this.  Eventually,
// this method will replace "object" presently
// only defined with _OSTORE_ flag set
//  
//////////////////////////////////////////////////////////////////////////////

koObject* OIf::obj ()
{
  return new OIf (one -> boo (), two -> obj (), three -> obj ());
}

