/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOObjList.C
// PROJ: COKO Compiler, KOLA Parcer
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOObjList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ObjList
// Parameters:	()
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

ObjList::ObjList ()
{
  first = NULL;
  last = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ObjList
// Parameters:	(const ObjList&)
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

ObjList::ObjList (const ObjList & other)
{
  if (other . First () == NULL) {
    first = NULL;
    last = NULL;
  } else if ((other . First ()) -> Next () == NULL) {
    first = new OLink (*(other . First ()));
    last = first;
  } else {
    first = new OLink (*(other . First ()));
    OLink *ptr = first -> Next ();
    OLink *pptr = first;
    while (ptr != NULL) {
      pptr = ptr;
      ptr = ptr -> Next ();
    };
    last = pptr;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~ObjList
// Parameters:	()
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

ObjList::~ObjList ()
{
  delete first;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  insert
// Parameters:	(koObject*)
//
//         								    
//////////////////////////////////////////////////////////////////////////////

ObjList *ObjList::insert (koObject* o)
{
  OLink *ol = new OLink (o);
  if (first == NULL) {
    first = ol;
    last = ol;
  } else {
    last -> Next (ol);
    last = ol;
  };

  return this;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  listeq
// Parameters:	(ObjList*)
//
//         								    
//////////////////////////////////////////////////////////////////////////////

int ObjList::listeq(ObjList* otherList)
{
  OLink* tmpLink;
  OLink* tmpOtherLink;
  tmpLink = first;
  tmpOtherLink = otherList->First();

  // basic cases
  if ((tmpLink == NULL) && (tmpOtherLink == NULL))
	return T;
  else if  ((tmpLink == NULL) || (tmpOtherLink == NULL))
	return F;

  // is every element in this list is also in the list "otherList"?
  while (tmpLink != NULL)
	{
	  int isEqual = F;
	  tmpOtherLink = otherList->First();
	  while ((tmpOtherLink != NULL) && (!isEqual))
		{
		  if ((tmpLink->Obj())->deepeq(tmpOtherLink->Obj()))
			isEqual = T;
		  tmpOtherLink = tmpOtherLink->Next();
		}
	  if (!isEqual)
		return F;
	  tmpLink = tmpLink->Next();
	}

  // is every element in the list "otherList" is also in this list?
  tmpLink = first;
  tmpOtherLink = otherList->First();
  while (tmpOtherLink != NULL)
	{
	  int isEqual = F;
      tmpLink = first;
	  while ((tmpLink != NULL) && (!isEqual))
		{
		  if ((tmpOtherLink->Obj())->deepeq(tmpLink->Obj()))
			isEqual = T;
		  tmpLink = tmpLink->Next();
		}
	  if (!isEqual)
		return F;
	  tmpOtherLink = tmpOtherLink->Next();
	}
  return T;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//  Return 
//    new OResChain (first, last)
//
/////////////////////////////////////////////////////////////////////////////

// OResChain* ObjList::reduce ()
// {
//   return new OResChain (first);
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation 
// Parameters:	()							  
//  res = ""
//  ptr = fst
//  while ptr != last
//    res = res + ptr -> representation () + ", "
//    ptr = ptr -> Next ()
//  if ptr != NULL
//    return res
//  else
//    res = res + ptr -> representation ()
//    return res
//
/////////////////////////////////////////////////////////////////////////////

char* ObjList::representation ()
{
  char *oldres;
  char *res;
    
  OLink *ptr = first;
  if (ptr != NULL) {
    char *orr = ptr -> representation ();
    res = new char [strlen (orr) + 1];
    sprintf (res, "%s", orr);
    delete [] orr;
    ptr = ptr -> Next ();

    while (ptr != NULL) {
      char *orr = ptr -> representation ();
      oldres = res;
      res = new char [strlen (oldres) + strlen (orr) + strlen (", ") + 1];
      sprintf (res, "%s, %s", oldres, orr);
      delete [] orr;
      delete [] oldres;
      ptr = ptr -> Next ();
    };
  } else {
    res = new char [1];
    sprintf (res, "");
  };

  return res;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* ObjList::prolog ()
{
  char *oldres;
  char *res;
    
  OLink *ptr = first;
  if (ptr != NULL) {
    char *orr = ptr -> prolog ();
    res = new char [strlen (orr) + 1];
    sprintf (res, "%s", orr);
    delete [] orr;
    ptr = ptr -> Next ();

    while (ptr != NULL) {
      char *orr = ptr -> prolog ();
      oldres = res;
      res = new char [strlen (oldres) + strlen (orr) + strlen (", ") + 1];
      sprintf (res, "%s, %s", oldres, orr);
      delete [] orr;
      delete [] oldres;
      ptr = ptr -> Next ();
    };
  } else {
    res = new char [1];
    sprintf (res, "");
  };

  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref ObjList::sp_prolog ()
{
  // fix this!
  return NULL;
}
#endif // _PROLOG_


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* ObjList::sql ()
{
  if (first == NULL) {
    char *res = new char [1];
	sprintf (res, "");
	return res;
  } else {
	char *arg1 = first -> sql ();
    if (strncmp (arg1, "ERROR", 5) == 0) {
	  return arg1;
	} else {
	  char *res = new char [strlen (arg1) + 1];
	  sprintf (res, "%s", arg1);
	  delete [] arg1;
	  return res;
	}
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* ObjList::oql ()
{
  if (first == NULL) {
    char *res = new char [1];
	sprintf (res, "");
	return res;
  } else {
	char *arg1 = first -> oql ();
    if (strncmp (arg1, "ERROR", 5) == 0) {
	  return arg1;
	} else {
	  char *res = new char [strlen (arg1) + 1];
	  sprintf (res, "%s", arg1);
	  delete [] arg1;
	  return res;
	}
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  obj
// Parameters:	()
//
// Symbolic simplification of this.  Eventually,
// this method will replace "object" presently
// only defined with _OSTORE_ flag set
//  
//////////////////////////////////////////////////////////////////////////////

ObjList* ObjList::objlist ()
{
  if (first == NULL) 
    return new ObjList ();
  else {
	return first -> objlist (new ObjList ());
  }
}

