/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Xiaoyu Wang (wangxy)
//  FILE: KOTAttr.H 
//  PROJ: COKO, KOLA
//  DATE: 9/00		                                                    
//
// SYNOPSIS
//	AttrTag: (@ OStr) 
//
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_ATTR_HEADER
#define KOLA_ATTR_HEADER 

#include "KGlobals.H"
#include "KOTag.H"
#include "KOOStr.H"
#include "KONameUnion.H"
#include "KONonType.H"

#include "PCState.H"
#include <string.h>

class  TAttr: public Tag
{
public:
  TAttr (OStr* s);
  // Constructor.

  int compares(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 
  
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.
  
  void setDEPnInvDEP(char*, CState*) {}
  // set dependency list and inverse dependency list
  
  void undoSetInvDEP(char*, CState*) {}
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.
  
  TAttr (const TAttr& other);
  // Copy Constructor.
  
  ~TAttr ();
  // Destructor.
  
  TAttr* Copy () const;
  // Virtual copy constructor.
    
  int deepeq(NonType * other);
  // Value-based equality
  
  NameUnion* Myname();
  // returns myname wrapped inside of a class NameUnion.

  int numChildren() const { return 1; }
  //  Number of arguments.

  OStr* One() const { return one; }
  
  int compareSTM(NonType* other, CState* s);

  void replaceChild(NonType* child, int which_child);
 
  void setOne(NonType* str){ delete one; one = (OStr *)str; }
  // replace existing child to a new one.
  
#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

  char* prolog ();
  // return string representation in prolog term

  char* representation ();

  void initCurrChild() { currChild = 0;}
  //  Set current child to 0, call it before 1st time call nextChild 
  
  NonType* nextChild();
  // Get the next child, of current child is the last one, return NULL; 
 
  const char * Name() const { return namestring; }
  // Returns the text representation of the tagexp.

#ifdef _OSTORE_
  Tag* tag() { return NULL; }
  // Returns the kola equivalent of an object.

  OSObject *result() { return NULL; }
  // Returns the OSObject equivalent of itself.

#endif // _OSTORE_

protected:
  OStr* one;
  int currChild;
};

#endif
