
/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Xiaoyu Wang(wangxy@cs.brandeis.edu)
// FILE: KOTagExpLink.C
// PROJ: COKO Compiler, KOLA Parcer
// DATE: 10/00
//
//////////////////////////////////////////////////////////////////////////////

#include "KOTagExpLink.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TagExpLink
// Parameters:	(TagExp* )							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

TagExpLink::TagExpLink (TagExp* te)
{
  _tagexp = te;
  next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TagExpLink
// Parameters:	(const TagExpLink&)
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

TagExpLink::TagExpLink (const TagExpLink & other)
{
  _tagexp = other.tagexp()->Copy();
  if (other.Next() == NULL)
    next = NULL;
  else
    next = new TagExpLink (*(other.Next()));
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~TagExpLink
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

TagExpLink::~TagExpLink ()
{
  delete _tagexp;
  delete next;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation 
// Parameters:	()
//  return tagexp -> representation ()
//
/////////////////////////////////////////////////////////////////////////////

char* TagExpLink::representation ()
{
  char *tr = _tagexp -> representation ();
  char *res = new char [strlen (tr) + 1];
  sprintf (res, "%s", tr);
  delete [] tr;
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* TagExpLink::prolog ()
{
  char *tr = _tagexp -> prolog ();
  char *res = new char [strlen (tr) + 1];
  sprintf (res, "%s", tr);
  delete [] tr;
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref TagExpLink::sp_prolog ()
{
  return _tagexp->sp_prolog();
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* TagExpLink::sql ()
{
  char *res = new char[strlen("SQL")+1];
  sprintf(res, "SQL");
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* TagExpLink::oql ()
{
  char *res = new char[strlen("OQL")+1];
  sprintf(res, "OQL");
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  tagexp
// Parameters:	()
//
// Symbolic simplification of this, Eventually, this method will replace
// "object" presently only defined  with _OSTORE_ flag set 
//
//////////////////////////////////////////////////////////////////////////////

TagExpList* TagExpLink::tagexplist (TagExpList *tl)
{
  TagExpList *tmp = tl->insert(_tagexp->tagexp());
  if( next == NULL) 
    return tmp;
  else
    return next->tagexplist(tmp);
}
