/* output from Ox version G1.04 */
%{
%}
/*
 * Copyright 1998, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
*/


/*
//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc@cs.brown.edu)
//  FILE: QL.y
//  PROJ: QLite -> KOLA translator (Ox Input File)
//  DATE: 7/98		                                                    
//
//////////////////////////////////////////////////////////////////////////////
*/
/*  Overview:
  
    This Translator translates queries posed in the language, Query Slim,
    into KOLA.  Query Slim is a superset of the query language, 
    Query Lite, described in:

      http://w3.rchland.ibm.com/~rbaxter/gbofQuery/qryLiteIO/qryLiteIO.html

    Unlike Query Lite, Query Slim permits:

      1.  Greater Expressivity in the SELECT clause

          Query Lite permits only a single identifier token
          in the SELECT clause of a query (corresponding to the identifier
          identifying individual elements in the collection 
          named in the query's FROM clause.  Therefore, Query Lite
		  queries are restricted to "selection queries".  Query Slim
          permits ANY Query Expression to be used in the SELECT
          clause.  This includes:

		  * path expressions (thereby permitting "projection queries" and
             not just "selection queries" to be posed)

		  * arithmetic expressions

          * other queries!! (thus permitting the construction of nested
		     collections)

     2.  Greater Expressivity in the FROM clause

          Query Lite only permits a single named collection to appear
		  in the FROM clause of a query, again limiting posed
		  queries to "selection queries" on named collections.  Query
		  Slim permits:

		  * unbounded numbers of collections to be named in the FROM
		    clause (thus allowing for "join queries")

		  * collections appearing in the FROM clause to be derived
		    rather than globally named.  Thus, such collections might
			be collection-valued attributes or method results (from
			the use of a path expression) or the results of other queries.

     3.  Greater Expressivity in the WHERE clause

          Query Lite only permits predicate expressions to be 
          conjunctions or disjunctions of simple predicates 
          (which compare path expression values to constants).
          Query Slim also permits expression of predicates
		  involving other queries, as in:

		  * membership predicates (e.g.: x.m1()...mn() IN SELECT ...)
		  * quantification predicates (e.g.:  FORALL x IN (SELECT ...): P (X)

    4.  Other additional queries including:

	   a.  Duplicate Eliminating Queries (SELECT DISTINCT vs. SELECT)

	   
    As yet, Query Slim does not include support for:

     1.  Constants of types 
	      short, byte, long, double, float, Dinteger, Dshort, Dlong, Dfloat, DDouble, char, DCharacter, DBooleab

     2.  Simple Predicate Operations: >, <, >=, <= and LIKE on Strings

     3.  Simple Predicate Operations: =, != on Booleans
	      (instead of x.foo() = FALSE, one would write: "NOT (x.foo())")
     
   I am as yet unclear as to the difference between "." and ".." as method
   invocations, and therefore treat them as the same thing.  Does the latter
   identify methods corresponding to table attributes?

   For the time being, I am assuming that all posed queries are type correct as I do no type-checking.
   Queries that are not type correct (e.g., 3 + "hello") may produce
   unpredictable results and may even result in segmentation faults.
   This will be corrected at a later time.

   I am also assuming that the input query passed to the translator has
   done the necessary conversion of variable references into their 
   values.  (i.e., select x from coll x where x.m1() = "+ abc +" such
   that abc is a variable would not be accepted.  Instead, it would
   be assumed that + abc + has been replaced by the value of abc at the time
   of query invocation.

   I am assuming that all variables appearing in the query are global if
   not declared as part of the query (e.g., sel and C1 in 

      "select x from C1 x where sel.compare (x)"

   To simplify parsing, I am requiring sort flags (ASC or DESC) to be declared in queries

*/
/* Idea Behind Translation:  Every Query is Translated into a function on 
    the empty environment, NULL.  An example translation of query q (T [q]) is
    shown below
  
     select struct (one: x, two: y)
     from x in Sens, y in Caps
     where x.reps.cap = y.name
  
   = select struct (one: V_1, two: V_0)
     from Sens, Caps
     where V_1.reps.cap = V_0.name
  
   would get initially translated into:
  
     iterate (eq OPLUS <cap o reps o snd o fst, name o snd>, <snd o fst, snd>) o 
      flat o 
       iterate (K (t), iter (K (t), id) o <id, K (Caps)>) o 
        flat o 
         iterate (K (t), iter (K (t), id) o <id, K (Sens)>) o 
          singleton ! *

   while

     select struct (one: x, two: y, three: z)
     from x in Sens, y in Regs, z in Cities
     where x.reps = y.name AND y.cap = z.name
  
   = select struct (one: V_2, two: V_1, three: V_0)
     from Sens, Regs, Caps
     where V_2.reps = V_1.name AND V_1.cap = V_0.name
  
   would get initially translated into:

     iterate ((eq OPLUS <reps o snd o fst o fst, name o snd o fst>) & (eq OPLUS <cap o snd o fst, name o snd>), (<<snd o fst o fst, snd o fst>, snd>)) o 
	  flat o iterate (K (t), iter (K (t), id) o <id, K (Caps)>) o 
	   flat o iterate (K (t), iter (K (t), id) o <id, K (Regs)>) o 
	    flat o iterate (K (t), iter (K (t), id) o <id, K (Sens)>) o 
         singleton ! *
  
*/   

%token COMMENT INTCON STRINGCON FLOATCON CHARCON TRUEV FALSEV NIL
%token DOT NE LE GE IS
%token NOT AND OR LIKE
%token ELT FLATTEN STRUCT SET UNION INTERSECT EXCEPT 
%token EXISTS FORALL INP SELECT DISTINCT FROM WHERE 
%token MOD ABS COUNT SUM MIN MAX AVG
%token ORDER GROUP BY IDENT ERROR ASC DESC

%left OR AND
%left NOT
%left '=' '>' '<' LE GE NE
%left '-' '+' INP
%left '*' '/' MOD UNION INTERSECT EXCEPT
%left DOT

%{
#include "KGlobals.H"
#include "KOFShiftL.H"
#include "KOFShiftR.H"
#include "KOBAnd.H"
#include "KOBEqu.H"
#include "KOBGE.H"
#include "KOBGT.H"
#include "KOBInvoke.H"
#include "KOBLE.H"
#include "KOBLT.H"
#include "KOBName.H"
#include "KOBNot.H"
#include "KOBOr.H"
#include "KOBPrimitive.H"
#include "KOBVar.H"
#include "KOBool.H"
#include "KOF1.H"
#include "KOF2.H"
#include "KOFAbs.H"
#include "KOFAdd.H"
#include "KOFAvg.H"
#include "KOFCElt.H"
#include "KOFChoose.H"
#include "KOFCompose.H"
#include "KOFCon.H"
#include "KOFConst.H"
#include "KOFCount.H"
#include "KOFDif.H"
#include "KOFDiv.H"
#include "KOFFlatten.H"
#include "KOFFold.H"
#include "KOFID.H"
#include "KOFTID.H"
#include "KOFIns.H"
#include "KOFInt.H"
#include "KOFInverse.H"
#include "KOFIter.H"
#include "KOFIterate.H"
#include "KOFJoin.H"
#include "KOFLSJoin.H"
#include "KOFMax.H"
#include "KOFMin.H"
#include "KOFMinus.H"
#include "KOFMod.H"
#include "KOFMult.H"
#include "KOFNJoin.H"
#include "KOFPCon.H"
#include "KOFPFold.H"
#include "KOFPlus.H"
#include "KOFRSJoin.H"
#include "KOFSet.H"
#include "KOFSingleton.H"
#include "KOFSort.H"
#include "KOFSum.H"
#include "KOFTimes.H"
#include "KOFUni.H"
#include "KOFUnnest.H"
#include "KOFUser.H"
#include "KOFVar.H"
#include "KOFunction.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "KOOAny.H"
#include "KOODif.H"
#include "KOODiv.H"
#include "KOOFloat.H"
#include "KOOId.H"
#include "KOOIf.H"
#include "KOOInt.H"
#include "KOOInter.H"
#include "KOOInvoke.H"
#include "KOOLink.H"
#include "KOOMinus.H"
#include "KOOMult.H"
#include "KOOName.H"
#include "KOOPair.H"
#include "KOOPlus.H"
#include "KOOSet.H"
#include "KOOStr.H"
#include "KOOTimes.H"
#include "KOOUni.H"
#include "KOOVar.H"
#include "KOObjList.H"
#include "KOObject.H"
#include "KOPCEX.H"
#include "KOPCElt.H"
#include "KOPCFA.H"
#include "KOPCombine.H"
#include "KOPConjunct.H"
#include "KOPConst.H"
#include "KOPDisjunct.H"
#include "KOPEqual.H"
#include "KOPExists.H"
#include "KOPForall.H"
#include "KOPGreater.H"
#include "KOPGreaterEqual.H"
#include "KOPInv.H"
#include "KOPIsnotnull.H"
#include "KOPIsnull.H"
#include "KOPLess.H"
#include "KOPLessEqual.H"
#include "KOPLex.H"
#include "KOPNegate.H"
#include "KOPNotEqual.H"
#include "KOPTimes.H"
#include "KOPUser.H"
#include "KOPVar.H"
#include "KOPredicate.H"
#include "KOTWrap.H"

#include "TYTAny.H"
#include "TYTBag.H"
#include "TYTBool.H"
#include "TYTChar.H"
#include "TYTDC.H"
#include "TYTError.H"
#include "TYTFlt.H"
#include "TYTFunc.H"
#include "TYTInt.H"
#include "TYTObj.H"
#include "TYTPair.H"
#include "TYTPred.H"
#include "TYTSet.H"
#include "TYTStr.H"
#include "TYTTup.H"
#include "TYTUnk.H"
#include "TYTUser.H"
#include "TYTVar.H"
#include "TYType.H"

#include "ENEnvironment.H"
#include "ENEnvEntry.H"

#include <iostream.h>
#include <strings.h>

#include "QLAuxCode.H"

// #include "result.H"

extern int lineno3;
extern int QLlex ();

extern char *QLtext;
int QLerror (char *);

////////////////////////////////////////////////////////////////////////////////
TObj *GetType (Environment *e, char *s)
{
  return e -> GetType (s);
}
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
NonType *SRule (Function *f, Predicate *p)
{
  if (f == NULL) {
	return new BInvoke (p, new OAny ());
  } else {
	return new OInvoke (f, new OAny ());
  } 
}
////////////////////////////////////////////////////////////////////////////////

extern NonType *result;

%}

 
 
 
 
  

 
    
 
  
 
  
 
 
  
%start yyyAugNonterm


%{


struct yyyT1 {int val;}; 
typedef struct yyyT1 *yyyP1; 


struct yyyT2 {char *string;}; 
typedef struct yyyT2 *yyyP2; 


struct yyyT3 {char *string;}; 
typedef struct yyyT3 *yyyP3; 


struct yyyT4 {char *string;}; 
typedef struct yyyT4 *yyyP4; 


struct yyyT5 {char *string;}; 
typedef struct yyyT5 *yyyP5; 


struct yyyT6 {NonType *res;}; 
typedef struct yyyT6 *yyyP6; 


struct yyyT7 {Function *func; Predicate *pred; Environment *ienv;}; 
typedef struct yyyT7 *yyyP7; 


struct yyyT8 {Function *func;  Environment *ienv; Environment *oenv;}; 
typedef struct yyyT8 *yyyP8; 


struct yyyT9 {Function *func;  Environment *ienv;}; 
typedef struct yyyT9 *yyyP9; 


struct yyyT10 {Predicate *pred;  Environment *ienv;}; 
typedef struct yyyT10 *yyyP10; 


struct yyyT11 {Function *func;}; 
typedef struct yyyT11 *yyyP11; 


struct yyyT12 {Predicate *pred;}; 
typedef struct yyyT12 *yyyP12; 


struct yyyT13 {int val;}; 
typedef struct yyyT13 *yyyP13; 
                                                      /*custom*/  
typedef unsigned char yyyWAT; 
typedef unsigned char yyyRCT; 
typedef unsigned short yyyPNT; 
typedef unsigned char yyyWST; 

#include <limits.h>
#define yyyR UCHAR_MAX  

 /* funny type; as wide as the widest of yyyWAT,yyyWST,yyyRCT  */ 
typedef unsigned short yyyFT;

                                                      /*stock*/  




struct yyyGenNode {void *parent;  
                   struct yyyGenNode **cL; /* child list */ 
                   yyyRCT *refCountList; 
                   yyyPNT prodNum;                      
                   yyyWST whichSym; /* which child of parent? */ 
                  }; 

typedef struct yyyGenNode yyyGNT; 



struct yyyTB {int isEmpty; 
              int typeNum; 
              int nAttrbs; 
              char *snBufPtr; 
              yyyWAT *startP,*stopP; 
             };  




extern struct yyyTB yyyTermBuffer; 
extern yyyWAT yyyLRCIL[]; 
extern void yyyGenLeaf(int nAttrbs,int typeNum,
                       yyyWAT *startP,yyyWAT *stopP
                      ); 


%}

%{
#include <stdio.h>
#include <stdlib.h>

int yyyYok = 1;
int yyyInitDone = 0;
char *yyySTsn;
yyyGNT *yyySTN;
int yyyGNSz = sizeof(yyyGNT);
int yyyProdNum,yyyRHSlength,yyyNattrbs,yyyTypeNum; 

extern yyyFT yyyRCIL[];

void yyyExecuteRRsection();
void yyyYoxInit();
void yyyYoxReset();
void yyyDecorate();
void yyyGenIntNode();
void yyyAdjustINRC(register yyyFT *,register yyyFT *);
void yyyUnsolvedInstSearchTrav(yyyGNT *pNode);
void yyyUnsolvedInstSearchTravAux(yyyGNT *pNode);
void yyyPrune(long prodNum);
void yyyerror();
void yyyShift();



#define yyyRSU(NUM1,NUM2,NUM3,NUM4) \
   yyyProdNum=NUM1;yyyRHSlength=NUM2;yyyNattrbs=NUM3;yyyTypeNum=NUM4;\
   if ((yychar <= 0) && (!yyyTermBuffer.isEmpty)) yyyShift(); 
%}


%%

yyyAugNonterm 
	:	{if (!yyyInitDone) 
		    {yyyYoxInit(); 
		     yyyInitDone = 1;
		    }
		 yyyYoxReset();
		}
		S
		{
		 yyyDecorate(); yyyExecuteRRsection();
		}
	;
S	    :  Comment Q

               {
                  Environment *nenv;
		  nenv = new Environment();
	       {if(yyyYok){
yyyRSU(1,2,1,6);
yyyGenIntNode();
 (((yyyP7)(((char *)((yyySTN->cL)[1]))+yyyGNSz))->ienv) = (nenv -> add_entry ("NULL", NULL));
                        yyyAdjustINRC(yyyRCIL+0,yyyRCIL+6);yyyPrune(1);}}}

               


Comment :  COMMENT Comment 
        {if(yyyYok){
yyyRSU(2,2,0,0);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+6,yyyRCIL+6);yyyPrune(2);}}|
        {if(yyyYok){
yyyRSU(3,0,0,0);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+6,yyyRCIL+6);}};


Q       :  INTCON

			 {if(yyyYok){
yyyRSU(4,1,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
			  (((yyyP7)yyySTsn)->func) = IntFunc ((((yyyP1)(((char *)((yyySTN->cL)[0]))+yyyGNSz))->val));
			    yyyAdjustINRC(yyyRCIL+6,yyyRCIL+12);yyyPrune(4);}}
			 
            /* F:  K (OInt (i)) */
	        
        |  STRINGCON

			 {if(yyyYok){
yyyRSU(5,1,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
			  (((yyyP7)yyySTsn)->func) = StrFunc ((((yyyP2)(((char *)((yyySTN->cL)[0]))+yyyGNSz))->string));
			    yyyAdjustINRC(yyyRCIL+12,yyyRCIL+18);yyyPrune(5);}}

            /* F:  K (OStr (s)) */
	        
        |  FLOATCON

			 {if(yyyYok){
yyyRSU(6,1,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
			  (((yyyP7)yyySTsn)->func) = FloatFunc ((((yyyP3)(((char *)((yyySTN->cL)[0]))+yyyGNSz))->string));
			    yyyAdjustINRC(yyyRCIL+18,yyyRCIL+24);yyyPrune(6);}}

            /* F:  K (OStr (s)) */

        |  NIL

			 {if(yyyYok){
yyyRSU(7,1,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NilFunc ();
			     (((yyyP7)yyySTsn)->pred) = NULL;
			 yyyAdjustINRC(yyyRCIL+24,yyyRCIL+30);yyyPrune(7);}}

            /* F:  K (OAny ()) */
			 
        |  TRUEV

			 {if(yyyYok){
yyyRSU(8,1,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NULL;
			     (((yyyP7)yyySTsn)->pred) = new PConst (new BPrimitive (T));
			 yyyAdjustINRC(yyyRCIL+30,yyyRCIL+36);yyyPrune(8);}}

             /* F:  K (BPrim (T)) */
		    
	    |  FALSEV

			 {if(yyyYok){
yyyRSU(9,1,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NULL;
			     (((yyyP7)yyySTsn)->pred) = new PConst (new BPrimitive (F));
			 yyyAdjustINRC(yyyRCIL+36,yyyRCIL+42);yyyPrune(9);}}
		    
             /* F:  K (BPrim (F)) */

	    |  '(' Q ')'

			 {if(yyyYok){
yyyRSU(10,3,3,7);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+42,yyyRCIL+51);}}

             /* F:  (Q) */

	    |  '-' Q

			 {if(yyyYok){
yyyRSU(11,2,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
			 yyyAdjustINRC(yyyRCIL+51,yyyRCIL+60);}}

             /* F:  C (MULT, -1) o Q */

        |    ELT '(' Q ')'

             {if(yyyYok){
yyyRSU(12,4,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+60,yyyRCIL+69);}}

            /* F:  SINGLETON o Q */

        |    ABS '(' Q ')'

             {if(yyyYok){
yyyRSU(13,4,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+69,yyyRCIL+78);}}

             /* F:  ABS o Q */

         |  Q '+' Q

			 {if(yyyYok){
yyyRSU(14,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
			 yyyAdjustINRC(yyyRCIL+78,yyyRCIL+90);}}

             /* F:  ADD o \<Q, Q\> */

         |  Q '-' Q

		     {if(yyyYok){
yyyRSU(15,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
			 yyyAdjustINRC(yyyRCIL+90,yyyRCIL+102);}}

             /* F:  SUB o \<Q, Q\> */

         |  Q '*' Q

			 {if(yyyYok){
yyyRSU(16,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
			 yyyAdjustINRC(yyyRCIL+102,yyyRCIL+114);}}

             /* F:  MULT o \<Q, Q\> */

         |  Q '/' Q

			 {if(yyyYok){
yyyRSU(17,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
			 yyyAdjustINRC(yyyRCIL+114,yyyRCIL+126);}}

             /* F:  DIV o \<Q, Q\> */


         |    Q MOD Q

             {if(yyyYok){
yyyRSU(18,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+126,yyyRCIL+138);}}

         |  Q '=' Q

			 {if(yyyYok){
yyyRSU(19,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func)  = NULL;
			    yyyAdjustINRC(yyyRCIL+138,yyyRCIL+150);}}

             /* P:  EQ OPLUS \<Q, Q\> */

         |  Q NE Q

			 {if(yyyYok){
yyyRSU(20,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func)  = NULL;
			    yyyAdjustINRC(yyyRCIL+150,yyyRCIL+162);}}

             /* P:  NEQ OPLUS \<Q, Q\> */

         |  Q '<' Q

			 {if(yyyYok){
yyyRSU(21,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func)  = NULL;
			    yyyAdjustINRC(yyyRCIL+162,yyyRCIL+174);}}
		    
             /* P:  LT OPLUS \<Q, Q\> */

         |  Q LE Q

			 {if(yyyYok){
yyyRSU(22,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func)  = NULL;
			    yyyAdjustINRC(yyyRCIL+174,yyyRCIL+186);}}
		    
             /* P:  LEQ OPLUS \<Q, Q\> */

         |  Q '>' Q

			 {if(yyyYok){
yyyRSU(23,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func)  = NULL;
			    yyyAdjustINRC(yyyRCIL+186,yyyRCIL+198);}}

             /* P:  GT OPLUS \<Q, Q\> */
		    
         |  Q GE Q

			 {if(yyyYok){
yyyRSU(24,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func)  = NULL;
			    yyyAdjustINRC(yyyRCIL+198,yyyRCIL+210);}}

             /* P:  GEQ OPLUS \<Q, Q\> */

         |  NOT Q

	    	 {if(yyyYok){
yyyRSU(25,2,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NULL;
			    yyyAdjustINRC(yyyRCIL+210,yyyRCIL+219);}}			 
		    
             /* P:  ~ (Q) */

         |  Q AND Q

			 {if(yyyYok){
yyyRSU(26,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NULL;
			    yyyAdjustINRC(yyyRCIL+219,yyyRCIL+231);}}

             /* P:  Q & Q */
		    
         |  Q OR Q

			 {if(yyyYok){
yyyRSU(27,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NULL;
			    yyyAdjustINRC(yyyRCIL+231,yyyRCIL+243);}}

             /* P:  Q | Q */

         |  FLATTEN '(' Q ')'

             {if(yyyYok){
yyyRSU(28,4,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+243,yyyRCIL+252);}}

             /* F:  FLAT o Q */

         |  Q UNION Q

             {if(yyyYok){
yyyRSU(29,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+252,yyyRCIL+264);}}

             /* F:  UNI (=) o \<Q, Q\> */

         |  Q INTERSECT Q

             {if(yyyYok){
yyyRSU(30,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+264,yyyRCIL+276);}}
                    
            /* F:  INT (=) o \<Q, Q\> */

         |  Q EXCEPT Q

             {if(yyyYok){
yyyRSU(31,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+276,yyyRCIL+288);}}

             /* F:  DIF (=) o \<Q, Q\> */

         |  Q INP Q

			 {if(yyyYok){
yyyRSU(32,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NULL;
			    yyyAdjustINRC(yyyRCIL+288,yyyRCIL+300);}}

             /* P:  EX (=) OPLUS \<Q, Q\> */

         |  COUNT '(' Q ')'

             {if(yyyYok){
yyyRSU(33,4,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+300,yyyRCIL+309);}}

             /* F: FOLD (0, K (1), add) o Q */

         |  SUM '(' Q ')'

             {if(yyyYok){
yyyRSU(34,4,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+309,yyyRCIL+318);}}

             /* F: FOLD (0, id, add) o Q */

         |  MIN '(' Q ')'

             {if(yyyYok){
yyyRSU(35,4,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+318,yyyRCIL+327);}}

             /* F: PFOLD (id, con (lt, fst, snd)) o Q */

         |  MAX '(' Q ')'

             {if(yyyYok){
yyyRSU(36,4,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+327,yyyRCIL+336);}}

             /* F: PFOLD (id, con (lt, snd, fst)) o Q */

         |  AVG '(' Q ')'

             {if(yyyYok){
yyyRSU(37,4,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
             yyyAdjustINRC(yyyRCIL+336,yyyRCIL+345);}}

             /* F: (pcon (C (neq, 0) OPLUS snd, div) o  
                             FOLD ([0, 0], \<id, K (1)\>, (add x add))) o Q */ 


         |  Q IS NIL

			 {if(yyyYok){
yyyRSU(38,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NULL;
			    yyyAdjustINRC(yyyRCIL+345,yyyRCIL+354);}}

             /* P:  ISNULL OPLUS Q */

         |  Q IS NOT NIL

			 {if(yyyYok){
yyyRSU(39,4,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NULL;
			    yyyAdjustINRC(yyyRCIL+354,yyyRCIL+363);}}

             /* P:  ISNOTNULL OPLUS Q */

         |  FORALL IDENT INP Q ':' '(' Q ')'

		     {if(yyyYok){
yyyRSU(40,8,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NULL;
			    yyyAdjustINRC(yyyRCIL+363,yyyRCIL+375);}}
		    
             /* P: FA (Q0) OPLUS <id, Q1> */

         |  EXISTS IDENT INP Q ':' '(' Q ')'

		     {if(yyyYok){
yyyRSU(41,8,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->func) = NULL;
			    yyyAdjustINRC(yyyRCIL+375,yyyRCIL+387);}}
		    
             /* P: EX (Q0) OPLUS <id, Q1> */

         |  SELECT DistOpt Q FROM QEnv WhereOpt OrderOpt

			 {if(yyyYok){
yyyRSU(42,7,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred)         = NULL;
			 yyyAdjustINRC(yyyRCIL+387,yyyRCIL+405);}}

            /* F: [sort (..) o ] [set o ] iterate (T (Q_1), T (Q_0)) o QEnv o singleton */

         |  STRUCT '(' DPrL ')'
 
	         {if(yyyYok){
yyyRSU(43,4,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred) = NULL;
			 yyyAdjustINRC(yyyRCIL+405,yyyRCIL+414);}}
		    
             /* F: DPrL */

         |  IDENT 

			 {if(yyyYok){
yyyRSU(44,1,3,7);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+414,yyyRCIL+420);}}


         |  PExp

			 {if(yyyYok){
yyyRSU(45,1,3,7);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+420,yyyRCIL+429);}}
		    
         ;


PExp    : Q DOT IDENT '(' NEQList ')'

			 {if(yyyYok){
yyyRSU(46,6,3,7);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+429,yyyRCIL+441);}}

           /* F:  IDENT o <Q, QLIST> */

        | Q DOT IDENT 

			 {if(yyyYok){
yyyRSU(47,3,3,7);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+441,yyyRCIL+450);}}

           /* F:  IDENT o Q */

        | Q DOT IDENT '(' ')'

			 {if(yyyYok){
yyyRSU(48,5,3,7);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+450,yyyRCIL+459);}}

           /* F:  IDENT o Q */
        ;

NEQList : NEQList ',' Q

			 {if(yyyYok){
yyyRSU(49,3,2,9);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+459,yyyRCIL+468);}}		    

        | Q

			 {if(yyyYok){
yyyRSU(50,1,2,9);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+468,yyyRCIL+474);}}
		            
        ;

WhereOpt : WHERE Q

             {if(yyyYok){
yyyRSU(51,2,2,10);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+474,yyyRCIL+480);}}
   
        |

             {if(yyyYok){
yyyRSU(52,0,2,10);
yyyGenIntNode();
 (((yyyP10)yyySTsn)->pred) = new PConst (new BPrimitive (1)); yyyAdjustINRC(yyyRCIL+480,yyyRCIL+483);}}
   
        ;


OrderOpt : ORDER BY MethList Direcn

             {if(yyyYok){
yyyRSU(53,4,2,9);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+483,yyyRCIL+486);yyyPrune(53);}}
   
          /* F: SORT (LESS OPLUS (<...<m1, m2>, ..., mn> x <...<m1, m2>, ..., mn>)) */

        |  ORDER BY IDENT Direcn

             {if(yyyYok){
yyyRSU(54,4,2,9);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+486,yyyRCIL+489);yyyPrune(54);}}
   
          /* F: SORT (LESS OPLUS <K (0), IDENT>) */

        |

             {if(yyyYok){
yyyRSU(55,0,2,9);
yyyGenIntNode();
 (((yyyP9)yyySTsn)->func) = new FID (); yyyAdjustINRC(yyyRCIL+489,yyyRCIL+492);}}

          /* F: ID */   

        ;



MethList : MethList ',' IDENT '(' ')'

             {if(yyyYok){
yyyRSU(56,5,1,11);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+492,yyyRCIL+495);yyyPrune(56);}}

        |  IDENT '(' ')'

             {if(yyyYok){
yyyRSU(57,3,1,11);
yyyGenIntNode();
 (((yyyP11)yyySTsn)->func)   = new FUser ((((yyyP5)(((char *)((yyySTN->cL)[0]))+yyyGNSz))->string)); yyyAdjustINRC(yyyRCIL+495,yyyRCIL+498);yyyPrune(57);}}

        ;



Direcn  : ASC

             {if(yyyYok){
yyyRSU(58,1,1,12);
yyyGenIntNode();
 (((yyyP12)yyySTsn)->pred) = new PLess (); yyyAdjustINRC(yyyRCIL+498,yyyRCIL+501);yyyPrune(58);}}     

        | DESC

             {if(yyyYok){
yyyRSU(59,1,1,12);
yyyGenIntNode();
 (((yyyP12)yyySTsn)->pred) = new PGreater (); yyyAdjustINRC(yyyRCIL+501,yyyRCIL+504);yyyPrune(59);}}     

        ;



DistOpt :  DISTINCT
                         
             {if(yyyYok){
yyyRSU(60,1,1,13);
yyyGenIntNode();
 (((yyyP13)yyySTsn)->val) = T; yyyAdjustINRC(yyyRCIL+504,yyyRCIL+507);yyyPrune(60);}}

        |

             {if(yyyYok){
yyyRSU(61,0,1,13);
yyyGenIntNode();
 (((yyyP13)yyySTsn)->val) = F; yyyAdjustINRC(yyyRCIL+507,yyyRCIL+510);}}

        ;


QEnv	:  Q IDENT

			 {if(yyyYok){
yyyRSU(62,2,3,8);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+510,yyyRCIL+519);}}

            /* F: iterate (K (true), flat o iter (K (true), id) o <id, T (Q)>) */  
                   
        |  QEnv ',' Q IDENT 

			 {if(yyyYok){
yyyRSU(63,4,3,8);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+519,yyyRCIL+531);}}

            /* F: iterate (K (true), flat o iter (K (true), id) o <id, T (Q_n)>) o QEnv */ 

        |  IDENT INP Q

			 {if(yyyYok){
yyyRSU(64,3,3,8);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+531,yyyRCIL+540);}}

            /* F: iterate (K (true), flat o iter (K (true), id) o <id, T (Q)>) */  
                   
        |  QEnv ',' IDENT INP Q

			 {if(yyyYok){
yyyRSU(65,5,3,8);
yyyGenIntNode();
yyyAdjustINRC(yyyRCIL+540,yyyRCIL+552);}}

            /* F: iterate (K (true), flat o iter (K (true), id) o <id, T (Q_n)>) o QEnv */ 

        ;


DPrL	:  IDENT ':' Q

			 {if(yyyYok){
yyyRSU(66,3,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred)   = NULL;

			 yyyAdjustINRC(yyyRCIL+552,yyyRCIL+561);}}

        |  DPrL ',' IDENT ':' Q

			 {if(yyyYok){
yyyRSU(67,5,3,7);
yyyGenIntNode();
 (((yyyP7)yyySTsn)->pred)  = NULL;
			 yyyAdjustINRC(yyyRCIL+561,yyyRCIL+573);}}

	    ;



%%

int QLerror (char *s)
{
  cerr <<  s << " found at line: " << lineno3 << " (" << QLtext << ")" << "\n\n";
  return 0;
}

/*
	        |    FLOATCON

			 @{ @i @Q.func@ = FloatFunc (@FLOATCON.val@);
			    @i @Q.pred@ = NULL;
			 @}


*/
                                                      /*custom*/  
long yyyMaxNbytesNodeStg = 2000000; 
long yyyMaxNrefCounts =    500000; 
long yyyMaxNchildren =     60000; 
long yyyMaxStackSize =     2000; 
long yyySSALspaceSize =    20000; 
long yyyRSmaxSize =        1000; 
long yyyTravStackMaxSize = 2000; 


struct yyyTB yyyTermBuffer; 

char *yyyNodeAndStackSpace; 

char *yyyNodeSpace;
char *yyyNextNodeSpace; 
char *yyyAfterNodeSpace; 


 
struct yyyGenNode **yyyChildListSpace;  
struct yyyGenNode **yyyNextCLspace; 
struct yyyGenNode **yyyAfterChildListSpace; 



yyyRCT *yyyRefCountListSpace;
yyyRCT *yyyNextRCLspace;  
yyyRCT *yyyAfterRefCountListSpace;   



struct yyySolvedSAlistCell {yyyWAT attrbNum; 
                            long next; 
                           }; 
#define yyyLambdaSSAL 0 
long yyySSALCfreeList = yyyLambdaSSAL; 
long yyyNewSSALC = 1; 
 
struct yyySolvedSAlistCell *yyySSALspace; 


 
struct yyyStackItem {struct yyyGenNode *node; 
                     long solvedSAlist; 
                     struct yyyGenNode *oldestNode; 
                    };  

long yyyNbytesStackStg; 
struct yyyStackItem *yyyStack; 
struct yyyStackItem *yyyAfterStack; 
struct yyyStackItem *yyyStackTop; 



struct yyyRSitem {yyyGNT *node; 
                  yyyWST whichSym; 
                  yyyWAT wa;  
                 };  

struct yyyRSitem *yyyRS;  
struct yyyRSitem *yyyRSTop;  
struct yyyRSitem *yyyAfterRS;  
 





yyyFT yyyRCIL[] = {
yyyR,0,2, 1,2,0, yyyR,0,0, yyyR,1,0, yyyR,0,0, yyyR,1,0, 
yyyR,0,0, yyyR,1,0, yyyR,0,0, yyyR,1,0, yyyR,0,0, yyyR,1,0, 
yyyR,0,0, yyyR,1,0, yyyR,0,1, yyyR,1,1, 1,2,1, yyyR,0,1, 
yyyR,1,0, 1,2,1, yyyR,0,1, yyyR,1,0, 2,2,1, yyyR,0,1, 
yyyR,1,0, 2,2,1, yyyR,0,2, yyyR,1,0, 0,2,1, 2,2,1, 
yyyR,0,2, yyyR,1,0, 0,2,1, 2,2,1, yyyR,0,2, yyyR,1,0, 
0,2,1, 2,2,1, yyyR,0,2, yyyR,1,0, 0,2,1, 2,2,1, 
yyyR,0,2, yyyR,1,0, 0,2,1, 2,2,1, yyyR,0,0, yyyR,1,2, 
0,2,1, 2,2,1, yyyR,0,0, yyyR,1,2, 0,2,1, 2,2,1, 
yyyR,0,0, yyyR,1,2, 0,2,1, 2,2,1, yyyR,0,0, yyyR,1,2, 
0,2,1, 2,2,1, yyyR,0,0, yyyR,1,2, 0,2,1, 2,2,1, 
yyyR,0,0, yyyR,1,2, 0,2,1, 2,2,1, yyyR,0,0, yyyR,1,1, 
1,2,1, yyyR,0,0, yyyR,1,2, 0,2,1, 2,2,1, yyyR,0,0, 
yyyR,1,2, 0,2,1, 2,2,1, yyyR,0,1, yyyR,1,0, 2,2,1, 
yyyR,0,2, yyyR,1,0, 0,2,1, 2,2,1, yyyR,0,2, yyyR,1,0, 
0,2,1, 2,2,1, yyyR,0,2, yyyR,1,0, 0,2,1, 2,2,1, 
yyyR,0,0, yyyR,1,2, 0,2,1, 2,2,1, yyyR,0,1, yyyR,1,0, 
2,2,1, yyyR,0,1, yyyR,1,0, 2,2,1, yyyR,0,1, yyyR,1,0, 
2,2,1, yyyR,0,1, yyyR,1,0, 2,2,1, yyyR,0,1, yyyR,1,0, 
2,2,1, yyyR,0,0, yyyR,1,1, 0,2,1, yyyR,0,0, yyyR,1,1, 
0,2,1, yyyR,0,0, yyyR,1,2, 3,2,1, 6,2,1, yyyR,0,0, 
yyyR,1,2, 3,2,1, 6,2,1, yyyR,0,5, yyyR,1,0, 2,2,1, 
4,1,1, 5,1,1, 6,1,1, yyyR,0,1, yyyR,1,0, 2,2,1, 
yyyR,0,1, yyyR,1,1, yyyR,0,1, yyyR,1,1, 0,2,1, yyyR,0,2, 
yyyR,1,2, 0,2,1, 4,1,1, yyyR,0,1, yyyR,1,1, 0,2,1, 
yyyR,0,1, yyyR,1,1, 0,2,1, yyyR,0,2, 0,1,1, 2,2,1, 
yyyR,0,1, 0,2,1, yyyR,0,1, 1,2,1, yyyR,0,0, yyyR,0,2, 
yyyR,0,1, yyyR,0,0, yyyR,0,1, yyyR,0,0, yyyR,0,0, yyyR,0,0, 
yyyR,0,0, yyyR,0,0, yyyR,0,1, yyyR,2,1, 0,2,1, yyyR,0,2, 
yyyR,2,1, 0,1,1, 2,2,1, yyyR,0,1, yyyR,2,1, 2,2,1, 
yyyR,0,2, yyyR,2,1, 0,1,1, 4,2,1, yyyR,0,1, yyyR,1,0, 
2,2,1, yyyR,0,2, yyyR,1,0, 0,2,1, 4,2,1, 
};

short yyyIIIEL[] = {0,
0,3,6,7,9,11,13,15,17,19,
23,26,31,36,40,44,48,52,56,60,
64,68,72,76,80,83,87,91,96,100,
104,108,112,117,122,127,132,137,141,146,
155,164,172,177,179,181,188,192,198,202,
204,207,208,213,218,219,225,229,231,233,
235,236,239,244,248,254,258,
};

long yyyIIEL[] = {
0,1,1,4,4,4,4,4,7,8,11,12,
15,16,19,19,22,22,25,25,28,28,31,31,
34,34,37,40,40,40,43,43,46,46,46,49,
49,52,55,55,58,61,64,64,67,70,73,73,
76,79,82,82,85,88,91,91,94,97,100,100,
103,106,109,109,112,115,118,118,121,124,127,127,
130,133,136,136,139,142,145,145,148,151,151,154,
157,160,160,163,166,169,169,172,175,175,175,178,
178,181,184,184,187,190,193,193,196,199,202,202,
205,208,211,211,214,217,217,217,220,220,223,223,
223,226,226,229,229,229,232,232,235,235,235,238,
238,241,241,241,244,244,247,250,250,250,253,256,
256,256,256,259,259,260,260,263,263,263,266,266,
269,269,270,270,273,273,273,276,276,279,279,280,
283,283,286,288,290,293,293,293,296,296,299,300,
303,306,309,312,312,313,313,315,315,318,321,321,
322,325,328,328,329,329,329,331,333,333,336,338,
341,343,343,346,348,350,350,350,351,352,354,354,
354,355,356,358,359,360,360,361,361,361,362,363,
363,363,364,364,365,365,366,366,367,370,373,374,
377,380,380,383,384,387,388,388,391,394,397,397,
398,398,401,404,405,405,408,411,414,414,415,415,

};

long yyyIEL[] = {
0,0,2,4,4,4,4,4,
6,6,6,6,8,8,8,8,
10,10,10,10,10,10,10,10,
10,10,10,10,12,14,16,16,
16,16,18,20,20,20,20,20,
22,24,24,24,24,24,26,28,
28,28,28,28,32,34,34,34,
36,36,36,36,36,40,42,42,
42,44,44,44,44,44,48,50,
50,50,52,52,52,52,52,56,
58,58,58,60,60,60,60,60,
64,66,66,66,68,68,68,68,
68,72,74,74,74,76,76,76,
76,76,80,82,82,82,84,84,
84,84,84,88,90,90,90,92,
92,92,92,92,96,98,98,98,
100,100,100,100,100,104,106,106,
106,108,108,108,108,108,112,114,
114,114,116,116,116,116,116,118,
118,120,120,120,120,124,124,126,
126,126,128,128,128,128,132,132,
134,134,134,136,136,136,136,138,
140,140,140,140,140,144,146,146,
146,148,148,148,148,148,152,154,
154,154,156,156,156,156,156,160,
162,162,162,164,164,164,164,164,
168,170,170,170,172,172,172,172,
172,174,176,176,176,176,176,178,
180,180,180,180,180,182,184,184,
184,184,184,186,188,188,188,188,
188,190,192,192,192,192,192,194,
196,196,196,196,196,198,200,200,
200,200,200,204,206,208,208,208,
208,210,210,210,210,214,216,218,
218,218,218,220,220,220,220,222,
224,226,226,226,228,228,234,236,
236,238,238,238,238,240,242,242,
242,242,242,246,250,250,250,252,
254,256,256,256,256,260,264,264,
264,268,272,272,272,272,274,278,
278,278,282,282,282,284,288,288,
288,292,292,296,298,298,300,300,
300,300,302,304,304,304,304,306,
306,308,308,308,308,308,308,310,
312,312,312,314,316,316,316,316,
318,320,320,322,322,322,322,322,
322,326,326,328,328,328,330,330,
332,332,334,334,338,340,340,340,
342,342,346,346,348,350,350,350,
350,352,352,354,354,358,360,362,
362,362,362,362,364,364,366,366,
366,366,366,370,372,372,372,372,
374,374,374,
};

yyyFT yyyEntL[] = {
0,0,0,0,0,0,0,0,0,0,2,2,0,0,0,1,
2,2,0,0,3,2,0,0,3,2,0,0,3,2,1,2,
0,0,0,0,3,2,1,2,0,0,0,0,3,2,1,2,
0,0,0,0,3,2,1,2,0,0,0,0,3,2,1,2,
0,0,0,0,3,2,1,2,0,1,0,1,3,2,1,2,
0,1,0,1,3,2,1,2,0,1,0,1,3,2,1,2,
0,1,0,1,3,2,1,2,0,1,0,1,3,2,1,2,
0,1,0,1,2,2,0,1,3,2,1,2,0,1,0,1,
3,2,1,2,0,1,0,1,3,2,0,0,3,2,1,2,
0,0,0,0,3,2,1,2,0,0,0,0,3,2,1,2,
0,0,0,0,3,2,1,2,0,1,0,1,3,2,0,0,
3,2,0,0,3,2,0,0,3,2,0,0,3,2,0,0,
1,2,0,1,1,2,0,1,7,2,4,2,7,2,0,1,
0,1,7,2,4,2,7,2,0,1,0,1,5,1,0,0,
0,0,0,0,7,1,6,1,3,2,0,0,0,0,3,2,
0,0,0,0,0,1,0,0,0,1,1,2,0,0,0,1,
5,1,1,2,0,0,0,1,0,0,0,1,0,0,0,1,
1,2,0,0,0,1,0,0,0,1,1,2,0,0,0,1,
0,0,0,1,3,2,1,1,0,0,0,0,1,2,0,0,
2,2,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,2,1,2,0,0,0,2,1,1,0,0,0,2,
3,2,0,0,0,2,0,2,3,2,0,2,0,0,1,1,
0,0,0,2,5,2,0,2,0,0,3,2,0,0,5,2,
1,2,0,0,0,0,
};

#define yyyPermitUserAlloc  0 


void yyyfatal(char *msg)
{fprintf(stderr,msg);exit(-1);} 



#define yyyNSof   'n' 
#define yyyRCof   'r' 
#define yyyCLof   'c' 
#define yyySof    's' 
#define yyySSALof 'S' 
#define yyyRSof   'q' 
#define yyyTSof   't' 



void yyyHandleOverflow(char which) 
  {char *msg1,*msg2; 
   long  oldSize,newSize; 
   switch(which) 
     {
      case yyyNSof   : 
           msg1 = "node storage overflow: ";
           oldSize = yyyMaxNbytesNodeStg; 
           break; 
      case yyyRCof   : 
           msg1 = "dependee count overflow: ";
           oldSize = yyyMaxNrefCounts; 
           break; 
      case yyyCLof   : 
           msg1 = "child list overflow: ";
           oldSize = yyyMaxNchildren; 
           break; 
      case yyySof    : 
           msg1 = "parse-tree stack overflow: ";
           oldSize = yyyMaxStackSize; 
           break; 
      case yyySSALof : 
           msg1 = "SSAL overflow: ";
           oldSize = yyySSALspaceSize; 
           break; 
      case yyyRSof   : 
           msg1 = "ready set overflow: ";
           oldSize = yyyRSmaxSize; 
           break; 
      case yyyTSof   : 
           msg1 = "traversal stack overflow: ";
           oldSize = yyyTravStackMaxSize; 
           break; 
      default        :;  
     }
   newSize = (3*oldSize)/2; 
   if (newSize < 100) newSize = 100; 
   fprintf(stderr,msg1); 
   fprintf(stderr,"size was %d.\n",oldSize); 
   if (yyyPermitUserAlloc) 
      msg2 = "     Try -Y%c%d option.\n"; 
      else 
      msg2 = "     Have to modify evaluator:  -Y%c%d.\n"; 
   fprintf(stderr,msg2,which,newSize); 
   exit(-1); 
  }



void yyySignalEnts(register yyyGNT *node,
                   register yyyFT *startP,register yyyFT *stopP) 
  {register yyyGNT *dumNode; 

   while (startP < stopP)  
     {
      if (!(*startP)) dumNode = node;  
         else dumNode = (node->cL)[(*startP)-1];   
      if (!(--((dumNode->refCountList)[*(startP+1)]
              ) 
           )
         ) 
         { 
          if (++yyyRSTop == yyyAfterRS) 
             {yyyHandleOverflow(yyyRSof); 
              break; 
             }
          yyyRSTop->node = dumNode; 
          yyyRSTop->whichSym = *startP;  
          yyyRSTop->wa = *(startP+1);  
         }  
      startP += 2;  
     }  
  } 




#define yyyCeiling(num,inc) (((inc) * ((num)/(inc))) + (((num)%(inc))?(inc):0)) 



int yyyAlignSize = 4;
int yyyNdSz[14];

int yyyNdPrSz[14];

typedef int yyyCopyType;

int yyyNdCopySz[14];
long yyyBiggestNodeSize = 0;

void yyyNodeSizeCalc()
  {int i;
   yyyGNSz = yyyCeiling(yyyGNSz,yyyAlignSize); 
   yyyNdSz[0] = 0;
   yyyNdSz[1] = sizeof(struct yyyT1);
   yyyNdSz[2] = sizeof(struct yyyT2);
   yyyNdSz[3] = sizeof(struct yyyT3);
   yyyNdSz[4] = sizeof(struct yyyT4);
   yyyNdSz[5] = sizeof(struct yyyT5);
   yyyNdSz[6] = sizeof(struct yyyT6);
   yyyNdSz[7] = sizeof(struct yyyT7);
   yyyNdSz[8] = sizeof(struct yyyT8);
   yyyNdSz[9] = sizeof(struct yyyT9);
   yyyNdSz[10] = sizeof(struct yyyT10);
   yyyNdSz[11] = sizeof(struct yyyT11);
   yyyNdSz[12] = sizeof(struct yyyT12);
   yyyNdSz[13] = sizeof(struct yyyT13);
   for (i=0;i<14;i++) 
       {yyyNdSz[i] = yyyCeiling(yyyNdSz[i],yyyAlignSize); 
        yyyNdPrSz[i] = yyyNdSz[i] + yyyGNSz;
        if (yyyBiggestNodeSize < yyyNdSz[i])
           yyyBiggestNodeSize = yyyNdSz[i];
        yyyNdCopySz[i] = yyyCeiling(yyyNdSz[i],sizeof(yyyCopyType)) / 
                         sizeof(yyyCopyType); 
       }
  }




void yyySolveAndSignal() {
register long yyyiDum,*yyypL;
register int yyyws,yyywa;
register yyyGNT *yyyRSTopN,*yyyRefN; 
register void *yyyRSTopNp; 


yyyRSTopNp = (yyyRSTopN = yyyRSTop->node)->parent;
yyyRefN= (yyyws = (yyyRSTop->whichSym))?((yyyGNT *)yyyRSTopNp):yyyRSTopN;
yyywa = yyyRSTop->wa; 
yyyRSTop--;
switch(yyyRefN->prodNum) {
case 1:  /***yacc rule 1***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 result = (((yyyP6)(((char *)yyyRSTopN)+yyyGNSz))->res) = SRule ((((yyyP7)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->pred));
                   break;
    }
  break;
  case 2:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 2:  /***yacc rule 2***/
  switch (yyyws) {
  }
break;
case 3:  /***yacc rule 3***/
  switch (yyyws) {
  }
break;
case 4:  /***yacc rule 4***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 5:  /***yacc rule 5***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 6:  /***yacc rule 6***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 7:  /***yacc rule 7***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 8:  /***yacc rule 8***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 9:  /***yacc rule 9***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 10:  /***yacc rule 10***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = CopyFunc ((((yyyP7)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->func));
			        break;
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred) = CopyPred ((((yyyP7)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->pred));
			     break;
    }
  break;
  case 2:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 11:  /***yacc rule 11***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = NegFunc ((((yyyP7)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->func));
			        break;
    }
  break;
  case 2:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 12:  /***yacc rule 12***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = ElementFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

                    break;
    }
  break;
  }
break;
case 13:  /***yacc rule 13***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = AbsFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

                    break;
    }
  break;
  }
break;
case 14:  /***yacc rule 14***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = PlusFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
			        break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);			 

			        break;
    }
  break;
  }
break;
case 15:  /***yacc rule 15***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = MinusFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
			        break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);			 
				 
			        break;
    }
  break;
  }
break;
case 16:  /***yacc rule 16***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = MultFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
			        break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);			 

			        break;
    }
  break;
  }
break;
case 17:  /***yacc rule 17***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = DivFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
			        break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);			 

			        break;
    }
  break;
  }
break;
case 18:  /***yacc rule 18***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = ModFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);                  

                    break;
    }
  break;
  }
break;
case 19:  /***yacc rule 19***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred)  = EqPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
	             break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 20:  /***yacc rule 20***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred)  = NeqPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
		         break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 21:  /***yacc rule 21***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred)  = LTPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
	             break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 22:  /***yacc rule 22***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred)  = LEQPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
		         break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 23:  /***yacc rule 23***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred)  = GTPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
		         break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 24:  /***yacc rule 24***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred)  = GEQPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
	             break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 25:  /***yacc rule 25***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred) = NotPred ((((yyyP7)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->pred));
			     break;
    }
  break;
  case 2:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 26:  /***yacc rule 26***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred) = AndPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->pred), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->pred));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 27:  /***yacc rule 27***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred) = OrPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->pred), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->pred));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 28:  /***yacc rule 28***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = FlattenFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

                    break;
    }
  break;
  }
break;
case 29:  /***yacc rule 29***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = UniFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);                         

                    break;
    }
  break;
  }
break;
case 30:  /***yacc rule 30***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = InterFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);                         

                    break;
    }
  break;
  }
break;
case 31:  /***yacc rule 31***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = DifFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);                         

                    break;
    }
  break;
  }
break;
case 32:  /***yacc rule 32***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred) = InPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
		         break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 33:  /***yacc rule 33***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = COUNTFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

                    break;
    }
  break;
  }
break;
case 34:  /***yacc rule 34***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = SUMFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

                    break;
    }
  break;
  }
break;
case 35:  /***yacc rule 35***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = MINFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

                    break;
    }
  break;
  }
break;
case 36:  /***yacc rule 36***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = MAXFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

                    break;
    }
  break;
  }
break;
case 37:  /***yacc rule 37***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = AVGFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

                    break;
    }
  break;
  }
break;
case 38:  /***yacc rule 38***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred) = IsNullPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func));
		         break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 39:  /***yacc rule 39***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred) = IsNotNullPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func));
		         break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 40:  /***yacc rule 40***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred) = FORALLPred ((((yyyP7)(((char *)((yyyRefN->cL)[3]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[6]))+yyyGNSz))->pred));
			     break;
    }
  break;
  case 2:  /**/
    switch (yyywa) {
    }
  break;
  case 4:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 7:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = QuanEnv ((((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv), (((yyyP5)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->string));

			        break;
    }
  break;
  }
break;
case 41:  /***yacc rule 41***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred) = EXISTSPred ((((yyyP7)(((char *)((yyyRefN->cL)[3]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[6]))+yyyGNSz))->pred));
			     break;
    }
  break;
  case 2:  /**/
    switch (yyywa) {
    }
  break;
  case 4:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 7:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = QuanEnv ((((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv), (((yyyP5)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->string));

			        break;
    }
  break;
  }
break;
case 42:  /***yacc rule 42***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func)         = SFWFunc ((((yyyP9)(((char *)((yyyRefN->cL)[6]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func), (((yyyP8)(((char *)((yyyRefN->cL)[4]))+yyyGNSz))->func), (((yyyP10)(((char *)((yyyRefN->cL)[5]))+yyyGNSz))->pred), (((yyyP13)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->val));
			        break;
    }
  break;
  case 2:  /**/
    switch (yyywa) {
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)         = (((yyyP8)(((char *)((yyyRefN->cL)[4]))+yyyGNSz))->oenv);
                    break;
    }
  break;
  case 5:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->ienv)        = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
                    break;
    }
  break;
  case 6:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP10)(((char *)yyyRSTopN)+yyyGNSz))->ienv)    = (((yyyP8)(((char *)((yyyRefN->cL)[4]))+yyyGNSz))->oenv);
                    break;
    }
  break;
  case 7:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP9)(((char *)yyyRSTopN)+yyyGNSz))->ienv)    = (((yyyP8)(((char *)((yyyRefN->cL)[4]))+yyyGNSz))->oenv);

			        break;
    }
  break;
  }
break;
case 43:  /***yacc rule 43***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = CopyFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 44:  /***yacc rule 44***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func) = IdentFunc ((((yyyP5)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->string), (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv));
			        break;
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred) = IdentPred ((((yyyP5)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->string), (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 45:  /***yacc rule 45***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func)    = CopyFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func));
			        break;
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred)    = CopyPred ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->pred));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)   = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 46:  /***yacc rule 46***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func)    = Path1Func ((((yyyP5)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->string), (((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP9)(((char *)((yyyRefN->cL)[4]))+yyyGNSz))->func));
			        break;
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred)    = Path1Pred ((((yyyP5)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->string), (((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP9)(((char *)((yyyRefN->cL)[4]))+yyyGNSz))->func));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)       = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    }
  break;
  case 5:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP9)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 47:  /***yacc rule 47***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func)    = Path2Func ((((yyyP5)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->string), (((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func));
			        break;
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred)    = Path2Pred ((((yyyP5)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->string), (((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)       = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 48:  /***yacc rule 48***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func)    = Path2Func ((((yyyP5)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->string), (((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func));
			        break;
    case 1:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->pred)    = Path2Pred ((((yyyP5)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->string), (((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)       = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 49:  /***yacc rule 49***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP9)(((char *)yyyRSTopN)+yyyGNSz))->func) = PairFunc ((((yyyP9)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP9)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP9)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)         = (((yyyP9)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 50:  /***yacc rule 50***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP9)(((char *)yyyRSTopN)+yyyGNSz))->func)   = CopyFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)         = (((yyyP9)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  }
break;
case 51:  /***yacc rule 51***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP10)(((char *)yyyRSTopN)+yyyGNSz))->pred) = CopyPred ((((yyyP7)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->pred));
                 break;
    }
  break;
  case 2:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv) = (((yyyP10)(((char *)yyyRefN)+yyyGNSz))->ienv); 
                    break;
    }
  break;
  }
break;
case 52:  /***yacc rule 52***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 53:  /***yacc rule 53***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP9)(((char *)yyyRSTopN)+yyyGNSz))->func)   = Sort1Func ((((yyyP11)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func), (((yyyP12)(((char *)((yyyRefN->cL)[3]))+yyyGNSz))->pred));     break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    }
  break;
  case 4:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 54:  /***yacc rule 54***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP9)(((char *)yyyRSTopN)+yyyGNSz))->func) = Sort2Func ((((yyyP5)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->string), (((yyyP12)(((char *)((yyyRefN->cL)[3]))+yyyGNSz))->pred));     break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    }
  break;
  case 4:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 55:  /***yacc rule 55***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 56:  /***yacc rule 56***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP11)(((char *)yyyRSTopN)+yyyGNSz))->func) = PairFunc ((((yyyP11)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), new FUser ((((yyyP5)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->string)));     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 57:  /***yacc rule 57***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 58:  /***yacc rule 58***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 59:  /***yacc rule 59***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 60:  /***yacc rule 60***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 61:  /***yacc rule 61***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 62:  /***yacc rule 62***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->func)     = QEnvFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func));
                    break;
    case 2:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->oenv)     = QEnvEnv ((((yyyP8)(((char *)yyyRefN)+yyyGNSz))->ienv), (((yyyP5)(((char *)((yyyRefN->cL)[1]))+yyyGNSz))->string));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)        = (((yyyP8)(((char *)yyyRefN)+yyyGNSz))->ienv);

                    break;
    }
  break;
  case 2:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 63:  /***yacc rule 63***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->func)  = QEnvFunc2 ((((yyyP8)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    case 2:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->oenv)  = QEnvEnv ((((yyyP8)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->oenv), (((yyyP5)(((char *)((yyyRefN->cL)[3]))+yyyGNSz))->string));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP8)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)       = (((yyyP8)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->oenv);
			    
                    break;
    }
  break;
  case 4:  /**/
    switch (yyywa) {
    }
  break;
  }
break;
case 64:  /***yacc rule 64***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->func)     = QEnvFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
                    break;
    case 2:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->oenv)     = QEnvEnv ((((yyyP8)(((char *)yyyRefN)+yyyGNSz))->ienv), (((yyyP5)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->string));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)        = (((yyyP8)(((char *)yyyRefN)+yyyGNSz))->ienv);

                    break;
    }
  break;
  }
break;
case 65:  /***yacc rule 65***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->func)  = QEnvFunc2 ((((yyyP8)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[4]))+yyyGNSz))->func));
                    break;
    case 2:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->oenv)  = QEnvEnv ((((yyyP8)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->oenv), (((yyyP5)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->string));
			     break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 1:
 (((yyyP8)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP8)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    }
  break;
  case 5:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)       = (((yyyP8)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->oenv);
			    
                    break;
    }
  break;
  }
break;
case 66:  /***yacc rule 66***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func)   = CopyFunc ((((yyyP7)(((char *)((yyyRefN->cL)[2]))+yyyGNSz))->func));
			        break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)    = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
case 67:  /***yacc rule 67***/
  switch (yyyws) {
  case 0:  /**/
    switch (yyywa) {
    case 0:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->func)  = PairFunc ((((yyyP7)(((char *)((yyyRefN->cL)[0]))+yyyGNSz))->func), (((yyyP7)(((char *)((yyyRefN->cL)[4]))+yyyGNSz))->func));
			        break;
    }
  break;
  case 1:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)  = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);
			        break;
    }
  break;
  case 3:  /**/
    switch (yyywa) {
    }
  break;
  case 5:  /**/
    switch (yyywa) {
    case 2:
 (((yyyP7)(((char *)yyyRSTopN)+yyyGNSz))->ienv)       = (((yyyP7)(((char *)yyyRefN)+yyyGNSz))->ienv);

			        break;
    }
  break;
  }
break;
} /* switch */ 

if (yyyws)  /* the just-solved instance was inherited. */ 
   {if (yyyRSTopN->prodNum) 
       {yyyiDum = yyyIIEL[yyyIIIEL[yyyRSTopN->prodNum]] + yyywa;
        yyySignalEnts(yyyRSTopN,yyyEntL + yyyIEL[yyyiDum],
                                yyyEntL + yyyIEL[yyyiDum+1]
                     );
       }
   } 
   else     /* the just-solved instance was synthesized. */ 
   {if ((char *)yyyRSTopNp >= yyyNodeSpace) /* node has a parent. */ 
       {yyyiDum = yyyIIEL[yyyIIIEL[((yyyGNT *)yyyRSTopNp)->prodNum] + 
                          yyyRSTopN->whichSym 
                         ] + 
                  yyywa;
        yyySignalEnts((yyyGNT *)yyyRSTopNp,
                      yyyEntL + yyyIEL[yyyiDum],
                      yyyEntL + yyyIEL[yyyiDum+1] 
                     );
       } 
       else   /* node is still on the stack--it has no parent yet. */ 
       {yyypL = &(((struct yyyStackItem *)yyyRSTopNp)->solvedSAlist); 
        if (yyySSALCfreeList == yyyLambdaSSAL) 
           {yyySSALspace[yyyNewSSALC].next = *yyypL; 
            if ((*yyypL = yyyNewSSALC++) == yyySSALspaceSize) 
               yyyHandleOverflow(yyySSALof); 
           }  
           else
           {yyyiDum = yyySSALCfreeList; 
            yyySSALCfreeList = yyySSALspace[yyySSALCfreeList].next; 
            yyySSALspace[yyyiDum].next = *yyypL; 
            *yyypL = yyyiDum;  
           } 
        yyySSALspace[*yyypL].attrbNum = yyywa; 
       } 
   }

} /* yyySolveAndSignal */ 



void yyyDoTraversals(){}
void yyyExecuteRRsection()  {
   int yyyi; 
   long yyynRefCounts; 
   long cycleSum = 0;
   long nNZrc = 0;

   if (!yyyYok) return; 
   yyynRefCounts = yyyNextRCLspace - yyyRefCountListSpace; 
   for (yyyi=0;yyyi<yyynRefCounts;yyyi++) 
     if (yyyRefCountListSpace[yyyi])
        {cycleSum += yyyRefCountListSpace[yyyi]; nNZrc++;} 
   if (nNZrc) 
      {
       fprintf(stderr,"\n\n\n**********\n");
       fprintf(stderr,
               "cycle detected in completed parse tree");
       fprintf(stderr,
               " after decoration.\n");
       fprintf(stderr,
         "searching parse tree for %d unsolved instances:\n",
               nNZrc
              );
       yyyUnsolvedInstSearchTravAux(yyyStackTop->node);
      }
   yyyDoTraversals();
} /* yyyExecuteRRsection */ 



yyyWAT yyyLRCIL[4] = {0,0,0,0,
};



void yyyYoxInit()                                  /*stock*/  
  { 

   yyyNodeSizeCalc(); 

   if ((yyyTermBuffer.snBufPtr = 
        (char *) malloc((size_t)(yyyBiggestNodeSize + sizeof(yyyCopyType)))
       )  
       == 
       ((char *) NULL) 
      )   
      yyyfatal("malloc error in yyyTermBuffer allocation\n");  
  
  
   yyyNbytesStackStg = yyyMaxStackSize*sizeof(struct yyyStackItem); 
   yyyNbytesStackStg = ((yyyNbytesStackStg/yyyAlignSize)+1)*yyyAlignSize;  
   if ((yyyNodeAndStackSpace = 
        (char *) malloc((size_t)(yyyNbytesStackStg + 
                                 yyyMaxNbytesNodeStg + 
                                 yyyGNSz + 
                                 yyyBiggestNodeSize + 
                                 sizeof(yyyCopyType) 
                                )
                       )
       )  
       == 
       ((char *) NULL) 
      )   
      yyyfatal("malloc error in ox node and stack space allocation\n");
   yyyStack = (struct yyyStackItem *) yyyNodeAndStackSpace; 
   yyyAfterStack = yyyStack + yyyMaxStackSize;  
   yyyNodeSpace = yyyNodeAndStackSpace + yyyNbytesStackStg;
   yyyAfterNodeSpace = yyyNodeSpace + yyyMaxNbytesNodeStg;
 
 
   if ((yyyRS = (struct yyyRSitem *) 
         malloc((size_t)((yyyRSmaxSize+1)*sizeof(struct yyyRSitem)))
       )  
       == 
       ((struct yyyRSitem *) NULL) 
      )   
      yyyfatal("malloc error in ox ready set space allocation\n");  
   yyyRS++; 
   yyyAfterRS = yyyRS + yyyRSmaxSize; 

 
   if ((yyyChildListSpace = 
        (yyyGNT **) malloc((size_t)(yyyMaxNchildren*sizeof(yyyGNT *)))
       )  
       == 
       ((yyyGNT **) NULL) 
      )   
      yyyfatal("malloc error in ox child list space allocation\n");  
   yyyAfterChildListSpace = yyyChildListSpace + yyyMaxNchildren; 

 
   if ((yyyRefCountListSpace = 
        (yyyRCT *) malloc((size_t)(yyyMaxNrefCounts * sizeof(yyyRCT)))
       )  
       == 
       ((yyyRCT *) NULL) 
      )   
      yyyfatal("malloc error in ox reference count list space allocation\n");  
   yyyAfterRefCountListSpace = yyyRefCountListSpace + yyyMaxNrefCounts;  
  
 
   if ((yyySSALspace = (struct yyySolvedSAlistCell *) 
          malloc((size_t)((yyySSALspaceSize+1) * 
                          sizeof(struct yyySolvedSAlistCell))
                         ) 
       ) 
       == 
       ((struct yyySolvedSAlistCell *) NULL) 
      ) 
      yyyfatal("malloc error in stack solved list space allocation\n"); 
  } /* yyyYoxInit */ 



void yyyYoxReset() 
  { 
   yyyTermBuffer.isEmpty = 1; 
   yyyStackTop = yyyStack; 
   while (yyyStackTop != yyyAfterStack) 
     (yyyStackTop++)->solvedSAlist = yyyLambdaSSAL; 
   yyyStackTop = yyyStack - 1; 
   yyyNextNodeSpace = yyyNodeSpace; 
   yyyRSTop = yyyRS - 1; 
   yyyNextCLspace = yyyChildListSpace;
   yyyNextRCLspace = yyyRefCountListSpace; 
  }  



void yyyDecorate() 
  { 
   while (yyyRSTop >= yyyRS) 
      yyySolveAndSignal();  
  } 



void yyyShift() 
  {yyyRCT *rcPdum; 
   register yyyCopyType *CTp1,*CTp2,*CTp3; 
   register yyyWAT *startP,*stopP;  

   if ((++yyyStackTop) == yyyAfterStack) 
      yyyHandleOverflow(yyySof);
   CTp2 = (yyyCopyType *)(yyyStackTop->oldestNode = 
                          yyyStackTop->node = 
                          (yyyGNT *)yyyNextNodeSpace 
                         ); 
   yyyTermBuffer.isEmpty = 1;
   ((yyyGNT *)CTp2)->parent = (void *)yyyStackTop; 
   ((yyyGNT *)CTp2)->cL = yyyNextCLspace;  
   rcPdum = ((yyyGNT *)CTp2)->refCountList = yyyNextRCLspace;  
   ((yyyGNT *)CTp2)->prodNum = 0; 
   if ((yyyNextRCLspace += yyyTermBuffer.nAttrbs) 
       > 
       yyyAfterRefCountListSpace 
      ) 
      yyyHandleOverflow(yyyRCof); 
   startP = yyyTermBuffer.startP;  
   stopP = yyyTermBuffer.stopP;  
   while (startP < stopP) rcPdum[*(startP++)] = 0; 
   if ((yyyNextNodeSpace += yyyNdPrSz[yyyTermBuffer.typeNum]) 
       > 
       yyyAfterNodeSpace 
      ) 
      yyyHandleOverflow(yyyNSof);  
   CTp1 = (yyyCopyType *)(yyyTermBuffer.snBufPtr); 
   CTp2 = (yyyCopyType *)(((char *)CTp2) + yyyGNSz); 
   CTp3 = CTp2 + yyyNdCopySz[yyyTermBuffer.typeNum]; 
   while (CTp2 < CTp3) *CTp2++ = *CTp1++; 
  } 



void yyyGenIntNode() 
  {register yyyWST i;
   register struct yyyStackItem *stDum;  
   register yyyGNT *gnpDum; 

   if ((stDum = (yyyStackTop -= (yyyRHSlength-1))) >= yyyAfterStack) 
      yyyHandleOverflow(yyySof);
   yyySTsn = ((char *)(yyySTN = (yyyGNT *)yyyNextNodeSpace)) + yyyGNSz; 
   yyySTN->parent       =  (void *)yyyStackTop;  
   yyySTN->cL           =  yyyNextCLspace; 
   yyySTN->refCountList =  yyyNextRCLspace; 
   yyySTN->prodNum      =  yyyProdNum; 
   if ((yyyNextCLspace+yyyRHSlength) > yyyAfterChildListSpace) 
      yyyHandleOverflow(yyyCLof); 
   for (i=1;i<=yyyRHSlength;i++) 
     {gnpDum = *(yyyNextCLspace++) = (stDum++)->node;  
      gnpDum->whichSym = i;  
      gnpDum->parent = (void *)yyyNextNodeSpace; 
     } 
   if ((yyyNextRCLspace += yyyNattrbs) > yyyAfterRefCountListSpace) 
      yyyHandleOverflow(yyyRCof); 
   if ((yyyNextNodeSpace += yyyNdPrSz[yyyTypeNum]) > yyyAfterNodeSpace) 
      yyyHandleOverflow(yyyNSof);  
  } 



#define yyyDECORfREQ 50 



void yyyAdjustINRC(register yyyFT *startP, register yyyFT *stopP) 
  {yyyWST i;
   long SSALptr,SSALptrHead,*cPtrPtr; 
   long *pL; 
   struct yyyStackItem *stDum;  
   yyyGNT *gnpDum; 
   long iTemp;
   register yyyFT *nextP;
   static unsigned short intNodeCount = yyyDECORfREQ;

   nextP = startP;
   while (nextP < stopP) 
     {if ((*nextP) == yyyR)  
         {(yyySTN->refCountList)[*(nextP+1)] = *(nextP+2);
         } 
         else 
         {(((yyySTN->cL)[*nextP])->refCountList)[*(nextP+1)] = *(nextP+2);
         } 
      nextP += 3;  
     }
   pL = yyyIIEL + yyyIIIEL[yyyProdNum]; 
   stDum = yyyStackTop;  
   for (i=1;i<=yyyRHSlength;i++) 
     {pL++; 
      SSALptrHead = SSALptr = *(cPtrPtr = &((stDum++)->solvedSAlist)); 
      if (SSALptr != yyyLambdaSSAL) 
         {*cPtrPtr = yyyLambdaSSAL; 
          do 
            {
             iTemp = (*pL+yyySSALspace[SSALptr].attrbNum);
             yyySignalEnts(yyySTN,
                           yyyEntL + yyyIEL[iTemp],
                           yyyEntL + yyyIEL[iTemp+1]
                          );  
             SSALptr = *(cPtrPtr = &(yyySSALspace[SSALptr].next)); 
            } 
            while (SSALptr != yyyLambdaSSAL);  
          *cPtrPtr = yyySSALCfreeList;  
          yyySSALCfreeList = SSALptrHead;  
         } 
     } 
   nextP = startP + 2;
   while (nextP < stopP) 
     {if (!(*nextP))
         {if ((*(nextP-2)) == yyyR)  
             {pL = &(yyyStackTop->solvedSAlist); 
              if (yyySSALCfreeList == yyyLambdaSSAL) 
                 {yyySSALspace[yyyNewSSALC].next = *pL; 
                  if ((*pL = yyyNewSSALC++) == yyySSALspaceSize) 
                     yyyHandleOverflow(yyySSALof); 
                 }  
                 else
                 {iTemp = yyySSALCfreeList; 
                  yyySSALCfreeList = yyySSALspace[yyySSALCfreeList].next; 
                  yyySSALspace[iTemp].next = *pL; 
                  *pL = iTemp;  
                 } 
              yyySSALspace[*pL].attrbNum = *(nextP-1); 
             } 
             else 
             {if ((gnpDum = (yyySTN->cL)[*(nextP-2)])->prodNum != 0)
                 {
                  iTemp = yyyIIEL[yyyIIIEL[gnpDum->prodNum]] + *(nextP-1);
                  yyySignalEnts(gnpDum, 
                                yyyEntL + yyyIEL[iTemp],  
                                yyyEntL + yyyIEL[iTemp+1] 
                               );    
                 }  
             } 
         } 
      nextP += 3; 
     } 
   yyyStackTop->node = yyySTN;
   if (!yyyRHSlength) yyyStackTop->oldestNode = yyySTN; 
   if (!--intNodeCount) 
      {intNodeCount = yyyDECORfREQ; 
       yyyDecorate(); 
      } 
  } 



void yyyPrune(long prodNum) 
  {  
   int i,n; 
   register char *cp1,*cp2;  
   register yyyRCT *rcp1,*rcp2,*rcp3;  
   long cycleSum = 0;
   long nNZrc = 0;
   yyyRCT *tempNextRCLspace;
   
   yyyDecorate();
   tempNextRCLspace = yyyNextRCLspace;
   yyyNextRCLspace = 
     (rcp1 = rcp2 = (yyyStackTop->oldestNode)->refCountList) + yyyNattrbs;
   rcp3 = (yyyStackTop->node)->refCountList; 
   while (rcp2 < rcp3) 
     if (*rcp2++) {cycleSum += *(rcp2 - 1); nNZrc++;} 
   if (nNZrc) 
      {
       fprintf(stderr,"\n\n\n----------\n");
       fprintf(stderr,
         "cycle detected during pruning of a subtree\n");
       fprintf(stderr,
         "  at whose root production %d is applied.\n",prodNum);
       yyyNextRCLspace = tempNextRCLspace; 
       fprintf(stderr,
         "prune aborted: searching subtree for %d unsolved instances:\n",
               nNZrc
              );
       yyyUnsolvedInstSearchTrav(yyyStackTop->node);
       return; 
      }
   for (i=0;i<yyyNattrbs;i++) rcp1[i] = rcp3[i]; 
   yyyNextCLspace = (yyyStackTop->oldestNode)->cL; 
   yyyNextNodeSpace = (char *)(yyyStackTop->oldestNode) + 
                      (n = yyyNdPrSz[yyyTypeNum]);
   cp1 = (char *)yyyStackTop->oldestNode; 
   cp2 = (char *)yyyStackTop->node; 
   for (i=0;i<n;i++) *cp1++ = *cp2++; 
   yyyStackTop->node = yyyStackTop->oldestNode; 
   (yyyStackTop->node)->refCountList = rcp1; 
   (yyyStackTop->node)->cL = yyyNextCLspace; 
  } 



void yyyGenLeaf(int nAttrbs,int typeNum,yyyWAT *startP,yyyWAT *stopP) 
  {
   if  (!(yyyTermBuffer.isEmpty)) yyyShift(); 
   yyyTermBuffer.isEmpty = 0;
   yyyTermBuffer.typeNum = typeNum; 
   yyyTermBuffer.nAttrbs = nAttrbs; 
   yyyTermBuffer.startP = startP; 
   yyyTermBuffer.stopP = stopP; 
   
  } 



void yyyerror()
  {yyyYok = 0; 
  } 



/* read the command line for changes in sizes of 
                  the evaluator's data structures */
void yyyCheckForResizes(int argc,char *argv[]) 
  {int i; 
   long dum; 
 
   if (!yyyPermitUserAlloc) return; 
   for (i=1;i<argc;i++) 
     { 
      if ((argv[i][0] != '-') || (argv[i][1] != 'Y')) continue; 
      if (strlen(argv[i]) < 4) goto yyyErrO1; 
      if (sscanf(argv[i]+3,"%d",&dum) != 1) goto yyyErrO1;
      if (dum < 2) dum = 2;
      switch (argv[i][2]) 
        {case yyyNSof:   yyyMaxNbytesNodeStg = dum; break; 
         case yyyRCof:   yyyMaxNrefCounts    = dum; break; 
         case yyyCLof:   yyyMaxNchildren     = dum; break; 
         case yyySof:    yyyMaxStackSize     = dum; break; 
         case yyySSALof: yyySSALspaceSize    = dum; break; 
         case yyyRSof:   yyyRSmaxSize        = dum; break; 
         case yyyTSof:   yyyTravStackMaxSize = dum; break; 
         default : goto yyyErrO1; 
        }
      continue;  
   yyyErrO1 : fprintf(stderr,"invalid command line option: %s\n",
                             argv[i] 
                     ); 
     } 
  } 
   
   
   


#define yyyLastProdNum 67


#define yyyNsorts 13


int yyyProdsInd[] = {
   0,
   0,   3,   6,   7,   9,  11,  13,  15,  17,  19,
  23,  26,  31,  36,  40,  44,  48,  52,  56,  60,
  64,  68,  72,  76,  80,  83,  87,  91,  96, 100,
 104, 108, 112, 117, 122, 127, 132, 137, 141, 146,
 155, 164, 172, 177, 179, 181, 188, 192, 198, 202,
 204, 207, 208, 213, 218, 219, 225, 229, 231, 233,
 235, 236, 239, 244, 248, 254, 258,
 264,
};


int yyyProds[][2] = {
{  84,   6},{ 694,   0},{  82,   7},{ 694,   0},{ 471,   0},
{ 694,   0},{ 694,   0},{  82,   7},{ 596,   1},{  82,   7},
{ 657,   2},{  82,   7},{ 579,   3},{  82,   7},{ 605,   0},
{  82,   7},{ 954,   0},{  82,   7},{ 805,   0},{  82,   7},
{ 396,   0},{  82,   7},{ 404,   0},{  82,   7},{ 436,   0},
{  82,   7},{  82,   7},{1132,   0},{ 396,   0},{  82,   7},
{ 404,   0},{  82,   7},{ 984,   0},{ 396,   0},{  82,   7},
{ 404,   0},{  82,   7},{  82,   7},{ 420,   0},{  82,   7},
{  82,   7},{  82,   7},{ 436,   0},{  82,   7},{  82,   7},
{  82,   7},{ 412,   0},{  82,   7},{  82,   7},{  82,   7},
{ 452,   0},{  82,   7},{  82,   7},{  82,   7},{ 140,   0},
{  82,   7},{  82,   7},{  82,   7},{ 564,   0},{  82,   7},
{  82,   7},{  82,   7},{ 632,   0},{  82,   7},{  82,   7},
{  82,   7},{ 556,   0},{  82,   7},{  82,   7},{  82,   7},
{ 630,   0},{  82,   7},{  82,   7},{  82,   7},{ 572,   0},
{  82,   7},{  82,   7},{  82,   7},{ 625,   0},{  82,   7},
{  82,   7},{1165,   0},{  82,   7},{  82,   7},{  82,   7},
{ 120,   0},{  82,   7},{  82,   7},{  82,   7},{ 737,   0},
{  82,   7},{  82,   7},{ 382,   0},{ 396,   0},{  82,   7},
{ 404,   0},{  82,   7},{  82,   7},{ 607,   0},{  82,   7},
{  82,   7},{  82,   7},{ 994,   0},{  82,   7},{  82,   7},
{  82,   7},{ 337,   0},{  82,   7},{  82,   7},{  82,   7},
{ 896,   0},{  82,   7},{  82,   7},{ 469,   0},{ 396,   0},
{  82,   7},{ 404,   0},{  82,   7},{ 770,   0},{ 396,   0},
{  82,   7},{ 404,   0},{  82,   7},{ 732,   0},{ 396,   0},
{  82,   7},{ 404,   0},{  82,   7},{  77,   0},{ 396,   0},
{  82,   7},{ 404,   0},{  82,   7},{ 376,   0},{ 396,   0},
{  82,   7},{ 404,   0},{  82,   7},{  82,   7},{ 739,   0},
{ 605,   0},{  82,   7},{  82,   7},{ 739,   0},{1165,   0},
{ 605,   0},{  82,   7},{ 168,   0},{ 387,   5},{ 896,   0},
{  82,   7},{ 540,   0},{ 396,   0},{  82,   7},{ 404,   0},
{  82,   7},{1091,   0},{ 387,   5},{ 896,   0},{  82,   7},
{ 540,   0},{ 396,   0},{  82,   7},{ 404,   0},{  82,   7},
{ 929,   0},{ 440,  13},{  82,   7},{ 559,   0},{ 497,   8},
{ 453,  10},{ 524,   9},{  82,   7},{1049,   0},{ 396,   0},
{1163,   7},{ 404,   0},{  82,   7},{ 387,   5},{  82,   7},
{ 308,   7},{ 308,   7},{  82,   7},{1155,   0},{ 387,   5},
{ 396,   0},{ 467,   9},{ 404,   0},{ 308,   7},{  82,   7},
{1155,   0},{ 387,   5},{ 308,   7},{  82,   7},{1155,   0},
{ 387,   5},{ 396,   0},{ 404,   0},{ 467,   9},{ 467,   9},
{ 428,   0},{  82,   7},{ 467,   9},{  82,   7},{ 453,  10},
{ 290,   0},{  82,   7},{ 453,  10},{ 524,   9},{ 693,   0},
{ 780,   0},{ 723,  11},{ 785,  12},{ 524,   9},{ 693,   0},
{ 780,   0},{ 387,   5},{ 785,  12},{ 524,   9},{ 723,  11},
{ 723,  11},{ 428,   0},{ 387,   5},{ 396,   0},{ 404,   0},
{ 723,  11},{ 387,   5},{ 396,   0},{ 404,   0},{ 785,  12},
{  96,   0},{ 785,  12},{ 631,   0},{ 440,  13},{ 948,   0},
{ 440,  13},{ 497,   8},{  82,   7},{ 387,   5},{ 497,   8},
{ 497,   8},{ 428,   0},{  82,   7},{ 387,   5},{ 497,   8},
{ 387,   5},{ 896,   0},{  82,   7},{ 497,   8},{ 497,   8},
{ 428,   0},{ 387,   5},{ 896,   0},{  82,   7},{1163,   7},
{ 387,   5},{ 540,   0},{  82,   7},{1163,   7},{1163,   7},
{ 428,   0},{ 387,   5},{ 540,   0},{  82,   7},
};


int yyySortsInd[] = {
  0,
  0,  1,  2,  3,  4,  5,  6,  9, 12, 14,
 16, 17, 18,
 19,
};


int yyySorts[] = {
  423, 1180, 1180, 1180, 1180,  899,  917,   52,  777,  917,
  777,  783,  917,  777,   52,  777,  917,   52,  423,
};



char *yyyStringTab[] = {
0,0,0,"EqPred",0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,"OrPred",0,0,0,
0,0,"IsNullPred",0,0,
"IsNotNullPred",0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,"pred",0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,"MAX",0,0,
0,0,"Q",0,"S",
"FID",0,0,0,0,
0,0,0,0,0,
0,"ASC",0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
"AND",0,"y",0,0,
0,0,0,0,0,
0,0,"GROUP",0,0,
0,0,0,0,0,
"MOD",0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,"FORALL",0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,"NULL",0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
"WHERE",0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,"PExp",0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,"EXCEPT",0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,"QEnvFunc2",0,0,0,
0,0,0,0,0,
0,"AVG",0,0,0,
0,0,"FLATTEN","Predicate",0,
0,0,"IDENT",0,0,
0,0,0,0,0,
"LIKE","'('",0,0,0,
0,0,0,0,"')'",
0,0,0,0,0,
0,0,"'*'","lexeme",0,
0,0,0,0,0,
"'+'",0,0,"val",0,
0,0,0,"','",0,
0,0,0,0,0,
0,"'-'",0,0,0,
"DistOpt",0,0,0,0,
0,0,0,0,0,
0,0,"'/'","WhereOpt","PConst",
0,0,0,0,"PLess",
0,0,0,0,0,
0,0,"NEQList",0,"COUNT",
0,"COMMENT",0,0,0,
0,0,0,0,"MAXFunc",
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,"QEnv",0,0,
"QEnvEnv",0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,"QEnvFunc",0,0,0,
0,0,0,0,"OrderOpt",
"SFWFunc",0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
"':'",0,0,"CHARCON","MINFunc",
0,0,0,0,0,
0,0,0,0,0,
0,"'<'",0,0,"FROM",
0,0,0,0,"'='",
0,0,0,0,"atoi",
0,0,"'>'",0,0,
0,"PGreater",0,0,"FLOATCON",
0,0,0,"COUNTFunc",0,
0,0,0,0,0,
0,0,0,0,0,
0,"INTCON",0,0,0,
0,0,0,0,0,
"NIL",0,"UNION",0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,"entry",
"GE",0,0,0,0,
"LE","DESC","NE",0,0,
"AVGFunc",0,0,0,0,
0,0,0,0,0,
"SUMFunc",0,0,0,0,
0,0,0,0,0,
0,0,"STRINGCON",0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
"FUser",0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,"Environment","ORDER","Comment",
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,"MethList",0,
0,0,0,0,0,
0,"AbsFunc","MIN",0,0,
0,0,"OR",0,"IS",
"Path1Func","Path2Func","PairFunc",0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,"IdentFunc",0,0,
0,0,0,0,0,
0,0,0,"NegFunc","NonType",
"SUM",0,0,0,0,
0,0,"ienv",0,0,
"BY",0,"nenv","oenv",0,
"Direcn",0,0,0,0,
"DivFunc","DifFunc",0,0,0,
0,0,0,0,0,
"NilFunc","MinusFunc",0,0,0,
"FALSEV",0,0,0,0,
0,0,0,0,0,
0,0,0,"FloatFunc","ElementFunc",
"FlattenFunc",0,0,0,0,
0,0,"PlusFunc",0,0,
0,0,0,0,0,
"IntFunc",0,"InterFunc","CopyFunc",0,
0,0,0,0,0,
0,0,"ModFunc","UniFunc",0,
0,0,0,0,0,
"Sort1Func","Sort2Func",0,0,0,
0,0,0,0,0,
0,0,"result",0,0,
0,0,0,0,"Function",
0,0,0,0,"ERROR",
0,0,0,0,"QuanEnv",
"GEQPred",0,0,0,0,
"LEQPred",0,0,"StrFunc",0,
0,"INP","MultFunc",0,"res",
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,"func",0,0,
0,0,0,0,0,
0,0,0,0,"SELECT",
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,"DISTINCT",0,
0,0,0,0,"TRUEV",
"SRule",0,0,0,0,
0,0,0,0,0,
0,0,"FORALLPred",0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,"ABS",
0,0,0,0,0,
0,0,0,0,"INTERSECT",
0,0,0,0,0,
0,0,0,0,"GTPred",
0,0,0,0,"LTPred",
0,0,0,0,0,
0,0,0,0,0,
0,0,0,"yytext",0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,"EXISTSPred",0,
0,0,0,0,0,
0,0,0,0,"STRUCT",
0,0,0,0,0,
0,0,0,0,0,
"'{'",0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,"'}'",0,0,0,
0,0,0,0,0,
0,0,0,0,0,
"SET","EXISTS",0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
"Path1Pred","Path2Pred","BPrimitive",0,0,
0,0,0,0,0,
0,0,"ELT",0,0,
0,0,"IdentPred",0,0,
0,0,0,0,0,
"add",0,0,"NeqPred",0,
0,"new",0,0,0,
"DOT",0,0,0,0,
0,0,0,"DPrL",0,
"NOT",0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
"string",0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,0,0,0,
0,0,"AndPred",0,0,
0,0,0,0,"InPred",
0,0,0,"CopyPred",0,
0,0,0,0,0,
0,0,0,"NotPred",0,
0,
};



#define yyySizeofProd(num) (yyyProdsInd[(num)+1] - yyyProdsInd[(num)])

#define yyyGSoccurStr(prodNum,symPos) \
   (yyyStringTab[yyyProds[yyyProdsInd[(prodNum)] + (symPos)][0]])

#define yyySizeofSort(num) (yyySortsInd[(num)+1] - yyySortsInd[(num)])

#define yyySortOf(prodNum,symPos) \
  (yyyProds[yyyProdsInd[(prodNum)] + (symPos)][1]) 

#define yyyAttrbStr(prodNum,symPos,attrbNum)                      \
  (yyyStringTab[yyySorts[yyySortsInd[yyySortOf(prodNum,symPos)] + \
                         (attrbNum)                               \
                        ]                                         \
               ]                                                  \
  )



void yyyShowProd(int i)
  {int j,nSyms;

   nSyms = yyySizeofProd(i);
   for (j=0; j<nSyms; j++)
     {
      fprintf(stderr,"%s",yyyGSoccurStr(i,j));
      if (j == 0) fprintf(stderr," : "); else fprintf(stderr," ");
     }
   fprintf(stderr,";\n");
  }



void yyyShowProds()
  {int i; for (i=1; i<=yyyLastProdNum; i++) yyyShowProd(i);}



void yyyShowSymsAndSorts()
  {int i; 

   for (i=1; i<=yyyLastProdNum; i++) 
     {int j, nSyms;

      fprintf(stderr,
              "\n\n\n---------------------------------- %3.1d\n",i);
      /* yyyShowProd(i); */ 
      nSyms = yyySizeofProd(i); 
      for (j=0; j<nSyms; j++) 
        {int k, sortSize;

         fprintf(stderr,"%s\n",yyyGSoccurStr(i,j));
         sortSize = yyySizeofSort(yyySortOf(i,j));
         for (k=0; k<sortSize; k++) 
            fprintf(stderr,"  %s\n",yyyAttrbStr(i,j,k));
         if (j == 0) fprintf(stderr,"->\n"); 
              else 
              fprintf(stderr,"\n"); 
        }
     }
  }



void yyyCheckNodeInstancesSolved(yyyGNT *np)
  {int mysort,sortSize,i,prodNum,symPos,inTerminalNode;
   int nUnsolvedInsts = 0;

   if (np->prodNum != 0)
     {inTerminalNode = 0;
      prodNum = np->prodNum;
      symPos = 0;
     }
   else
     {inTerminalNode = 1;
      prodNum = ((yyyGNT *)(np->parent))->prodNum;
      symPos = np->whichSym;
     }
   mysort = yyySortOf(prodNum,symPos);
   sortSize = yyySizeofSort(mysort);
   for (i=0; i<sortSize; i++)
     if ((np->refCountList)[i] != 0) nUnsolvedInsts += 1;
   if (nUnsolvedInsts)
     {fprintf(stderr,
      "\nFound node that has %d unsolved attribute instance(s).\n",
              nUnsolvedInsts
             );
      fprintf(stderr,"Node is labeled \"%s\".\n",
             yyyGSoccurStr(prodNum,symPos));
      if (inTerminalNode)
        {fprintf(stderr,
                 "Node is terminal.  Its parent production is:\n  ");
         yyyShowProd(prodNum);
        }
      else
        {fprintf(stderr,"Node is nonterminal.  ");
         if (((char *)(np->parent)) >= yyyNodeSpace)
           {fprintf(stderr,
                    "Node is %dth child in its parent production:\n  ",
                   np->whichSym
                  );
            yyyShowProd(((yyyGNT *)(np->parent))->prodNum);
           }
         fprintf(stderr,
                 "Node is on left hand side of this production:\n  ");
         yyyShowProd(np->prodNum);
        }
      fprintf(stderr,"The following instances are unsolved:\n");
      for (i=0; i<sortSize; i++)
        if ((np->refCountList)[i] != 0)
          fprintf(stderr,"     %-16s still has %1d dependencies.\n",
                  yyyAttrbStr(prodNum,symPos,i),(np->refCountList)[i]);
     }
  }



void yyyUnsolvedInstSearchTravAux(yyyGNT *pNode)
  {yyyGNT **yyyCLpdum;
   int i;
  
   yyyCheckNodeInstancesSolved(pNode); 
   yyyCLpdum = pNode->cL;
   while
     ((yyyCLpdum != yyyNextCLspace) && ((*yyyCLpdum)->parent == pNode))
     {
      yyyUnsolvedInstSearchTravAux(*yyyCLpdum);
      yyyCLpdum++;
     }
  }



void yyyUnsolvedInstSearchTrav(yyyGNT *pNode)
  {yyyGNT **yyyCLpdum;
   int i;
  
   yyyCLpdum = pNode->cL;
   while
     ((yyyCLpdum != yyyNextCLspace) && ((*yyyCLpdum)->parent == pNode))
     {
      yyyUnsolvedInstSearchTravAux(*yyyCLpdum);
      yyyCLpdum++;
     }
  }



